Package: grmtree
Title: Recursive Partitioning for Graded Response Models
Version: 0.2.2
Date: 2026-08-22
Authors@R: c(person(given = "Olayinka I.", family = "Arimoro", role = c("aut", "cre"), email = "olayinka.arimoro@ucalgary.ca", comment = c(ORCID = "0009-0009-9464-589X")),
             person(given = "Tolulope T.", family = "Sajobi", role = "aut", email = "ttsajobi@ucalgary.ca"),
             person(given = "Lisa M.", family = "Lix", role = "aut", email = "Lisa.Lix@umanitoba.ca"),
             person(given = "Matthew T.", family = "James", role = "ctb", email = "mjames@ucalgary.ca"),
             person(given = "Maria", family = "Santana", role = "ctb", email = "mjsantan@ucalgary.ca"),
             person(given = "Emmanuel", family = "Ugochukwu", role = "ctb", email = "favour879@gmail.com"))
Maintainer: Olayinka I. Arimoro <olayinka.arimoro@ucalgary.ca>
Description: Provides methods for recursive partitioning based on the 
  'Graded Response Model' ('GRM'), extending the 'MOB' algorithm from the 
  'partykit' package. The package allows for fitting
  'GRM' trees that partition the population into homogeneous 
  subgroups based on item response patterns and covariates. 
  Includes specialized plotting functions for visualizing 'GRM' trees
  with different terminal node displays (threshold regions, 
  parameter profiles, and factor score distributions). The package also implements the Longitudinal GRMTree for detecting response shift in PROMs measured at two time points, embedding a constrained two-factor longitudinal GRM within recursive partitioning, with post-hoc characterization of recalibration and reprioritization.
  For more details on the methods, see Samejima (1969) <doi:10.1002/J.2333-8504.1968.TB00153.X>, Komboz et al. (2018) <doi:10.1177/0013164416664394> and Arimoro et al. (2025) <doi:10.1007/s11136-025-04018-6>.
License: GPL-3
Depends: R (>= 4.1.0), partykit (>= 1.2-9), mirt (>= 1.46.1)
Imports: stats, graphics, grid, ggplot2, rlang, strucchange, parallel
Suggests: hlt, dplyr, magrittr, testthat (>= 3.0.0), knitr, rmarkdown,
        psychotools, psychotree, psych
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://github.com/Predicare1/grmtree
BugReports: https://github.com/Predicare1/grmtree/issues
Repository: CRAN
VignetteBuilder: knitr
LazyData: true
NeedsCompilation: no
Packaged: 2026-08-22 23:25:45 UTC; Olayinka A
Author: Olayinka I. Arimoro [aut, cre] (ORCID:
    <https://orcid.org/0009-0009-9464-589X>),
  Tolulope T. Sajobi [aut],
  Lisa M. Lix [aut],
  Matthew T. James [ctb],
  Maria Santana [ctb],
  Emmanuel Ugochukwu [ctb]
Date/Publication: 2026-08-23 00:50:02 UTC
Built: R 4.5.2; ; 2026-08-23 04:12:46 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(c,grmforest)
S3method(plot,grmtree)
S3method(plot,longitudinal_grmtree)
S3method(plot,varimp)
S3method(print,grmforest)
S3method(print,grmtree)
S3method(print,rs_characterization)
S3method(print,varimp)
export(discrpar_grmtree)
export(discrpar_longitudinal_grmtree)
export(fscores_grmtree)
export(fscores_longitudinal_grmtree)
export(generate_node_scores_dataset)
export(grmforest)
export(grmforest.control)
export(grmtree)
export(grmtree.control)
export(itempar_grmtree)
export(itempar_longitudinal_grmtree)
export(latentpar_longitudinal_grmtree)
export(longitudinal_grmtree)
export(plot_rs_heatmap)
export(plot_rs_tree)
export(prepare_longitudinal_data)
export(rs_characterize)
export(threshpar_grmtree)
export(threshpar_longitudinal_grmtree)
export(varimp)
import(grid)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(mirt,coef)
importFrom(mirt,estfun.AllModelClass)
importFrom(mirt,extract.mirt)
importFrom(mirt,fscores)
importFrom(mirt,logLik)
importFrom(mirt,mirt)
importFrom(mirt,mirt.model)
importFrom(partykit,data_party)
importFrom(partykit,info_node)
importFrom(partykit,mob)
importFrom(partykit,mob_control)
importFrom(partykit,nodeapply)
importFrom(partykit,nodeids)
importFrom(partykit,nodeprune)
importFrom(rlang,.data)
importFrom(stats,AIC)
importFrom(stats,complete.cases)
importFrom(stats,formula)
importFrom(stats,model.frame)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,predict)
importFrom(strucchange,sctest)

# grmtree: Recursive Partitioning for Graded Response Models

<!-- badges: start -->

[![R-CMD-check](https://github.com/Predicare1/grmtree/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Predicare1/grmtree/actions/workflows/R-CMD-check.yaml) [![CRAN version](https://www.r-pkg.org/badges/version/grmtree)](https://cran.r-project.org/package=grmtree) [![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/grmtree)](https://cran.r-project.org/package=grmtree) [![License: GPL-3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.html) [![Code size](https://img.shields.io/github/languages/code-size/Predicare1/grmtree.svg)](https://github.com/Predicare1/grmtree)

<!-- badges: end -->

## Overview

The `grmtree` package implements advanced tree-based recursive partitioning methods for Graded Response Models (GRM), providing a powerful framework for detecting and analyzing differential item functioning (DIF) in polytomous items from patient-reported outcome measures (PROMs) and other psychological assessments. DIF occurs when individuals with the same underlying latent trait (e.g., health status, quality of life, or psychological attribute) respond differently to assessment items based on extraneous characteristics such as age, gender, education level, or clinical subgroups. This measurement bias can compromise the validity and fairness of assessments across diverse populations.

The GRMTree methodology combines the psychometric rigor of item response theory with the interpretability of decision trees to:

-   Identify specific covariates associated with DIF
-   Detect complex interaction effects between patient characteristics
-   Provide visual representations of how item functioning varies across population subgroups
-   Support the development of equitable assessment instruments

The package also implements the **Longitudinal GRMTree** for detecting **response shift** in PROMs measured at two time points, embedding a constrained two-factor longitudinal GRM within recursive partitioning and characterizing recalibration and reprioritization within data-driven subgroups.

## Key Features

-   **DIF Detection**: Advanced algorithms for detecting differential item functioning in polytomous items using model-based recursive partitioning
-   **Response Shift Detection**: Longitudinal GRMTree for detecting and characterizing response shift (recalibration, reprioritization) in PROMs measured at two time points
-   **Visual Analytics**: Specialized plotting functions for visualizing threshold parameters, discrimination parameters, factor score distributions, and response shift across terminal nodes
-   **Parameter Extraction**: Comprehensive methods for extracting and analyzing item parameters (thresholds, discrimination), factor scores, and node-specific characteristics
-   **Ensemble Methods**: GRM Forests implementation for robust variable importance analysis and enhanced DIF detection stability
-   **Multiple Testing Corrections**: Flexible p-value adjustment methods (Bonferroni, Holm, Benjamini-Hochberg, etc.) to control Type I error inflation
-   **Seamless Integration**: Full compatibility with the `mirt` package for GRM estimation and the `partykit` ecosystem for tree visualization

This package is particularly valuable for researchers, psychometricians, and health outcomes specialists who require robust methods for ensuring measurement invariance and equity in their assessment instruments across diverse populations.

## Installation

Install from CRAN (when available):

```{r}
install.packages("grmtree")
```

Install the development version from GitHub

```{r}
# Install devtools if not already installed
install.packages("devtools")

# Install the grmtree
devtools::install_github("Predicare1/grmtree")

# Install with vignettes (takes longer but includes vignettes)
devtools::install_github("Predicare1/grmtree", build_vignettes = TRUE)
```

## Quick Start

```{r}
# Load the package
library(grmtree)

# Load the data
data("grmtree_data") # Sample dataset included with package

# Prepare the data
resp.data <- grmtree_data %>% 
  mutate_at(vars(starts_with("MOS")), as.ordered) %>% 
  mutate_at(vars(c(sex, Education)), as.factor) 

# Create response as outcomes
resp.data$resp <- data.matrix(resp.data[, 1:8])

## GRMTree control parameters with Benjamini-Hochberg
grm_control <- grmtree.control(
  minbucket = 350,
  p_adjust = "BH", alpha = 0.05)

# Fit a GRM tree
tree <- grmtree(resp ~ sex + age + Education,
                       data = resp.data,
                       control = grm_control)

# Print the GRMTree model
print(tree)

# Visualize the tree
plot(tree) # Default regions plot
plot(tree, type = "profile") # Parameter profiles
plot(tree, type = "histogram") # Factor score distributions

# Extract parameters
threshpar_grmtree(tree) # Threshold parameters
discrpar_grmtree(tree) # Discrimination parameters
```

## Longitudinal GRMTree: Response Shift Detection

The Longitudinal GRMTree detects response shift in PROMs measured at two time
points, using a two-phase approach: Phase 1 identifies patient subgroups whose
longitudinal measurement model differs, and Phase 2 tests for response shift
within each subgroup and classifies the affected items.

```{r}
# Load the synthetic longitudinal dataset
data("grmtree_long_data")

# Prepare the wide-format response matrix
items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
              "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")

ld <- prepare_longitudinal_data(
  data       = grmtree_long_data,
  items_t1   = items_t1,
  items_t2   = paste0(items_t1, "_year1"),
  covariates = c("sex", "age", "residency", "job",
                 "education", "comorbidity_count", "ever_smoker")
)

# Phase 1: fit the tree to identify subgroups
tree <- longitudinal_grmtree(
  resp_wide ~ sex + age + residency + job +
    education + comorbidity_count + ever_smoker,
  data = ld, n_items = 8,
  control = grmtree.control(minbucket = 200)
)
print(tree)

# Phase 2: characterize response shift within each subgroup
rs <- rs_characterize(tree, p_adjust = "fdr", global_p_adjust = "bonferroni")
print(rs)

# Visualize response shift
plot_rs_tree(tree, rs)
plot_rs_heatmap(rs)
```

## Vignettes

Detailed interactive tutorials are available:

```{r}
browseVignettes("grmtree")
```

1.  Getting Started with the grmtree Package: Basic GRM tree implementation
2.  GRM Forests for Robust DIF Detection: Ensemble methods for robust DIF detection
3.  Response Shift Detection with the Longitudinal GRMTree: Two-phase detection and characterization of response shift in longitudinal PROMs

## GRM Forests Example

```{r}
# Fit a forest with 100 trees
forest <- grmforest(resp ~ sex + age + Education,
                       data = resp.data,
                   ctrl = grmforest.control(control = grm_control, n_tree = 100))

# Variable importance
vim <- varimp(forest)
print(vim)
plot(vim)

# Examine individual trees
plot(forest$trees[[1]]) # First tree
```

# References

## Methodological Foundations

-   Samejima, F. (1969). Estimation of latent ability using a response pattern of graded scores. Psychometrika Monograph Supplement, 34, 100-114.
-   Strobl, C., Kopf, J., & Zeileis, A. (2015). Rasch trees: A new method for detecting differential item functioning in the Rasch model. Psychometrika, 80(2), 289-316.
-   Komboz, B., Strobl, C., & Zeileis, A. (2018). Tree-based global model tests for polytomous Rasch models. Educational and psychological measurement, 78(1), 128–166. doi.org/10.1177/0013164416664394.
-   Oort, F. J. (2005). Using structural equation modeling to detect response shifts and true change. Quality of Life Research, 14(3), 587–598. <https://doi.org/10.1007/s11136-004-0830-y>.
-   Arimoro, O. I., Lix, L. M., Patten, S. B., Sawatzky, R., Sebille, V., Liu, J., Wiebe, S., Josephson, C. B., & Sajobi, T. T. (2025). Tree-based latent variable model for assessing differential item functioning in patient-reported outcome measures: a simulation study. Quality of life research : an international journal of quality of life aspects of treatment, care and rehabilitation, 10.1007/s11136-025-04018-6. Advance online publication. <https://doi.org/10.1007/s11136-025-04018-6>.

## Applied Examples

-   Arimoro, O. I., Josephson, C. B., James, M. T., Patten, S. B., Wiebe, S., Lix, L. M., & Sajobi, T. T. (2024). Screening for depression in patients with epilepsy: same questions but different meaning to different patients. Quality of life research : an international journal of quality of life aspects of treatment, care and rehabilitation, 33(12), 3409–3419. <https://doi.org/10.1007/s11136-024-03782-1>.

# Authors

Olayinka Imisioluwa Arimoro ([olayinka.arimoro\@ucalgary.ca](mailto:olayinka.arimoro@ucalgary.ca){.email}), Lisa M. Lix, Tolulope T. Sajobi

# Contributing

Contributions are welcome! Please submit issues and pull requests via GitHub: <https://github.com/Predicare1/grmtree/issues>

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----eval=F-------------------------------------------------------------------
# ## Install packages from CRAN repository
# install.packages(c("dplyr", "grmtree", "mirt", "psych"))

## ----message=FALSE, warning=FALSE---------------------------------------------
library(dplyr)        # For data manipulation
library(grmtree)      # For tree-based GRM DIF Test
library(mirt)         # For traditional GRM
library(psych)        # For psychometrics

## ----message=FALSE------------------------------------------------------------
## Load the data
data("grmtree_data", package = "grmtree")

## Take a glimpse at the data
glimpse(grmtree_data)

## ----warning=FALSE------------------------------------------------------------
## Create the response data (the 8 MOS-SS items)
response_data <- grmtree_data %>% 
  dplyr::select(MOS_Listen:MOS_Understand) %>% 
  mutate_at(vars(starts_with("MOS")), as.ordered)

## Create response as outcomes
response_data$resp <- data.matrix(response_data[, 1:8])

## ----eval=FALSE---------------------------------------------------------------
# ## Calculate the polychoric correlation of items
# polycorr_mos <- polychoric(response_data$resp, global=FALSE)$rho
# 
# ## Return the eigen values
# polycorr_eigen_mos <- eigen(polycorr_mos)$values
# round(polycorr_eigen_mos, 3)
# 
# ## Ratio of first and second eigen value
# mos_ratio <- round(round(polycorr_eigen_mos, 3)[1]/round(polycorr_eigen_mos, 3)[2],3)
# 
# ## Print the result
# mos_ratio
# 
# cat("The ratio of the first-to-second eigen value is", mos_ratio,
#     "which is >4 suggesting that the unidimensionality assumption is satisfied", "\n")

## ----eval=FALSE---------------------------------------------------------------
# ## Scree plot of eigen values
# plot(1:length(polycorr_eigen_mos), polycorr_eigen_mos,
#      type = "b", pch = 20, xlab = "", ylab = "Eigen values")

## ----eval=FALSE---------------------------------------------------------------
# ## Perform EFA with 1 factors
# efa_mos <- fa(response_data$resp, nfactors = 1, rotate = "varimax", fm = "mle")
# 
# ## Print the results
# summary(efa_mos)
# print(efa_mos, sort=TRUE)
# efa_mos$loadings
# 
# ## Visualizing factor structure using fa.diagram
# fa.diagram(efa_mos, main = "EFA Factor Structure")
# 
# ## Scree plot to visualize the number of factors
# fa.parallel(response_data$resp, fa = "fa")

## -----------------------------------------------------------------------------
## Create GRM
mos_grm <- mirt(data = response_data$resp, 
                model = 1,
                itemtype = "graded", SE = TRUE, method = "EM",
                verbose = FALSE)

## -----------------------------------------------------------------------------
## Get the coefficients
mos_coef <- coef(mos_grm, IRTpars = T, simplify = TRUE)
mos_coef

## Get the residuals
residuals(mos_grm, type = "Q3")

## Compute the M2 model fit statistic
M2(mos_grm, type = "C2")

## Infit and outfit statistics
mirt::itemfit(mos_grm, c('S_X2', 'infit'), method = 'EAP')

## Could also use method = 'ML'

## ----eval = FALSE-------------------------------------------------------------
# ## Plot the category response curves
# plot(mos_grm, type = 'trace')
# 
# ## Plot the test information
# plot(mos_grm, type = 'info')

## ----grmtree-process, echo=FALSE, out.width="75%", fig.align='center', fig.cap="GRMTree implementation process showing the four-step recursive partitioning approach for detecting differential item functioning"----
knitr::include_graphics("GRMTree.png")

## ----warning=FALSE------------------------------------------------------------

## Prepare the data
resp.data <- grmtree_data %>% 
  mutate_at(vars(starts_with("MOS")), as.ordered) %>% 
  mutate_at(vars(c(sex, residency, depressed,
                   Education, job, smoker,
                   multimorbidity)), as.factor) 

## Explore the data
head(resp.data)

## Check the structure of the data
glimpse(resp.data)

## Create response as outcomes
resp.data$resp <- data.matrix(resp.data[, 1:8])

## GRMTree control parameters with Benjamini-Hochberg
grm_control <- grmtree.control(
  minbucket = 350,
  p_adjust = "BH", alpha = 0.05)


## Fit the GRMTree model
mos_grmtree <- grmtree(resp ~ sex + age + bmi + Education + depressed +
                       residency + job + multimorbidity + smoker,  
                       data = resp.data,
                       control = grm_control)

## ----eval=FALSE---------------------------------------------------------------
# ## Bonferroni correction
# tree_bonf <- grmtree(response ~ covariate1 + covariate2, data = df,
#                     control = grmtree.control(p_adjust = "bonferroni"))
# 
# ## Hommel
# tree_bh <- grmtree(response ~ covariate1 + covariate2, data = df,
#                   control = grmtree.control(p_adjust = "hommel"))
# 
# ## Holm-Bonferroni
# tree_holm <- grmtree(response ~ covariate1 + covariate2, data = df,
#                     control = grmtree.control(p_adjust = "holm"))

## -----------------------------------------------------------------------------
## Print the tree model
print(mos_grmtree)
print(mos_grmtree, FUN = function(x) " *")

## -----------------------------------------------------------------------------
## Create the regions plot (by default)
plot(mos_grmtree)

## This also creates a regions plot
plot(mos_grmtree, type = "regions",  tnex = 2L)

## Create the histogram plot of the factor scores
plot(mos_grmtree, type = "histogram",  tnex = 2L)

## Create the profile plot with different options
plot(mos_grmtree, type = "profile", tnex = 2L, what = "threshold")
plot(mos_grmtree, type = "profile", tnex = 2L, what = "discrimination")
plot(mos_grmtree, type = "profile", tnex = 2L, what = "item")

## -----------------------------------------------------------------------------
## Return the names of the covariates to know the position of age
names(mos_grmtree$data)

## Rename the age to Age (Uncomment the code below and change to the correct name)
#names(mos_grmtree$data)[3] <- "Age"

## Create the regions plot (by default)
plot(mos_grmtree)

## -----------------------------------------------------------------------------
## Extract and print the threshold parameters
thresholds <- threshpar_grmtree(mos_grmtree)
print(thresholds)

## Extract and print the discrimination parameters
discriminations <- discrpar_grmtree(mos_grmtree)
print(discriminations)

## Extract and print the item parameters
itemspars <- itempar_grmtree(mos_grmtree)
print(itemspars)

## ----setup, include=FALSE-----------------------------------------------------

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----eval=F-------------------------------------------------------------------
# ## Install packages from CRAN repository
# install.packages(c("dplyr", "grmtree"))

## ----message=FALSE, warning=FALSE---------------------------------------------
library(dplyr)        # For data manipulation
library(grmtree)      # For tree-based GRM DIF Test

## ----message=FALSE------------------------------------------------------------
## Load the data
data("grmtree_data", package = "grmtree")

## Take a glimpse at the data
glimpse(grmtree_data)

## Prepare the data
resp.data <- grmtree_data %>% 
  mutate_at(vars(starts_with("MOS")), as.ordered) %>% 
  mutate_at(vars(c(sex, residency, depressed,
                   Education, job, smoker,
                   multimorbidity)), as.factor) 

## Note: Partitioning variables must be factor/numeric (character columns are coerced automatically, but converting yourself is cleaner).

## Explore the data
head(resp.data)

## Check the structure of the data
glimpse(resp.data)

## Create response as outcomes
resp.data$resp <- data.matrix(resp.data[, 1:8])

## -----------------------------------------------------------------------------
## Get help on the control parameter
# ?grmforest.control

## GRMTree control parameters with Benjamini-Hochberg 
grm_control <- grmtree.control(
  minbucket = 350,
  p_adjust = "BH", alpha = 0.05)

## Define the forest control parameters
p <- 9

forest_control <- grmforest.control(
  n_tree = 3, # Number of trees (Reduced for vignette build time)
  sampling = "subsample",  # subsample method; bootstrap also available
  sample_fraction = 0.632,
  mtry = ceiling(sqrt(p)),  # Usually the square root of the number of covariates
  control = grm_control,
  remove_dead_trees = TRUE, # Remove any null GRMTree
  seed = 123,
  n_cores = 2            # Parallel now works (PSOCK/Windows included)
)

## ----eval=FALSE---------------------------------------------------------------
# ## Fit the GRM forest
# mos_forest <- grmforest(
#   resp ~ sex + age + bmi + Education +
#   residency + depressed + job + multimorbidity + smoker,
#   data = resp.data,
#   control = forest_control
# )
# 
# ## Get the summary of the fitted forest
# print(mos_forest)
# 
# ## Plot a tree in the forest
# plot(mos_forest$trees[[1]])

## ----eval=FALSE---------------------------------------------------------------
# ## Calculate the variable importance
# importance <- varimp(mos_forest, seed = 123, verbose = T)
# 
# ## Print the result of the variable importance
# print(importance)

## ----eval=FALSE---------------------------------------------------------------
# ## Plot the variable importance scores (ggplot is the default)
# plot(importance)
# 
# ## Plot onlt the top 5 importance variables
# plot(importance, xlab = "", top_n = 5)
# 
# ## Plot the base R version
# plot(importance, use_ggplot = FALSE)
# 
# ## Custom colors
# plot(importance, col = c("green", "red"))
# 
# ## Rename the variable names in the order from the variable importance result
# names(importance) <- c("Age", "Smoking Status", "BMI",
#                        "Multimorbidity", "Sex", "Education",
#                        "Residency", "Depression", "Employment")
# 
# ## Now create the plot with informative names
# plot(importance)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----eval=FALSE---------------------------------------------------------------
# ## Install packages from CRAN repository
# install.packages(c("dplyr", "grmtree", "mirt"))

## ----message=FALSE, warning=FALSE---------------------------------------------
library(dplyr)        # For data manipulation
library(grmtree)      # For the Longitudinal GRMTree
library(mirt)         # For the underlying IRT estimation

## ----message=FALSE------------------------------------------------------------
## Load the data
data("grmtree_long_data", package = "grmtree")

## Take a glimpse at the data
glimpse(grmtree_long_data)

## -----------------------------------------------------------------------------
## Baseline item names
items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
              "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")

## Follow-up item names (same order, with the _year1 suffix)
items_t2 <- paste0(items_t1, "_year1")

## Build the wide-format data
ld <- prepare_longitudinal_data(
  data       = grmtree_long_data,
  items_t1   = items_t1,
  items_t2   = items_t2,
  covariates = c("sex", "age", "residency", "job",
                 "education", "comorbidity_count", "ever_smoker")
)

## The response matrix lives in the resp_wide column (16 = 2 x 8 columns)
dim(ld$resp_wide)

## ----eval=FALSE---------------------------------------------------------------
# ## Fit the Longitudinal GRMTree
# tree <- longitudinal_grmtree(
#   resp_wide ~ sex + age + residency + job +
#     education + comorbidity_count + ever_smoker,
#   data    = ld,
#   n_items = 8,
#   control = grmtree.control(minbucket = 200, p_adjust = "bonferroni", alpha = 0.05)
# )
# 
# ## Print the tree structure
# print(tree)

## ----eval=FALSE---------------------------------------------------------------
# ## Phase 2: characterize response shift
# rs <- rs_characterize(
#   tree,
#   p_adjust        = "fdr",          # item-level correction within nodes
#   global_p_adjust = "bonferroni",   # omnibus correction across nodes
#   rs_threshold    = 0.3             # RS-type classification threshold
# )
# 
# ## View the results
# print(rs)

## ----eval=FALSE---------------------------------------------------------------
# rs$global       # one row per terminal node (omnibus results)
# rs$item_level   # one row per item per RS-detected node
# rs$parameters   # constrained and unconstrained item parameters per node

## ----eval=FALSE---------------------------------------------------------------
# ## Short item labels for display
# mos_labels <- c("Listen", "Info", "Crisis", "Confide",
#                 "Advice", "Fears", "Personal", "Understand")
# 
# ## Annotated tree with RS results in the terminal nodes
# plot_rs_tree(tree, rs, item_labels = mos_labels)
# 
# ## Standalone item-level RS heatmap
# plot_rs_heatmap(rs, item_labels = mos_labels)
# 
# ## Heatmap with chi-squared values shown in the cells
# plot_rs_heatmap(rs, item_labels = mos_labels, show_chi2 = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# ## Region plot of the tree with item names
# plot(tree, names = mos_labels, tnex = 2L)

## ----eval=FALSE---------------------------------------------------------------
# ## Threshold parameters (8 items per node)
# threshpar_longitudinal_grmtree(tree)
# 
# ## Discrimination parameters (from the a1 column)
# discrpar_longitudinal_grmtree(tree)
# 
# ## Combined item parameters (discrimination + thresholds)
# itempar_longitudinal_grmtree(tree)
# 
# ## Latent trait summary per node (mu_T2, variance, T1-T2 correlation)
# latentpar_longitudinal_grmtree(tree)

## ----eval=FALSE---------------------------------------------------------------
# ## Factor scores per terminal node
# scores <- fscores_longitudinal_grmtree(tree)
# head(scores[["3"]])   # Theta_T1 and Theta_T2 for the older subgroup

## ----eval=FALSE---------------------------------------------------------------
# ## Augment the original data with node assignments and factor scores
# scored <- generate_node_scores_dataset(tree, data = ld)
# glimpse(scored)   # all original columns + node + Theta_T1 + Theta_T2

#' Compute Latent Factor Scores for Each Terminal Node in a GRM Tree
#'
#' This function calculates latent factor scores for each terminal node in a GRM
#' tree object using specified scoring method (EAP, MAP, ML, or WLE).
#'
#' @param grmtree_obj A GRM tree object (from `grmtree()` function) containing
#'   fitted models in its terminal nodes.
#' @param method Scoring method to use: "EAP" (default), "MAP", "ML", or "WLE".
#'   See `mirt::fscores()` for details.
#'
#' @return A named list where each element contains the factor scores for a
#'   terminal node. Names correspond to node IDs. Returns NULL for nodes where
#'   computation fails. If no scores can be computed for any node, returns NULL
#'   with a warning.
#'
#' @examples
#' \donttest{
#'   library(grmtree)
#'   library(hlt)
#'   data("asti", package = "hlt")
#'   asti$resp <- data.matrix(asti[, 1:4])
#'
#'   # Fit GRM tree with gender and group as partitioning variables
#'   tree <- grmtree(resp ~ gender + group,
#'           data = asti,
#'           control = grmtree.control(minbucket = 30))
#'
#' # Compute EAP scores for all terminal nodes
#' node_scores <- fscores_grmtree(tree)
#'
#' # Compute MAP scores instead
#' node_scores_map <- fscores_grmtree(tree, method = "MAP")
#' }
#'
#' @seealso \code{\link[mirt]{fscores}} for factor scoring methods,
#' \code{\link{grmtree}} fits a Graded Response Model Tree,
#' \code{\link{grmforest}} for GRM Forests, \code{\link{threshpar_grmtree}} for
#' extracting threshold parameters, \code{\link{discrpar_grmtree}} for
#' extracting discrimination parameters, \code{\link{itempar_grmtree}} for
#' extracting item parameters, \code{\link{generate_node_scores_dataset}}
#' generates combined dataset with node assignments and factor scores
#'
#' @export
fscores_grmtree <- function(grmtree_obj, method = "EAP") {
  # Input validation
  if (missing(grmtree_obj)) {
    stop("Argument 'grmtree_obj' is missing with no default.")
  }
  if (!inherits(grmtree_obj, "grmtree")) {
    stop("grmtree_obj must be a GRM tree object from grmtree() function.")
  }
  if (!method %in% c("EAP", "MAP", "ML", "WLE")) {
    stop("method must be one of: 'EAP', 'MAP', 'ML', or 'WLE'")
  }

  # Identify terminal nodes - will return at least root node (1) for empty trees
  terminal_nodes <- partykit::nodeids(grmtree_obj, terminal = TRUE)

  # Initialize list to store scores for each node
  scores_list <- vector("list", length(terminal_nodes))
  names(scores_list) <- as.character(terminal_nodes)

  # Loop over each terminal node
  for (node_id in terminal_nodes) {
    message("Processing node: ", node_id)
    # Get data for the node
    node_data <- tryCatch(
      partykit::data_party(grmtree_obj, id = node_id),
      error = function(e) {
        return(data.frame()) # Return empty data.frame to trigger next check
      }
    )

    if (nrow(node_data) == 0) next

    # Extract the fitted model for the node
    node_model <- tryCatch(
      apply_to_models(grmtree_obj, node = node_id, drop = TRUE),
      error = function(e) NULL
    )

    if (is.null(node_model)) next

    # Compute factor scores for the node WITH convergence checking
    node_scores <- suppressWarnings(
      tryCatch({
        result <- mirt::fscores(node_model, method = method)
        # Check for convergence issues
        if (any(is.na(result)) || any(!is.finite(result))) {
          warning("Some factor score estimates may not have converged properly")
        }
        if (is.matrix(result)) result[, 1] else result  # Ensure vector output
      }, error = function(e) {
        warning("Factor score computation failed for node ", node_id, ": ", e$message)
        return(NULL)
      })
    )

    if (!is.null(node_scores)) {
      scores_list[[as.character(node_id)]] <- node_scores
    }
  }

  # Remove NULL elements
  scores_list <- Filter(Negate(is.null), scores_list)

  if (length(scores_list) == 0) {
    stop("No factor scores were successfully computed for any node.")
  }

  return(scores_list)
}

library(testthat)
library(hlt)
library(mirt)

## Test that fscores_grmtree input validation works
test_that("fscores_grmtree input validation works", {
  # Create a simple GRM tree for testing
  set.seed(123)
  sim_data <- data.frame(
    x1 = rnorm(100),
    x2 = sample(0:1, 100, replace = TRUE),
    item1 = sample(1:5, 100, replace = TRUE),
    item2 = sample(1:5, 100, replace = TRUE)
  )

  # Fit a simple tree (assuming grmtree function exists)
  tree <- grmtree(cbind(item1, item2) ~ x1 + x2, data = sim_data)

  # Test input validation
  expect_error(fscores_grmtree(), "missing with no default")
  expect_error(fscores_grmtree("not_a_tree"), "must be a GRM tree object")
  expect_error(fscores_grmtree(tree, method = "invalid"), "must be one of")
})


## Test that fscores_grmtree returns correct structure
test_that("fscores_grmtree returns correct structure", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  ## Asti data
  data("asti", package = "hlt")
  ## Create response as a matrix
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))

  # Test output structure - allow messages from mirt
  expect_message(
    result <- fscores_grmtree(tree),
    "Processing node"
  )
  expect_type(result, "list")
  expect_true(length(result) > 0)
  expect_true(all(sapply(result, is.numeric)))

  # Test different methods - expect messages
  test_method <- function(method)  {
    # Just expect the message and check the result structure
    expect_message(
      res <- fscores_grmtree(tree, method),
      "Terminal nodes|Processing node"
    )
    expect_true(all(sapply(res, is.numeric)))
  }

  test_method("MAP")
  test_method("ML")
  test_method("WLE")

  # Test invalid inputs
  expect_error(fscores_grmtree(list()), "must be a GRM tree object")
  expect_error(fscores_grmtree(tree, method = "invalid"),
               "must be one of: 'EAP', 'MAP', 'ML', or 'WLE'")
})

#' Calculate Variable Importance for GRM Forest
#'
#' Computes permutation importance scores for variables in a GRM forest using
#' out-of-bag samples. Importance is measured by the decrease in log-likelihood
#' when a variable's values are permuted.
#'
#' @param forest A `grmforest` object created by `grmforest()`.
#' @param method Importance calculation method (currently only "permutation").
#' @param verbose Logical indicating whether to show progress messages.
#' @param seed Random seed for reproducibility.
#'
#' @return A named numeric vector of importance scores with class `varimp`.
#'   Higher values indicate more important variables.
#'
#' @examples
#' \donttest{
#' library(grmtree)
#' library(hlt)
#' data("asti", package = "hlt")
#' asti$resp <- data.matrix(asti[, 1:4])
#'
#' ## Fit the GRM Forest
#' forest <- grmforest(resp ~ gender + group, data = asti,
#' control = grmforest.control(n_tree = 5))
#'   importance <- varimp(forest)
#'
#' ## Print and plot the variable importance scores
#' print(importance)
#' plot(importance)
#' }
#'
#' @seealso \code{\link{grmtree}} fits a Graded Response Model Tree,
#'   \code{\link{grmforest}} for GRM Forests, \code{\link{grmforest.control}}
#'   creates a control object for `grmforest`, \code{\link{plot.varimp}} creates
#'   a bar plot of variable importance scores
#'
#' @export
#' @importFrom stats predict
varimp <- function(forest, method = "permutation", verbose = FALSE, seed = NULL) {
  # Validate inputs
  if (!inherits(forest, "grmforest")) {
    stop("'forest' must be a grmforest object created by grmforest()")
  }

  if (method != "permutation") {
    stop("Only permutation importance is implemented")
  }

  if (!is.null(seed)) set.seed(seed)


  # Get variable names from formula
  var_names <- all.vars(forest$formula)[-1]
  if (length(var_names) == 0) {
    stop("No predictor variables found in formula")
  }

  importance_scores <- numeric(length(var_names))
  names(importance_scores) <- var_names

  # Compute baseline performance
  baseline_perf <- evaluate_forest(forest, verbose)
  if (verbose) {
    message("Baseline OOB performance: ", baseline_perf)
    message("Using ", length(forest$trees), " trees")
  }

  # Calculate importance for each variable
  for (var in var_names) {
    if (verbose) message("\nProcessing variable: ", var)

    permuted_perf <- 0
    valid_trees <- 0

    for (i in seq_along(forest$trees)) {
      tree <- forest$trees[[i]]
      oob_data <- forest$oob_samples[[i]]

      # Skip invalid trees or OOB data
      if (is.null(tree) || is.null(oob_data) || nrow(oob_data) == 0) {
        if (verbose) message("Skipping tree ", i, " - no valid OOB data")
        next
      }

      # Create permuted copy
      permuted_data <- oob_data
      permuted_data[[var]] <- sample(permuted_data[[var]])

      # Evaluate permutation
      tree_perf <- evaluate_tree(tree, permuted_data, verbose)
      if (!is.na(tree_perf)) {
        permuted_perf <- permuted_perf + tree_perf
        valid_trees <- valid_trees + 1
        if (verbose) message("Tree ", i, " perf: ", tree_perf)
      }
    }

    # Calculate importance score
    if (valid_trees > 0) {
      importance_scores[var] <- baseline_perf - (permuted_perf / valid_trees)
      if (verbose) {
        message("Variable: ", var,
                " | Importance: ", importance_scores[var],
                " (based on ", valid_trees, " trees)")
      }
    } else {
      warning("No valid trees for variable '", var, "' - importance set to 0")
      importance_scores[var] <- 0
    }
  }

  if (valid_trees == 0) {
    warning("No valid trees with OOB data available")
    importance_scores <- numeric(length(var_names))
    names(importance_scores) <- var_names
    class(importance_scores) <- c("varimp", "numeric")
    return(importance_scores)  # Return properly structured zero vector
  }

  # Sort by importance
  importance_scores <- sort(importance_scores, decreasing = TRUE)
  class(importance_scores) <- c("varimp", "numeric")

  return(importance_scores)
}

# Internal helper functions --------------------------------------------------

evaluate_forest <- function(forest, verbose = FALSE) {
  total_perf <- 0
  valid_trees <- 0

  for (i in seq_along(forest$trees)) {
    tree <- forest$trees[[i]]
    oob_data <- forest$oob_samples[[i]]

    if (!is.null(tree) && !is.null(oob_data) && nrow(oob_data) > 0) {
      tree_perf <- evaluate_tree(tree, oob_data, verbose)
      if (!is.na(tree_perf)) {
        total_perf <- total_perf + tree_perf
        valid_trees <- valid_trees + 1
        if (verbose) message("Tree ", i, " baseline perf: ", tree_perf)
      }
    }
  }

  if (valid_trees == 0) {
    warning("No valid trees with OOB data available")
    return(0)
  }

  return(total_perf / valid_trees)
}

evaluate_tree <- function(tree, data, verbose = FALSE) {
  if (is.null(tree) || nrow(data) == 0) return(NA_real_)

  # Get terminal nodes
  terminal_nodes <- tryCatch(
    predict(tree, data, type = "node"),
    error = function(e) {
      if (verbose) warning("Prediction failed: ", e$message)
      return(rep(1, nrow(data))) # Default to root node
    }
  )

  loglik <- 0
  for (node_id in unique(terminal_nodes)) {
    node_data <- data[terminal_nodes == node_id, , drop = FALSE]
    if (nrow(node_data) > 0) {
      node_model <- tryCatch(
        get_node_model(tree, node_id),
        error = function(e) {
          if (verbose) warning("Node model error: ", e$message)
          NULL
        }
      )

      if (!is.null(node_model)) {
        node_loglik <- calculate_node_loglik(node_model, node_data)
        loglik <- loglik + node_loglik
      }
    }
  }

  return(loglik)
}

calculate_node_loglik <- function(grm_model, data) {
  # Extract response matrix - assumes response is in a matrix column called 'resp'
  y <- as.matrix(data$resp)
  if (is.null(y)) stop("Response matrix 'resp' not found in data")

  # Get item parameters
  item_params <- tryCatch(
    mirt::coef(grm_model, simplify = TRUE)$items,
    error = function(e) stop("Failed to get item parameters: ", e$message)
  )

  loglik <- 0
  for (i in seq_len(nrow(y))) {
    for (j in seq_len(ncol(y))) {
      response <- y[i, j]
      if (is.na(response)) next

      a <- item_params[j, "a1"]
      d <- item_params[j, grep("d", colnames(item_params))]

      # Calculate response probability
      if (response == 1) {
        prob <- stats::plogis(a * (0 - d[1]))
      } else if (response == length(d) + 1) {
        prob <- 1 - stats::plogis(a * (0 - d[length(d)]))
      } else {
        prob <- stats::plogis(a * (0 - d[response])) - stats::plogis(a * (0 - d[response - 1]))
      }

      # Avoid log(0)
      prob <- pmax(prob, .Machine$double.eps)
      loglik <- loglik + log(prob)
    }
  }

  return(loglik)
}

get_node_model <- function(tree, node_id) {
  node <- tryCatch(
    partykit::nodeapply(tree, ids = node_id, FUN = function(x) x)[[1]],
    error = function(e) stop("Failed to get node: ", e$message)
  )

  if (is.null(node$info$object)) {
    stop("No model found in node ", node_id)
  }

  return(node$info$object)
}

library(testthat)
library(mirt)
library(hlt)

## Test for basic varimp functionality
test_that("varimp works with valid forest", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Create forest - may or may not have warnings
  suppressWarnings({
    forest <- grmforest(resp ~ gender + group, data = asti,
                        control = grmforest.control(n_tree = 5))
  })

  # Basic functionality
  imp <- varimp(forest)

  expect_type(imp, "double")
  expect_named(imp)
  expect_s3_class(imp, "varimp")

  # With verbose - expect specific message
  expect_message(
    varimp(forest, verbose = TRUE),
    "Processing variable"
  )

  # With seed - test reproducibility
  imp1 <- varimp(forest, seed = 123)
  imp2 <- varimp(forest, seed = 123)
  expect_equal(imp1, imp2)

  # Additional test - check we have some non-zero importance values
  expect_true(any(imp != 0))
})

## Test case for edge cases
test_that("varimp handles edge cases", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Create forest
  suppressWarnings({
    forest <- grmforest(resp ~ gender + group, data = asti,
                        control = grmforest.control(n_tree = 5))
  })

  # Invalid inputs
  expect_error(varimp(list()), "must be a grmforest object")
  expect_error(varimp(forest, method = "invalid"), "Only permutation importance is implemented")

  # Create a properly structured bad forest with no valid trees
  bad_forest <- structure(
    list(
      trees = list(NULL, NULL),  # All trees NULL
      oob_samples = list(asti[1:10,], asti[11:20,]),
      formula = resp ~ gender + group,
      data = asti,
      call = NULL
    ),
    class = "grmforest"
  )

  # Expect warnings about no valid trees - suppress additional warnings
  suppressWarnings({
    expect_warning(
      result <- varimp(bad_forest),
      "No valid trees"
    )
  })

  # Should return named vector of zeros with proper class
  expect_type(result, "double")
  expect_named(result)
  expect_s3_class(result, "varimp")
  expect_true(all(result == 0))
  expect_length(result, 2)
})

## Test for varimp plot
test_that("plot.varimp works correctly", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")
  skip_if_not_installed("ggplot2")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Create the forest object
  suppressWarnings({
    forest <- grmforest(resp ~ gender + group, data = asti,
                        control = grmforest.control(n_tree = 5))
  })

  # Create the variable importance
  imp <- varimp(forest)

  # Base R plot
  expect_silent(plot(imp, use_ggplot = FALSE))

  # ggplot2 plot if available
  expect_silent(plot(imp, use_ggplot = TRUE))

  # Top n variables
  expect_silent(plot(imp, top_n = 1))

  # Invalid inputs
  expect_error(plot.varimp(list()), "must be a varimp object")
  expect_error(plot(imp, top_n = 0), "must be NULL or a positive integer")

  # Additional test for non-varimp objects
  expect_error(
    plot.varimp(structure(list(), class = "not_varimp")),
    "must be a varimp object"
  )
})

## Test for the internal helper functions
test_that("internal helper functions work", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 5))

  # get_node_model
  nodes <- partykit::nodeids(tree, terminal = TRUE)
  expect_silent(model <- get_node_model(tree, nodes[1]))
  expect_true(inherits(model, "SingleGroupClass") || is.null(model))

  # calculate_node_loglik
  dat <- asti[1:5, ]
  attr(dat, "response.var") <- "resp"
  expect_type(calculate_node_loglik(model, dat), "double")

  # evaluate_tree
  expect_type(evaluate_tree(tree, asti[1:5, ]), "double")

  # evaluate_forest
  suppressWarnings({
    forest <- grmforest(resp ~ gender + group, data = asti,
                        control = grmforest.control(n_tree = 5))
  })
  expect_type(evaluate_forest(forest), "double")
})

.onLoad <- function(libname, pkgname) {
  # Register S3 methods explicitly to ensure they work even when
  # other packages mask base generics
  register_s3_method("print", "grmtree")
  register_s3_method("plot", "grmtree")
}

# Standalone S3 registration helper
register_s3_method <- function(generic, class, method = NULL) {
  if (is.null(method)) {
    method <- get(paste0(generic, ".", class), envir = parent.frame())
  }

  registerS3method(
    generic,
    class,
    method,
    envir = asNamespace(utils::packageName())
  )
}

#' Control Parameters for GRM Forest
#'
#' Creates a control object for `grmforest` containing parameters that control
#' the forest growing process including sampling, tree growing, and error
#' handling.
#'
#' @param n_tree Number of trees in the forest (default: 100).
#' @param sampling Sampling method: "bootstrap" (with replacement) or
#'   "subsample" (without replacement) (default: "bootstrap").
#' @param sample_fraction Fraction of data to sample for each tree (default:
#'   0.632).
#' @param mtry Number of variables randomly sampled as candidates at each split.
#'   If NULL, all variables are considered (default: NULL).
#' @param remove_dead_trees Logical indicating whether to remove trees that
#'   encounter errors during fitting (default: TRUE).
#' @param control Control parameters for individual trees created by
#'   `grmtree.control()`.
#' @param alpha Significance level for splitting (default: 0.05).
#' @param minbucket Minimum number of observations in terminal nodes (default:
#'   20).
#' @param seed Random seed for reproducibility (default: NULL).
#'
#' @return A list of class `grmforest_control` containing: \item{n_tree}{Number
#'   of trees} \item{sampling}{Sampling method} \item{sample_fraction}{Sample
#'   fraction} \item{mtry}{Number of variables to try at each split}
#'   \item{remove_dead_trees}{Whether to remove failed trees}
#'   \item{control}{Tree control parameters} \item{seed}{Random seed}
#'
#' @examples
#' library(grmtree)
#' # Control with 50 trees using subsampling
#' ctrl <- grmforest.control(n_tree = 50, sampling = "subsample")
#'
#' # Control with specific tree parameters
#' ctrl <- grmforest.control(
#'   control = grmtree.control(minbucket = 30, alpha = 0.01)
#' )
#'
#' @seealso \code{\link{grmtree.control}} creates a control object for
#' `grmtree`, \code{\link{plot.grmtree}} creates plot for the `grmtree` object,
#' \code{\link{grmforest}} for GRM Forests,
#'
#' @export
grmforest.control <- function(
    n_tree = 100,
    sampling = "bootstrap",
    sample_fraction = 0.632,
    mtry = NULL,
    remove_dead_trees = TRUE,
    control = grmtree.control(),
    alpha = 0.05,
    minbucket = 20,
    seed = NULL
) {
  # Validate number of trees
  if (!is.numeric(n_tree) || length(n_tree) != 1) {
    stop("'n_tree' must be a single numeric value")
  }
  if (n_tree < 1) {
    stop("'n_tree' must be at least 1")
  }

  # Validate sampling method
  sampling_methods <- c("bootstrap", "subsample")
  if (!sampling %in% sampling_methods) {
    stop("'sampling' must be one of: ", paste(sampling_methods, collapse = ", "))
  }

  # Validate sample fraction
  if (!is.numeric(sample_fraction) || length(sample_fraction) != 1) {
    stop("'sample_fraction' must be a single numeric value")
  }
  if (sample_fraction <= 0 || sample_fraction > 1) {
    stop("'sample_fraction' must be between 0 and 1")
  }

  # Validate mtry
  if (!is.null(mtry) && (!is.numeric(mtry) || mtry < 1)) {
    stop("'mtry' must be NULL or a positive integer")
  }

  # Validate control parameters
  if (!inherits(control, "grmtree_control")) {
    stop("'control' must be created by grmtree.control()")
  }

  # Validate seed
  if (!is.null(seed) && !is.numeric(seed)) {
    stop("'seed' must be NULL or a numeric value")
  }

  # Return control object
  control_obj <- list(
    n_tree = as.integer(n_tree),
    sampling = sampling,
    sample_fraction = sample_fraction,
    mtry = if (!is.null(mtry)) as.integer(mtry) else NULL,
    remove_dead_trees = isTRUE(remove_dead_trees),
    control = control,
    seed = if (!is.null(seed)) as.integer(seed) else NULL
  )

  class(control_obj) <- "grmforest_control"
  return(control_obj)
}

library(testthat)
library(hlt)
library(mirt)

## Test for grmforest.control()
test_that("grmforest.control creates valid control object", {
  # Default parameters
  ctrl <- grmforest.control()
  expect_s3_class(ctrl, "grmforest_control")
  expect_equal(ctrl$n_tree, 100L)

  # Custom parameters
  ctrl <- grmforest.control(
    n_tree = 5,
    sampling = "subsample",
    sample_fraction = 0.5,
    mtry = 2,
    seed = 123
  )
  expect_equal(ctrl$n_tree, 5L)
  expect_equal(ctrl$sampling, "subsample")
  expect_equal(ctrl$seed, 123L)

  # Invalid parameters
  expect_error(grmforest.control(n_tree = 0), "must be at least 1")
  expect_error(grmforest.control(sampling = "invalid"), "must be one of")
  expect_error(grmforest.control(sample_fraction = 1.1), "between 0 and 1")
})

## Test for grmforest fitting
test_that("grmforest fits with valid inputs", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Test basic forest - may or may not have warnings depending on tree success
  suppressWarnings({
    forest <- grmforest(resp ~ gender + group, data = asti,
                        control = grmforest.control(n_tree = 5))
  })

  expect_s3_class(forest, "grmforest")
  expect_true(length(forest$trees) > 0)  # At least some trees succeeded
  expect_equal(length(forest$trees), length(forest$oob_samples))

  # Test with subsampling
  suppressWarnings({
    forest <- grmforest(resp ~ gender + group, data = asti,
                        control = grmforest.control(n_tree = 5, sampling = "subsample"))
  })

  expect_s3_class(forest, "grmforest")
  expect_true(length(forest$trees) > 0)

  # Test invalid formula
  expect_error(
    suppressWarnings(grmforest(group ~ gender, data = asti)),
    "Response variable must be a matrix of item responses"
  )
})

## grmforest with edge cases and tree failures
test_that("grmforest handles errors appropriately", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Too small dataset
  expect_error(
    grmforest(resp[1:5,] ~ gender + group, data = asti[1:5,]), "Insufficient data")

  # With tree failures allowed - may or may not have warnings
  ctrl <- grmforest.control(n_tree = 5, remove_dead_trees = TRUE)
  suppressWarnings({
    forest <- grmforest(resp ~ gender + group, data = asti, control = ctrl)
  })
  expect_s3_class(forest, "grmforest")
  expect_true(length(forest$trees) > 0)

  # Test invalid formula
  expect_error(
    grmforest(group ~ gender, data = asti),
    "Response variable must be a matrix"
  )

  # Test invalid control object
  bad_ctrl <- grmforest.control()
  class(bad_ctrl) <- "not_control"
  expect_error(
    grmforest(resp ~ gender + group, data = asti, control = bad_ctrl),
    "'control' must be created by grmforest.control()"
  )
})

## Test for print method
test_that("print.grmforest works correctly", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  suppressWarnings({
    forest <- grmforest(resp ~ gender + group, data = asti,
                        control = grmforest.control(n_tree = 2, seed = 123))
  })

  expect_output(print(forest), "GRM Forest with")
})

#' Print Method for GRM Tree Objects
#'
#' Displays a formatted summary of a GRM (Graded Response Model) tree object.
#' This function extends \code{print.modelparty} from the partykit package with
#' specialized formatting for GRM trees.
#'
#' @param x A GRM tree object of class 'grmtree'.
#' @param title Character string specifying the title for the print output
#'   (default: "Graded Response Model Tree").
#' @param objfun Character string labeling the objective function
#'   (default: "negative log-likelihood").
#' @param ... Additional arguments passed to \code{print.modelparty}.
#'
#' @return Invisibly returns the GRM tree object. Primarily called for its side
#'   effect of printing a formatted summary.
#'
#' @details The print method provides a comprehensive summary of the GRM tree,
#' including:
#' \itemize{
#'   \item Model formula used for fitting
#'   \item Tree structure with node information
#'   \item Item parameter estimates for each terminal node
#'   \item Confidence intervals for parameters
#'   \item Group parameters (mean and covariance)
#'   \item Summary statistics (number of nodes, objective function value)
#' }
#'
#' @examples
#' \donttest{
#' library(grmtree)
#' library(hlt)
#' data("asti", package = "hlt")
#' asti$resp <- data.matrix(asti[, 1:4])
#'
#' # Fit GRM tree
#' tree <- grmtree(resp ~ gender + group,
#'                 data = asti,
#'                 control = grmtree.control(minbucket = 30))
#'
#' # Print the tree summary
#' print(tree)
#'
#' # Alternative syntax (automatically calls print.grmtree)
#' tree
#' }
#'
#' @seealso \code{\link[partykit]{print.modelparty}} for the underlying printing
#'   infrastructure, \code{\link{grmtree}} for creating GRM tree objects
#'
#'
#' @method print grmtree
#' @export


print.grmtree <- function(x,
                          title = "Graded Response Model Tree",
                          objfun = "negative log-likelihood",
                          ...) {

  if (!inherits(x, "grmtree")) {
    stop("'x' must be a grmtree object")
  }

  # Explicitly call partykit's print method
  partykit::print.modelparty(x, title = title, objfun = objfun, ...)

  invisible(x)
}

#' Generate Dataset with Node Assignments and Factor Scores
#'
#' Creates a dataset by augmenting the original data with node assignments
#' and computed factor scores. Unlike the previous version which only returned
#' model frame variables, this version merges node and score information back
#' to the full original data frame.
#'
#' @param object A \code{grmtree} or \code{longitudinal_grmtree} object.
#' @param data The original data frame used to fit the tree. If provided,
#'   the output contains all columns from this data frame plus node and
#'   factor score columns. If NULL (default), returns only model frame
#'   variables (backward-compatible behavior).
#' @param method Scoring method: "EAP" (default), "MAP", "ML", or "WLE".
#'
#' @return A data.frame containing:
#'   \describe{
#'     \item{All original columns}{From \code{data} if provided}
#'     \item{node}{Factor indicating terminal node membership}
#'     \item{factor_score}{For cross-sectional grmtree: single latent score.
#'       For longitudinal: Theta_T1.}
#'     \item{Theta_T1}{(Longitudinal only) Latent trait at T1}
#'     \item{Theta_T2}{(Longitudinal only) Latent trait at T2}
#'   }
#'
#' @details
#' The function works by:
#' \enumerate{
#'   \item Predicting node membership for each observation using
#'     \code{predict(object, type = "node")}
#'   \item Computing factor scores within each terminal node using the
#'     node-specific model
#'   \item Merging the results back to the original data by row position
#' }
#'
#' When \code{data} is provided, the function ensures that the output
#' contains all columns from the original data frame, not just the
#' variables used in the model formula. This is important when the
#' original data contains clinical variables, identifiers, or other
#' columns not used as partitioning variables.
#'
#' @examplesIf interactive()
#' # Cross-sectional GRMTree
#' library(grmtree)
#' library(hlt)
#' data("asti", package = "hlt")
#' asti$resp <- data.matrix(asti[, 1:4])
#'
#'   # Fit GRM tree with gender and group as partitioning variables
#'   tree <- grmtree(resp ~ gender + group,
#'           data = asti,
#'           control = grmtree.control(minbucket = 30))
#'
#' # Generate combined dataset
#' scored_data <- generate_node_scores_dataset(tree, data = asti)
#'
#' # Plot scores by node
#' boxplot(factor_score ~ node, data = scored_data)
#'
#' @seealso \code{\link{grmtree}} fits a Graded Response Model Tree,
#' \code{\link{grmforest}} for GRM Forests, \code{\link{fscores_grmtree}} for
#' computing factor scores, \code{\link{threshpar_grmtree}} for extracting
#' threshold parameters, \code{\link{discrpar_grmtree}} for extracting
#' discrimination parameters, \code{\link{itempar_grmtree}} for extracting item
#' parameters, \code{\link{longitudinal_grmtree}} for longitudinal GRMTree,
#' \code{\link{fscores_longitudinal_grmtree}}, for computing factor scores for longitudinal GRMTree
#'
#' @export
#' @importFrom stats na.pass

generate_node_scores_dataset <- function(object, data = NULL, method = "EAP") {

  if (!inherits(object, "grmtree")) {
    stop("'object' must be a grmtree or longitudinal_grmtree object")
  }
  if (!method %in% c("EAP", "MAP", "ML", "WLE")) {
    stop("method must be one of: 'EAP', 'MAP', 'ML', or 'WLE'")
  }

  is_longitudinal <- inherits(object, "longitudinal_grmtree")

  # ---- Get node assignments ----
  # predict() on a mob object returns the terminal node ID for each obs
  node_ids <- predict(object, type = "node")

  terminal_nodes <- sort(unique(node_ids))
  n_obs <- length(node_ids)

  # ---- Compute factor scores per node ----
  if (is_longitudinal) {
    scores_list <- fscores_longitudinal_grmtree(object, method = method)
  } else {
    scores_list <- fscores_grmtree(object, method = method)
  }

  # ---- Build output vectors ----
  if (is_longitudinal) {
    theta_t1 <- rep(NA_real_, n_obs)
    theta_t2 <- rep(NA_real_, n_obs)

    for (nd_id in names(scores_list)) {
      nd_int <- as.integer(nd_id)
      obs_idx <- which(node_ids == nd_int)
      nd_scores <- scores_list[[nd_id]]

      if (nrow(nd_scores) != length(obs_idx)) {
        warning("Score count mismatch in node ", nd_id,
                ": expected ", length(obs_idx), ", got ", nrow(nd_scores))
        next
      }
      theta_t1[obs_idx] <- nd_scores$Theta_T1
      theta_t2[obs_idx] <- nd_scores$Theta_T2
    }
  } else {
    factor_score <- rep(NA_real_, n_obs)

    for (nd_id in names(scores_list)) {
      nd_int <- as.integer(nd_id)
      obs_idx <- which(node_ids == nd_int)
      nd_scores <- scores_list[[nd_id]]

      if (length(nd_scores) != length(obs_idx)) {
        warning("Score count mismatch in node ", nd_id)
        next
      }
      factor_score[obs_idx] <- nd_scores
    }
  }

  # ---- Create node labels ----
  node_factor <- factor(
    node_ids,
    levels = terminal_nodes,
    labels = paste0("Node ", terminal_nodes)
  )

  # ---- Build output data frame ----
  if (!is.null(data)) {
    # Merge back to original data
    if (nrow(data) != n_obs) {
      warning("Original data has ", nrow(data), " rows but model has ",
              n_obs, " observations (after na.action). ",
              "Attempting to match by model frame row indices.")
      # Get the row indices that survived na.action
      mf <- model.frame(formula(object), data = data, na.action = na.pass)
      complete_idx <- complete.cases(mf)
      if (sum(complete_idx) == n_obs) {
        out <- data[complete_idx, , drop = FALSE]
      } else {
        warning("Cannot match rows. Returning model frame variables only.")
        out <- data.frame(row.names = 1:n_obs)
      }
    } else {
      out <- data
    }
  } else {
    # Backward-compatible: return model frame variables
    out <- data.frame(row.names = 1:n_obs)
  }

  # Add node and scores
  out$node <- node_factor

  if (is_longitudinal) {
    out$Theta_T1 <- theta_t1
    out$Theta_T2 <- theta_t2
  } else {
    out$factor_score <- factor_score
  }

  rownames(out) <- NULL
  return(out)
}

#' Fit a Forest of Graded Response Model Trees for Ensemble-Based DIF Detection
#'
#' This function implements a forest of graded response model trees (GRM Forest)
#' using bootstrap aggregation (bagging) or random subsampling to enhance the
#' detection and analysis of differential item functioning (DIF) in polytomous items.
#' The GRM Forest approach combines the strengths of multiple GRMTrees to provide
#' more robust and stable DIF detection, particularly for complex datasets with
#' high-dimensional covariates or subtle DIF patterns.
#'
#' The algorithm works by fitting multiple GRMTrees, each on a random sample of
#' the original data (either through bootstrap sampling or subsampling). For each
#' tree, approximately one-third of the observations are left out as out-of-bag
#' (OOB) samples, which are used for internal validation and variable importance
#' calculation. The ensemble approach reduces variance, minimizes overfitting,
#' and provides more reliable identification of covariates associated with DIF.
#'
#' Key advantages of the GRM Forest approach include:
#' - Enhanced stability in DIF detection across different sampling variations
#' - Robust variable importance measures that quantify the relative contribution
#'   of each covariate to DIF patterns
#' - Reduced false positive rates through consensus-based detection
#' - Ability to handle high-dimensional covariate spaces effectively
#' - Internal validation through out-of-bag error estimation
#'
#' The forest implementation supports both bootstrap aggregation (where samples
#' are drawn with replacement) and subsampling (without replacement), allowing
#' flexibility for different data characteristics and research objectives.
#'
#' @param formula A formula specifying the model structure with the response
#'   matrix on the left and partitioning variables on the right (e.g.,
#'   `response_matrix ~ age + gender + education + clinical_variables`).
#' @param data A data frame containing the response matrix and partitioning
#'   variables. The response matrix should contain polytomous items coded
#'   as ordered factors.
#' @param control A control object created by `grmforest.control()`.
#' @param ... Additional arguments passed to underlying `grmtree()` function.
#'
#' @return An object of class `grmforest` containing: \item{trees}{List of
#'   fitted GRM trees} \item{oob_samples}{List of out-of-bag samples for each
#'   tree} \item{formula}{The model formula} \item{data}{The original dataset}
#'   \item{call}{The function call}
#'
#' @examples
#'
#' \donttest{
#'   library(grmtree)
#'   library(hlt)
#'   data("asti", package = "hlt")
#'   asti$resp <- data.matrix(asti[, 1:4])
#'
#'   # Fit forest with default parameters
#'   forest <- grmforest(resp ~ gender + group, data = asti)
#'
#'   # Fit with custom control
#'   ctrl <- grmforest.control(n_tree = 20, sampling = "subsample")
#'   forest <- grmforest(resp ~ gender + group, data = asti, control = ctrl)
#' }
#'
#' @seealso \code{\link{grmtree}} fits a Graded Response Model Tree,
#' \code{\link{grmtree.control}} creates a control object for `grmtree`,
#' \code{\link{grmforest.control}} creates a control object for `grmforest`,
#' \code{\link{varimp}} calculates the variable importance for GRM Forest,
#' \code{\link{plot.varimp}} creates a bar plot of variable importance scores
#'
#' @export
grmforest <- function(formula, data, control = grmforest.control(), ...) {
  # Validate inputs
  if (!inherits(formula, "formula")) {
    stop("'formula' must be a valid formula object")
  }

  if (!is.data.frame(data)) {
    stop("'data' must be a data.frame")
  }


  # Parse formula to check response variable
  mf <- tryCatch(
    model.frame(formula, data = data),
    error = function(e) {
      stop("Error in model.frame: ", e$message)
    }
  )

  y <- model.response(mf)
  if (!is.matrix(y)) {
    stop("Response variable must be a matrix of item responses")
  }

  if (!inherits(control, "grmforest_control")) {
    stop("'control' must be created by grmforest.control()")
  }

  if (nrow(data) < 10) {
    stop("Insufficient data: nrow(data) must be at least 10")
  }

  # Set random seed if specified
  if (!is.null(control$seed)) {
    set.seed(control$seed)
  }

  # Initialize forest structure
  forest <- list(
    trees = vector("list", control$n_tree),
    oob_samples = vector("list", control$n_tree),
    formula = formula,
    data = data,
    call = match.call()
  )

  n <- nrow(data)
  sample_size <- round(n * control$sample_fraction)

  # Grow forest
  for (i in seq_len(control$n_tree)) {
    # Sample data
    if (control$sampling == "bootstrap") {
      sample_idx <- sample.int(n, size = sample_size, replace = TRUE)
    } else {
      sample_idx <- sample.int(n, size = sample_size, replace = FALSE)
    }
    oob_idx <- setdiff(seq_len(n), sample_idx)

    # Fit tree with error handling
    tree <- tryCatch({
      grmtree(
        formula,
        data = data[sample_idx, , drop = FALSE],
        control = control$control,
        ...
      )
    }, error = function(e) {
      if (control$remove_dead_trees) {
        warning("Tree ", i, " failed: ", e$message)
        NULL
      } else {
        stop("Tree ", i, " failed: ", e$message)
      }
    })

    # Store results
    if (!is.null(tree)) {
      forest$trees[[i]] <- tree
      forest$oob_samples[[i]] <- data[oob_idx, , drop = FALSE]
    }

    # Progress reporting
    if (interactive() && i %% 10 == 0) {
      cat(sprintf("Fitted %d/%d trees\n", i, control$n_tree))
    }
  }

  # Remove NULL entries if any trees failed
  failed_trees <- sapply(forest$trees, is.null)
  if (any(failed_trees)) {
    forest$trees <- forest$trees[!failed_trees]
    forest$oob_samples <- forest$oob_samples[!failed_trees]
    warning(sum(failed_trees), " trees failed and were removed")
  }

  if (length(forest$trees) == 0) {
    stop("All trees failed to fit")
  }

  class(forest) <- "grmforest"
  return(forest)
}



#' Print Method for GRM Forest
#'
#' @param x A `grmforest` object
#' @param ... Additional arguments (currently unused)
#'
#' @return Invisibly returns the input object
#'
#' @export
print.grmforest <- function(x, ...) {
  if (!inherits(x, "grmforest")) {
    stop("'x' must be a grmforest object")
  }

  cat("GRM Forest with", length(x$trees), "trees\n")

  invisible(x)
}

################################################################################
##
## Response Shift Visualization Functions
##
## Plot 1: plot_rs_tree() — Annotated tree with RS summary panels
## Plot 2: plot_rs_heatmap() — Item-level RS heatmap across nodes
##
## Author: Olayinka Arimoro
## Date: March 2026
##
################################################################################

#' Plot Response Shift Summary Tree
#'
#' Displays the Longitudinal GRMTree structure with RS characterization
#' results annotated in each terminal node panel. Each panel shows the
#' omnibus test result, latent trait parameters, and an item-level RS
#' heatmap color-coded by RS type.
#'
#' @param tree A \code{longitudinal_grmtree} object.
#' @param rs An \code{rs_characterization} object from
#'   \code{\link{rs_characterize}}.
#' @param item_labels Optional character vector of short item labels.
#'   If NULL, uses "Item 1", "Item 2", etc.
#' @param tnex Numeric scaling factor for terminal node panels (default: 2.5).
#' @param drop_terminal Logical (default: TRUE).
#' @param ... Additional arguments passed to \code{plot.modelparty}.
#'
#' @return Invisibly returns the tree object.
#'
#' @details
#' Each terminal node panel contains:
#' \itemize{
#'   \item Omnibus RS result: chi-squared, adjusted p-value, and detection
#'     status
#'   \item Latent parameters: mu_T2 and cor(T1,T2)
#'   \item Item-level RS bar: colored cells for each item indicating RS type
#'     (blue = recalibration, red = reprioritization, purple = both,
#'     orange = significant small effect, gray = none/not tested)
#' }
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Phase 1: fit the longitudinal GRM tree
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Phase 2: characterize response shift within each subgroup
#'   rs <- rs_characterize(ltree, p_adjust = "fdr",
#'     global_p_adjust = "bonferroni")
#'
#'   plot_rs_tree(ltree, rs,
#'     item_labels = c("Listen", "Info", "Crisis", "Confide",
#'       "Advice", "Fears", "Personal", "Understand"))
#'
#' @export
#' @import grid
plot_rs_tree <- function(tree, rs, item_labels = NULL, tnex = 2.5,
                         drop_terminal = TRUE, ...) {

  if (!inherits(tree, "longitudinal_grmtree")) {
    stop("'tree' must be a longitudinal_grmtree object")
  }
  if (!inherits(rs, "rs_characterization")) {
    stop("'rs' must be an rs_characterization object from rs_characterize()")
  }

  n_items <- rs$n_items
  if (is.null(item_labels)) {
    item_labels <- paste0("I", 1:n_items)
  }
  if (length(item_labels) != n_items) {
    stop("item_labels must have length ", n_items)
  }

  # RS type color mapping
  rs_colors <- c(
    "Recalibration" = "#3182BD",           # blue
    "Reprioritization" = "#E6550D",        # red/orange
    "Both" = "#756BB1",                    # purple
    "Significant (small effect)" = "#FDAE6B", # light orange
    "None" = "#D9D9D9"                     # light gray
  )

  # Build panel function
  panelfun <- function(node) {
    id <- as.character(partykit::id_node(node))
    nobs <- partykit::info_node(node)$nobs
    g <- rs$global

    # Find this node's row in global results
    node_row <- which(g$Node == as.integer(id))

    if (length(node_row) == 0) return()

    node_g <- g[node_row, ]
    has_adj <- "LRT_p_adj" %in% names(g)

    # Get item-level results for this node
    if (!is.null(rs$item_level)) {
      node_items <- rs$item_level[rs$item_level$Node == as.integer(id), ]
    } else {
      node_items <- NULL
    }

    # ---- Setup viewport ----
    lab <- paste0("rs_node_", id)
    top.vp <- grid::viewport(
      layout = grid::grid.layout(nrow = 4, ncol = 1,
                                 heights = grid::unit(c(1.2, 1, 1, 1.5), c("lines", "lines", "lines", "null"))
      ),
      width = grid::unit(0.95, "npc"),
      height = grid::unit(1, "npc") - grid::unit(2, "lines"),
      name = paste0(lab, "_top")
    )
    grid::pushViewport(top.vp)
    grid::grid.rect(gp = grid::gpar(fill = "white", col = "gray80"))

    # ---- Row 1: Node title ----
    grid::pushViewport(grid::viewport(layout.pos.row = 1, layout.pos.col = 1))
    grid::grid.text(
      paste0("Node ", id, " (n = ", nobs, ")"),
      gp = grid::gpar(fontface = "bold", cex = 0.85)
    )
    grid::upViewport()

    # ---- Row 2: Omnibus result ----
    grid::pushViewport(grid::viewport(layout.pos.row = 2, layout.pos.col = 1))

    p_display <- if (has_adj && !is.na(node_g$LRT_p_adj)) {
      formatC(node_g$LRT_p_adj, format = "e", digits = 2)
    } else {
      formatC(node_g$LRT_p, format = "e", digits = 2)
    }

    rs_label <- if (isTRUE(node_g$RS_detected)) "RS: YES" else "RS: No"
    rs_col <- if (isTRUE(node_g$RS_detected)) "#C0392B" else "#27AE60"

    grid::grid.text(
      paste0(rs_label, "  (p=", p_display, ")"),
      gp = grid::gpar(cex = 0.7, col = rs_col, fontface = "bold")
    )
    grid::upViewport()

    # ---- Row 3: Latent parameters ----
    grid::pushViewport(grid::viewport(layout.pos.row = 3, layout.pos.col = 1))
    grid::grid.text(
      paste0("\u03BC_T2=", round(node_g$mu_T2, 2),
             "  r=", round(node_g$cor_T1_T2, 2)),
      gp = grid::gpar(cex = 0.65, col = "gray30")
    )
    grid::upViewport()

    # ---- Row 4: Item RS heatmap bar ----
    grid::pushViewport(grid::viewport(layout.pos.row = 4, layout.pos.col = 1))

    # Determine RS type per item
    item_types <- rep("None", n_items)
    if (!is.null(node_items) && nrow(node_items) > 0) {
      has_p_adj <- "LRT_p_adj" %in% names(node_items)
      for (i in 1:nrow(node_items)) {
        item_idx <- node_items$Item[i]
        p_val <- if (has_p_adj) node_items$LRT_p_adj[i] else node_items$LRT_p[i]
        if (!is.na(p_val) && p_val < rs$alpha) {
          item_types[item_idx] <- node_items$RS_type[i]
        }
      }
    }

    # Draw colored rectangles for each item
    cell_width <- 1 / n_items
    for (i in 1:n_items) {
      col <- rs_colors[item_types[i]]
      if (is.na(col)) col <- "#D9D9D9"

      grid::grid.rect(
        x = grid::unit((i - 0.5) * cell_width, "npc"),
        y = grid::unit(0.55, "npc"),
        width = grid::unit(cell_width * 0.85, "npc"),
        height = grid::unit(0.55, "npc"),
        gp = grid::gpar(fill = col, col = "white", lwd = 0.5)
      )

      # Item label below
      grid::grid.text(
        item_labels[i],
        x = grid::unit((i - 0.5) * cell_width, "npc"),
        y = grid::unit(0.08, "npc"),
        gp = grid::gpar(cex = 0.45)
      )
    }

    grid::upViewport()  # row 4
    grid::upViewport()  # top
  }

  # Plot using partykit infrastructure
  partykit::plot.modelparty(
    tree,
    terminal_panel = panelfun,
    tnex = tnex,
    drop_terminal = drop_terminal,
    ...
  )

  # Add legend at bottom
  grid::grid.text(
    "RS Type:  \u25A0 Recalibration  \u25A0 Reprioritization  \u25A0 Both  \u25A0 Small effect  \u25A0 None",
    x = grid::unit(0.5, "npc"),
    y = grid::unit(0.02, "npc"),
    gp = grid::gpar(cex = 0.6, col = "gray40")
  )

  invisible(tree)
}


#' Plot Item-Level Response Shift Heatmap
#'
#' Creates a standalone heatmap showing RS type for each item across all
#' terminal nodes. Significant items are filled with RS-type colors;
#' non-significant items are white/light gray.
#'
#' @param rs An \code{rs_characterization} object from
#'   \code{\link{rs_characterize}}.
#' @param item_labels Optional character vector of item labels.
#'   If NULL, uses "Item 1", "Item 2", etc.
#' @param node_labels Optional character vector of node labels.
#'   If NULL, uses "Node X (n=Y)" from the global results.
#' @param show_chi2 Logical. If TRUE, display chi-squared values inside
#'   cells. Default is FALSE.
#' @param title Optional plot title. If NULL, uses a default title.
#' @param ... Additional arguments (unused).
#'
#' @return Invisibly returns the rs object. Called for its side effect of
#'   producing a plot.
#'
#' @details
#' The heatmap uses the following color coding:
#' \describe{
#'   \item{Blue}{Recalibration (threshold change)}
#'   \item{Red/Orange}{Reprioritization (discrimination change)}
#'   \item{Purple}{Both recalibration and reprioritization}
#'   \item{Light orange}{Significant but small effect}
#'   \item{White}{Not significant or not tested}
#' }
#'
#' Cells with significant RS (after p-value adjustment) are marked with
#' an asterisk (*). The omnibus RS result for each node is displayed at
#' the top of each column.
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Phase 1: fit the longitudinal GRM tree
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Phase 2: characterize response shift within each subgroup
#'   rs <- rs_characterize(ltree, p_adjust = "fdr",
#'     global_p_adjust = "bonferroni")
#'
#'   # Basic heatmap
#'   plot_rs_heatmap(rs)
#'
#'   # With custom labels
#'   plot_rs_heatmap(rs,
#'     item_labels = c("Listen", "Info", "Crisis", "Confide",
#'       "Advice", "Fears", "Personal", "Understand"))
#'
#'   # Show chi-squared values
#'   plot_rs_heatmap(rs, show_chi2 = TRUE)
#'
#' @export
plot_rs_heatmap <- function(rs, item_labels = NULL, node_labels = NULL,
                            show_chi2 = FALSE, title = NULL, ...) {

  if (!inherits(rs, "rs_characterization")) {
    stop("'rs' must be an rs_characterization object from rs_characterize()")
  }

  n_items <- rs$n_items
  g <- rs$global
  nodes <- g$Node
  n_nodes <- length(nodes)

  if (is.null(item_labels)) {
    item_labels <- paste("Item", 1:n_items)
  }
  if (is.null(node_labels)) {
    node_labels <- paste0("Node ", g$Node, "\n(n=", g$n, ")")
  }
  if (is.null(title)) {
    title <- "Item-Level Response Shift Characterization"
  }

  # RS type color mapping
  rs_colors <- c(
    "Recalibration" = "#3182BD",
    "Reprioritization" = "#E6550D",
    "Both" = "#756BB1",
    "Significant (small effect)" = "#FDAE6B",
    "None" = "#F0F0F0"
  )

  # Build the matrix of RS types and chi2 values
  rs_matrix <- matrix("Not tested", nrow = n_items, ncol = n_nodes)
  chi2_matrix <- matrix(NA, nrow = n_items, ncol = n_nodes)
  sig_matrix <- matrix(FALSE, nrow = n_items, ncol = n_nodes)

  if (!is.null(rs$item_level)) {
    has_p_adj <- "LRT_p_adj" %in% names(rs$item_level)

    for (i in 1:nrow(rs$item_level)) {
      row <- rs$item_level[i, ]
      node_col <- which(nodes == row$Node)
      item_row <- row$Item

      if (length(node_col) > 0 && item_row >= 1 && item_row <= n_items) {
        rs_matrix[item_row, node_col] <- row$RS_type
        chi2_matrix[item_row, node_col] <- row$LRT_chi2

        p_val <- if (has_p_adj) row$LRT_p_adj else row$LRT_p
        sig_matrix[item_row, node_col] <- !is.na(p_val) && p_val < rs$alpha
      }
    }
  }

  # Mark nodes where omnibus was not significant as "No RS"
  for (j in 1:n_nodes) {
    if (!isTRUE(g$RS_detected[j])) {
      rs_matrix[, j] <- "None"
    }
  }

  # ---- Plot setup ----
  # Margins: left for item labels, top for node labels + omnibus, right for legend
  left_margin <- 0.22
  right_margin <- 0.18
  top_margin <- 0.15
  bottom_margin <- 0.06
  plot_width <- 1 - left_margin - right_margin
  plot_height <- 1 - top_margin - bottom_margin

  cell_w <- plot_width / n_nodes
  cell_h <- plot_height / n_items

  # Start new page
  grid::grid.newpage()

  # Title
  grid::grid.text(
    title,
    x = grid::unit(0.5, "npc"),
    y = grid::unit(0.97, "npc"),
    gp = grid::gpar(fontface = "bold", cex = 1.1)
  )

  # ---- Draw omnibus results at top ----
  for (j in 1:n_nodes) {
    x_center <- left_margin + (j - 0.5) * cell_w

    # Node label
    grid::grid.text(
      node_labels[j],
      x = grid::unit(x_center, "npc"),
      y = grid::unit(1 - top_margin + 0.06, "npc"),
      gp = grid::gpar(cex = 0.7, fontface = "bold")
    )

    # Omnibus result
    has_adj <- "LRT_p_adj" %in% names(g)
    p_val <- if (has_adj && !is.na(g$LRT_p_adj[j])) g$LRT_p_adj[j] else g$LRT_p[j]
    rs_det <- isTRUE(g$RS_detected[j])

    omnibus_text <- if (rs_det) {
      paste0("RS: p=", formatC(p_val, format = "e", digits = 1))
    } else {
      "No RS"
    }
    omnibus_col <- if (rs_det) "#C0392B" else "#27AE60"

    grid::grid.text(
      omnibus_text,
      x = grid::unit(x_center, "npc"),
      y = grid::unit(1 - top_margin + 0.02, "npc"),
      gp = grid::gpar(cex = 0.6, col = omnibus_col, fontface = "bold")
    )
  }

  # ---- Draw heatmap cells ----
  for (i in 1:n_items) {
    # Item label on left
    y_center <- 1 - top_margin - (i - 0.5) * cell_h
    grid::grid.text(
      item_labels[i],
      x = grid::unit(left_margin - 0.02, "npc"),
      y = grid::unit(y_center, "npc"),
      just = "right",
      gp = grid::gpar(cex = 0.65)
    )

    for (j in 1:n_nodes) {
      x_center <- left_margin + (j - 0.5) * cell_w

      # Cell color
      rs_type <- rs_matrix[i, j]
      if (is.na(rs_type) || rs_type == "Not tested") {
        fill_col <- "#FFFFFF"
      } else {
        fill_col <- rs_colors[rs_type]
        if (is.na(fill_col)) fill_col <- "#F0F0F0"
      }

      # Only color significant cells
      if (!sig_matrix[i, j] && rs_type != "None" && rs_type != "Not tested") {
        # Significant at unadjusted but not adjusted: light tint
        fill_col <- "#F5F5F5"
      }

      # Draw cell
      grid::grid.rect(
        x = grid::unit(x_center, "npc"),
        y = grid::unit(y_center, "npc"),
        width = grid::unit(cell_w * 0.9, "npc"),
        height = grid::unit(cell_h * 0.85, "npc"),
        gp = grid::gpar(fill = fill_col, col = "gray70", lwd = 0.5)
      )

      # Add chi2 value or significance marker
      if (show_chi2 && !is.na(chi2_matrix[i, j])) {
        grid::grid.text(
          round(chi2_matrix[i, j], 1),
          x = grid::unit(x_center, "npc"),
          y = grid::unit(y_center, "npc"),
          gp = grid::gpar(cex = 0.5, col = if (sig_matrix[i, j]) "white" else "gray50")
        )
      } else if (sig_matrix[i, j]) {
        grid::grid.text(
          "*",
          x = grid::unit(x_center, "npc"),
          y = grid::unit(y_center, "npc"),
          gp = grid::gpar(cex = 0.9, col = "white", fontface = "bold")
        )
      }
    }
  }

  # ---- Legend ----
  legend_types <- c("Recalibration", "Reprioritization", "Both",
                    "Significant (small effect)", "None")
  legend_x_start <- 1 - right_margin + 0.02
  legend_y_start <- 0.85

  grid::grid.text(
    "RS Type",
    x = grid::unit(legend_x_start, "npc"),
    y = grid::unit(legend_y_start + 0.03, "npc"),
    just = "left",
    gp = grid::gpar(cex = 0.6, fontface = "bold")
  )

  for (k in seq_along(legend_types)) {
    y_pos <- legend_y_start - (k - 1) * 0.055

    grid::grid.rect(
      x = grid::unit(legend_x_start + 0.01, "npc"),
      y = grid::unit(y_pos, "npc"),
      width = grid::unit(0.025, "npc"),
      height = grid::unit(0.025, "npc"),
      gp = grid::gpar(fill = rs_colors[legend_types[k]], col = "gray50")
    )

    # Short labels for legend
    short_labels <- c("Recalib.", "Reprior.", "Both", "Small", "None")
    grid::grid.text(
      short_labels[k],
      x = grid::unit(legend_x_start + 0.035, "npc"),
      y = grid::unit(y_pos, "npc"),
      just = "left",
      gp = grid::gpar(cex = 0.5)
    )
  }

  # Footnote
  adj_text <- paste0(
    "Omnibus: ", rs$global_p_adjust, " adjusted",
    "  |  Item-level: ", rs$p_adjust, " adjusted",
    "  |  * p < ", rs$alpha
  )
  grid::grid.text(
    adj_text,
    x = grid::unit(0.5, "npc"),
    y = grid::unit(0.01, "npc"),
    gp = grid::gpar(cex = 0.5, col = "gray50")
  )

  invisible(rs)
}

library(testthat)
library(mirt)


# ==============================================================================
# rs_characterize()  input validation (fast, no fitting)
# ==============================================================================
test_that("rs_characterize validates its inputs", {
  # Wrong object class
  expect_error(rs_characterize(list()), "must be a longitudinal_grmtree object")

  # Bad p-adjust methods (checked before any fitting) require a real object,
  # so build a stub with the right class and info slot.
  stub <- structure(list(info = list(n_items = 4)),
                    class = c("longitudinal_grmtree", "grmtree",
                              "modelparty", "party"))
  expect_error(rs_characterize(stub, p_adjust = "invalid"),
               "valid p.adjust method")
  expect_error(rs_characterize(stub, global_p_adjust = "invalid"),
               "valid p.adjust method")
})

# ==============================================================================
# Visualization helpers  input validation (fast)
# ==============================================================================
test_that("plot_rs_tree and plot_rs_heatmap validate their inputs", {
  stub_tree <- structure(list(info = list(n_items = 4)),
                         class = c("longitudinal_grmtree", "grmtree",
                                   "modelparty", "party"))
  expect_error(plot_rs_tree(list(), list()),
               "must be a longitudinal_grmtree object")
  expect_error(plot_rs_tree(stub_tree, list()),
               "must be an rs_characterization object")
  expect_error(plot_rs_heatmap(list()),
               "must be an rs_characterization object")
})

################################################################################
##
## LONGITUDINAL GRMTree: Functions for Response Shift Detection (Production)
##
## This file contains the core functions for the Longitudinal GRMTree method:
##   - longitudinal_grmfit(): Internal fitting function for the constrained
##     two-factor longitudinal GRM (production version: builds constraints
##     directly from the known parameter structure instead of fitting an
##     unconstrained model first, making it fast enough for repeated calls
##     inside MOB).
##   - longitudinal_grmtree(): Main tree-fitting function using MOB.
##   - prepare_longitudinal_data(): Data preparation helper.
##
## Note: Phase 2 response shift characterization (rs_characterize) and its
## print method live in a separate file (rs_characterize.R).
##
## Author: Olayinka Arimoro
## Date: February 2026
##
################################################################################

#' Internal Function: Fit Constrained Longitudinal Graded Response Model
#'
#' Fits a constrained two-factor longitudinal graded response model (GRM) to
#' wide-format item response data from two time points. This is an internal
#' function called by \code{\link{longitudinal_grmtree}} during model-based
#' recursive partitioning and is not intended to be used directly.
#'
#' This production version builds the equality constraints directly from the
#' known parameter structure rather than fitting an unconstrained model first to
#' discover that structure. Because the function is called hundreds of times
#' during tree construction, avoiding the extra (unconstrained) model fit at
#' every node yields a substantial speed-up with no change in results.
#'
#' @param y A numeric matrix of item responses in wide format. The first
#'   \code{n_items} columns contain responses at Time 1 (T1) and the next
#'   \code{n_items} columns contain responses at Time 2 (T2), for a total of
#'   \code{2 * n_items} columns. Each row represents one individual. The number
#'   of columns must be even and at least 4.
#' @param x Optional predictor matrix. Handled internally by the MOB framework
#'   and not used directly in model fitting.
#' @param start Optional starting values for model parameters. Passed to
#'   \code{\link[mirt]{mirt}}.
#' @param weights Optional case weights. Not currently used.
#' @param offset Optional offset. Not currently used.
#' @param estfun Logical indicating whether to compute empirical estimating
#'   (score) functions for the MOB parameter instability tests. When
#'   \code{TRUE}, standard errors are computed during model fitting (SE = TRUE)
#'   and \code{\link[mirt]{estfun.AllModelClass}} is called to extract the score
#'   contributions for each individual. Default is \code{FALSE}.
#' @param object Logical indicating whether to return the full
#'   \code{\link[mirt]{mirt}} model object. Default is \code{FALSE}. Set to
#'   \code{TRUE} when the fitted model is needed for post-hoc analyses (e.g., in
#'   \code{\link{rs_characterize}}).
#' @param ... Additional arguments passed to \code{\link[mirt]{mirt}}.
#'
#' @return A list containing:
#'   \item{coefficients}{Item parameter estimates from
#'     \code{mirt::coef(fit, IRTpars = TRUE, simplify = TRUE)}, including
#'     discrimination (\code{a1}, \code{a2}) and threshold (\code{b1}, ...,
#'     \code{bK}) parameters for all \code{2 * n_items} items, plus latent
#'     trait means and covariance matrix.}
#'   \item{objfun}{Negative log-likelihood of the fitted model. Used by MOB
#'     to evaluate splits via partitioned log-likelihood maximization.}
#'   \item{estfun}{Empirical estimating (score) functions if \code{estfun =
#'     TRUE}; \code{NULL} otherwise. An \code{N x p} matrix where \code{N}
#'     is the number of individuals and \code{p} is the number of model
#'     parameters. Used by MOB for score-based structural change tests.}
#'   \item{object}{The full \code{SingleGroupClass} model object
#'     if \code{object = TRUE}; \code{NULL} otherwise.}
#'
#' @details ## Constrained Two-Factor Longitudinal GRM
#'
#' The function specifies a two-factor GRM where the latent factors
#' \eqn{\theta_{T1}} and \eqn{\theta_{T2}} represent the construct at Time 1 and
#' Time 2, respectively. The model structure is:
#'
#' \deqn{\theta_{T1} \sim N(0, 1)} \deqn{\theta_{T2} \sim N(\mu_{T2},
#' \sigma^2_{T2})} \deqn{Cov(\theta_{T1}, \theta_{T2}) = \sigma_{12}}
#'
#' Items 1 through \eqn{M} load on \eqn{\theta_{T1}} and items \eqn{M+1} through
#' \eqn{2M} load on \eqn{\theta_{T2}}, where \eqn{M} is the number of items per
#' time point.
#'
#' Under the null hypothesis of no response shift, item parameters are
#' constrained equal across time points. For each item \eqn{m}:
#' \itemize{
#'   \item Discrimination: \eqn{a_{1,m} = a_{2,m+M}} (the T1 item's loading
#'     on \eqn{\theta_{T1}} equals the T2 item's loading on \eqn{\theta_{T2}})
#'   \item Thresholds: \eqn{b_{k,m} = b_{k,m+M}} for all category thresholds
#'     \eqn{k = 1, \ldots, K_m}
#' }
#'
#' The number of threshold constraints per item pair is determined by the
#' minimum number of observed response categories across T1 and T2 for that item
#' (minus 1). This handles cases where sparse response categories at one time
#' point produce fewer estimated thresholds (e.g., category 5 may be empty at T1
#' but observed at T2).
#'
#' ## Constraint Syntax
#'
#' Constraints are specified using \code{mirt}'s CONSTRAIN syntax:
#' \itemize{
#'   \item Discrimination: \code{(i, a1, j, a2)} constrains item \code{i}'s
#'     \code{a1} parameter to equal item \code{j}'s \code{a2} parameter
#'   \item Thresholds: \code{(i, j, dk)} constrains the \code{dk} threshold
#'     to be equal between items \code{i} and \code{j}
#' }
#'
#' where \code{i} is the T1 item index and \code{j = i + n_items} is the
#' corresponding T2 item index.
#'
#' ## Technical Details
#'
#' The EM algorithm is run with a maximum of 1000 cycles (\code{technical =
#' list(NCYCLES = 1000)}) to accommodate the complexity of the constrained
#' two-factor model. Verbose output from \code{mirt} is suppressed. If the model
#' fails to converge or encounters an error, the function stops with an
#' informative error message.
#'
#' @seealso \code{\link{grmtree}} fits a Graded Response Model Tree,
#'   \code{\link{longitudinal_grmtree}} for the main tree-fitting function that
#'   calls this internally, \code{\link[mirt]{mirt}} for the underlying IRT
#'   estimation engine, \code{\link{rs_characterize}} for post-hoc response
#'   shift testing within terminal nodes
#'
#' @keywords internal
#' @importFrom mirt mirt mirt.model coef logLik estfun.AllModelClass
longitudinal_grmfit <- function(y, x = NULL, start = NULL, weights = NULL,
                                offset = NULL, ...,
                                estfun = FALSE, object = FALSE) {
  if (!is.matrix(y)) y <- as.matrix(y)
  n_cols <- ncol(y)
  if (n_cols < 4 || n_cols %% 2 != 0) {
    stop("Response matrix must have an even number of columns. Found: ", n_cols)
  }
  n_items <- n_cols / 2
  # ---- Detect number of thresholds PER ITEM PAIR ----
  # Items may have different observed categories at T1 vs T2
  # (e.g., category 5 may be empty at T1 but observed at T2)
  # We must constrain only the thresholds that BOTH items have.
  # Use the MINIMUM category count within each T1-T2 pair.
  n_cats_per_col <- apply(y, 2, function(col) length(unique(na.omit(col))))
  # ---- Build model specification ----
  model_str <- paste0(
    'Theta_T1 = 1-', n_items, '\n',
    'Theta_T2 = ', n_items + 1, '-', 2 * n_items, '\n',
    'COV = Theta_T1*Theta_T2\n',
    'MEAN = Theta_T2'
  )
  # ---- Build constraints per item pair ----
  constraint_parts <- character(0)
  for (i in 1:n_items) {
    j <- i + n_items
    # Constrain discrimination: a1 of T1 item = a2 of T2 item
    constraint_parts <- c(constraint_parts, paste0("(", i, ", a1, ", j, ", a2)"))
    # Constrain thresholds: use min categories across T1 and T2 for this item
    n_thresh_i <- min(n_cats_per_col[i], n_cats_per_col[j]) - 1
    if (n_thresh_i > 0) {
      for (d in 1:n_thresh_i) {
        constraint_parts <- c(constraint_parts, paste0("(", i, ", ", j, ", d", d, ")"))
      }
    }
  }
  all_constraints <- paste(constraint_parts, collapse = ", ")
  # ---- Fit constrained model (ONE model, not two) ----
  model_spec <- paste0(model_str, '\nCONSTRAIN = ', all_constraints)
  fit <- tryCatch({
    mirt::mirt(
      data = y,
      model = mirt::mirt.model(model_spec),
      itemtype = 'graded',
      SE = estfun,
      verbose = FALSE,
      technical = list(NCYCLES = 1000),
      ...
    )
  }, error = function(e) {
    stop("Longitudinal GRM fitting failed: ", e$message)
  })
  # ---- Extract coefficients ----
  coefs <- mirt::coef(fit, IRTpars = TRUE, simplify = TRUE)
  if (is.null(coefs)) stop("No coefficients returned.")
  # ---- Extract estimating functions ----
  ef <- NULL
  if (estfun) {
    ef <- tryCatch({
      mirt::estfun.AllModelClass(fit)
    }, error = function(e) {
      warning("estfun extraction failed: ", e$message)
      NULL
    })
  }
  list(
    coefficients = coefs,
    objfun = -as.numeric(mirt::logLik(fit)),
    estfun = ef,
    object = if (object) fit else NULL
  )
}

#' Fit a Longitudinal Graded Response Model Tree for Response Shift Detection
#'
#' This function implements a tree-based longitudinal graded response model
#' using model-based recursive partitioning (MOB) to detect measurement
#' heterogeneity in patient-reported outcome measures (PROMs) measured at two
#' time points. The Longitudinal GRMTree extends the cross-sectional
#' \code{\link[grmtree]{grmtree}} to the longitudinal setting by embedding
#' a constrained two-factor GRM within the MOB framework. The resulting tree
#' identifies patient subgroups where the longitudinal measurement model
#' differs, providing the foundation for subgroup-specific response shift
#' characterization via \code{\link{rs_characterize}}.
#'
#' @param formula A formula specifying the model structure with a wide-format
#'   response matrix on the left-hand side and partitioning variables on the
#'   right-hand side (e.g., \code{resp_wide ~ age + sex + comorbidity_count}).
#'   The response matrix must have \code{2 * n_items} columns: the first
#'   \code{n_items} columns are Time 1 (T1) responses and the next
#'   \code{n_items} columns are Time 2 (T2) responses for the same items.
#' @param data A data frame containing the variables in the formula. Must
#'   include the response matrix as a matrix-valued column (created via
#'   \code{data$resp_wide <- as.matrix(...)}) and all partitioning variables
#'   as separate columns.
#' @param n_items Integer specifying the number of items per time point. If
#'   \code{NULL} (default), automatically detected as half the number of
#'   columns in the response matrix. Must satisfy
#'   \code{ncol(response) == 2 * n_items}.
#' @param na.action How to handle missing values. Default is \code{na.omit},
#'   which removes rows with any missing values in the response matrix or
#'   partitioning variables.
#' @param control A list of control parameters created by
#'   \code{\link[grmtree]{grmtree.control}}. Key parameters include:
#'   \describe{
#'     \item{\code{minbucket}}{Minimum number of observations in a terminal
#'       node. Should be at least 10 times the number of free parameters per
#'       node to ensure stable estimation. For 8 items with 5 categories each,
#'       the constrained longitudinal GRM has approximately 40 item parameters
#'       plus 3 latent parameters, suggesting \code{minbucket >= 200}.}
#'     \item{\code{alpha}}{Significance level for the parameter instability
#'       tests. Default is 0.05.}
#'     \item{\code{p_adjust}}{Method for adjusting p-values across covariates
#'       at each split. Options include \code{"bonferroni"} (default, applied
#'       locally during tree construction), \code{"BH"}
#'       (Benjamini-Hochberg, applied post-hoc with pruning), and others.
#'       See \code{\link[grmtree]{grmtree.control}} for details.}
#'   }
#' @param mtry Number of variables randomly sampled as candidates at each
#'   split. If \code{NULL} (default), all variables are considered. Can be
#'   used for random forest extensions.
#' @param ... Additional arguments passed to the internal fitting function
#'   \code{\link{longitudinal_grmfit}} and ultimately to
#'   \code{\link[mirt]{mirt}}.
#'
#' @return An object of class \code{c("longitudinal_grmtree", "grmtree",
#'   "modelparty", "party")} containing the fitted tree structure. The object
#'   inherits from \code{\link[partykit]{modelparty}} and includes:
#'   \describe{
#'     \item{Tree structure}{Accessible via standard \code{partykit} methods
#'       such as \code{\link[partykit]{nodeids}},
#'       \code{\link[partykit]{data_party}}, and indexing with \code{[[}.}
#'     \item{Node models}{Each terminal node contains a fitted constrained
#'       longitudinal GRM with item parameters (discrimination and thresholds),
#'       latent trait means (\eqn{\mu_{T2}}), and latent trait covariance
#'       matrix (\eqn{\sigma^2_{T2}}, \eqn{r_{T1,T2}}).}
#'     \item{\code{info$n_items}}{Number of items per time point.}
#'     \item{\code{info$model_type}}{\code{"longitudinal_grm"}.}
#'     \item{\code{info$p_adjust}}{The p-value adjustment method used.}
#'     \item{\code{info$call}}{The original function call.}
#'   }
#'
#' @details
#' ## Overview
#'
#' The Longitudinal GRMTree is a unified framework for response shift
#' detection that operates in two phases:
#'
#' \strong{Phase 1 (this function):} Identifies patient subgroups where the
#' constrained longitudinal measurement model differs. The constrained model
#' represents the null hypothesis of no response shift (item parameters equal
#' across time). MOB tests whether this null model's parameters are stable
#' across patient covariates and recursively partitions the sample where
#' instability is detected.
#'
#' \strong{Phase 2 (\code{\link{rs_characterize}}):} Within each terminal
#' node, tests whether item parameters actually change from T1 to T2 by
#' comparing the constrained model to an unconstrained model using likelihood
#' ratio tests. This phase characterizes response shift at the item level,
#' classifying changes as recalibration, reprioritization, or both.
#'
#' ## Constrained Two-Factor Longitudinal GRM
#'
#' Let \eqn{Y_{im}} denote the response of individual \eqn{i} to item \eqn{m}
#' at time \eqn{t}. The longitudinal GRM specifies two correlated latent
#' factors:
#'
#' \deqn{\theta_{i,T1} \sim N(0, 1)}
#' \deqn{\theta_{i,T2} \sim N(\mu_{T2}, \sigma^2_{T2})}
#' \deqn{Cov(\theta_{i,T1}, \theta_{i,T2}) = \sigma_{12}}
#'
#' For item \eqn{m} at time \eqn{t}, the graded response model is:
#'
#' \deqn{P(Y_{im,t} \geq j | \theta_{i,t}) = \frac{\exp(a_{m,t}(\theta_{i,t}
#' - b_{mj,t}))}{1 + \exp(a_{m,t}(\theta_{i,t} - b_{mj,t}))}}
#'
#' where \eqn{a_{m,t}} is the discrimination parameter and \eqn{b_{mj,t}} are
#' threshold parameters for item \eqn{m} at time \eqn{t}.
#'
#' Under the no-response-shift constraint:
#' \deqn{a_{m,T1} = a_{m,T2} \quad \text{and} \quad b_{mj,T1} = b_{mj,T2}
#' \quad \forall m, j}
#'
#' This means item parameters are identical across time, so any observed
#' changes in responses are attributed to true latent change (\eqn{\mu_{T2}})
#' rather than changes in measurement properties.
#'
#' ## The Longitudinal GRMTree Algorithm
#'
#' \strong{Step 1: Global Model Estimation.} Fit the constrained longitudinal
#' GRM to all individuals at the root node, estimating item parameters
#' \eqn{\hat{\beta}} and latent parameters \eqn{(\mu_{T2}, \sigma^2_{T2},
#' \sigma_{12})} via maximum likelihood.
#'
#' \strong{Step 2: Parameter Stability Testing.} For each covariate
#' \eqn{X_p}, compute individual score function contributions
#' \eqn{s(\hat{\beta}; \mathbf{y}_i)} and test whether these scores
#' fluctuate systematically with \eqn{X_p} using structural change tests.
#' The null hypothesis is that model parameters are stable across all
#' values of \eqn{X_p}.
#'
#' \strong{Step 3: Recursive Partitioning.} If significant instability is
#' detected (after p-value adjustment):
#' \itemize{
#'   \item Select the covariate \eqn{X_p^*} with strongest instability
#'   \item Find the optimal split point \eqn{c^*} maximizing the partitioned
#'     log-likelihood
#'   \item Split the sample: \eqn{X_p^* \leq c^*} vs. \eqn{X_p^* > c^*}
#' }
#'
#' \strong{Step 4: Recursion.} Repeat Steps 1--3 within each child node
#' until no significant instability remains or the minimum node size is
#' reached.
#'
#' \strong{Step 5 (Post-hoc):} Apply \code{\link{rs_characterize}} to test
#' for response shift within each terminal node.
#'
#' ## Formal Model Structure
#'
#' The fitted Longitudinal GRMTree provides a piecewise constrained
#' longitudinal GRM:
#'
#' \deqn{P(Y_{im,t} = k | \theta_{i,t}, \mathbf{x}_i) = \sum_{b=1}^B
#' I(\mathbf{x}_i \in \mathcal{X}_b) \cdot P_b(Y_{im,t} = k | \theta_{i,t})}
#'
#' where \eqn{B} is the number of terminal nodes, \eqn{\mathcal{X}_b} is
#' the covariate subspace defining terminal node \eqn{b}, and \eqn{P_b} is
#' the node-specific constrained longitudinal GRM. Each terminal node
#' contains:
#' \itemize{
#'   \item Node-specific constrained item parameters (equal across T1 and T2)
#'   \item Node-specific latent trait parameters: \eqn{\mu_{T2,b}},
#'     \eqn{\sigma^2_{T2,b}}, \eqn{r_{T1,T2,b}}
#' }
#'
#' ## Interpretation
#'
#' The tree structure identifies patient subgroups where the constrained
#' longitudinal measurement model differs. This can reflect:
#' \itemize{
#'   \item Different item discrimination patterns across subgroups
#'   \item Different threshold locations across subgroups
#'   \item Different magnitudes of true latent change (\eqn{\mu_{T2}})
#'   \item Different test-retest correlations
#' }
#'
#' Crucially, the tree does \strong{not} directly detect response shift
#' (temporal change in item parameters within a subgroup). Response shift
#' is tested in Phase 2 using \code{\link{rs_characterize}}, which relaxes
#' the equality constraints within each terminal node. A tree with only one
#' terminal node (no split) indicates that no covariate moderates the
#' measurement model; it does \strong{not} imply the absence of response
#' shift, since \code{\link{rs_characterize}} can still detect uniform RS in
#' the unsplit root node.
#'
#' ## Relationship to Existing Methods
#'
#' The Longitudinal GRMTree extends several existing approaches:
#' \itemize{
#'   \item \strong{GRMTree} (Arimoro et al., 2026): Cross-sectional DIF
#'     detection using tree-based GRM. The longitudinal extension embeds a
#'     two-factor model instead of a single-factor model.
#'   \item \strong{LIRTree} (Ames & Leventhal, 2021): Longitudinal
#'     tree-based IRT using Rasch/2PL models. The GRM extension allows
#'     item-specific discrimination parameters, which are important for
#'     PROMs where items vary in discriminating ability.
#'   \item \strong{Oort SEM} (Oort, 2005): Response shift detection via
#'     structural equation modeling. The tree-based approach removes the
#'     requirement for pre-specified subgroups.
#' }
#'
#' ## Practical Recommendations
#'
#' \itemize{
#'   \item \strong{Minimum node size:} Use at least 10--25 times the number
#'     of free parameters per node. For \eqn{M} items with \eqn{K} response
#'     categories, the constrained model has approximately \eqn{M(K-1) + M + 3}
#'     parameters (\eqn{M} discriminations, \eqn{M(K-1)} thresholds, and 3
#'     latent parameters). For 8 items with 5 categories:
#'     \eqn{8 \times 4 + 8 + 3 = 43} parameters, suggesting
#'     \code{minbucket = 200--400}.
#'   \item \strong{Number of covariates:} Bonferroni correction becomes
#'     increasingly conservative with more covariates. Consider using
#'     \code{p_adjust = "BH"} for exploratory analyses with many covariates.
#'   \item \strong{Sample size:} With typical PROM instruments (5--15 items),
#'     a total sample of at least 500--1000 is recommended for adequate power
#'     to detect meaningful splits.
#'   \item \strong{Response matrix preparation:} Use
#'     \code{\link{prepare_longitudinal_data}} to construct the wide-format
#'     response matrix from separate T1 and T2 item columns.
#' }
#'
#' @references
#' Arimoro, O. I., Lix, L. M., Patten, S. B., Sawatzky, R., Sebille, V.,
#' Liu, J., Wiebe, S., Josephson, C. B., & Sajobi, T. T. (2025). Tree-based
#' latent variable model for assessing differential item functioning in
#' patient-reported outcome measures: a simulation study. \emph{Quality of
#' Life Research}. \doi{10.1007/s11136-025-04018-6}
#'
#' Ames, A. J., & Leventhal, B. C. (2021). Application of a longitudinal
#' IRTree model: response style changes over time. \emph{Educational and
#' Psychological Measurement}, 81(3), 561--582.
#'
#' Oort, F. J. (2005). Using structural equation modeling to detect response
#' shifts and true change. \emph{Quality of Life Research}, 14(3), 587--598.
#'
#' Samejima, F. (1969). Estimation of latent ability using a response pattern
#' of graded scores. \emph{Psychometrika Monograph Supplement}, 34, 100--114.
#'
#' Sprangers, M. A., & Schwartz, C. E. (1999). Integrating response shift
#' into health-related quality of life research. \emph{Social Science &
#' Medicine}, 48(11), 1507--1515.
#'
#' Zeileis, A., Hothorn, T., & Hornik, K. (2008). Model-based recursive
#' partitioning. \emph{Journal of Computational and Graphical Statistics},
#' 17(2), 492--514.
#'
#' @author Olayinka Imisioluwa Arimoro \email{olayinka.arimoro@ucalgary.ca},
#'   Lisa M. Lix, Tolulope T. Sajobi
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#'   # Load the synthetic longitudinal data
#'   data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Phase 1: fit the longitudinal GRM tree
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Print tree structure
#'   print(ltree)
#'
#'   # Plot threshold regions
#'   plot(ltree, type = "regions", tnex = 2L)
#'
#'   # Phase 2: characterize response shift within each subgroup
#'   rs <- rs_characterize(ltree, p_adjust = "fdr",
#'     global_p_adjust = "bonferroni")
#'   print(rs)
#'
#'
#' @seealso
#'   \code{\link{rs_characterize}} for Phase 2 response shift characterization,
#'   \code{\link{prepare_longitudinal_data}} for data preparation,
#'   \code{\link[grmtree]{grmtree}} for cross-sectional DIF detection,
#'   \code{\link[grmtree]{grmtree.control}} for control parameters,
#'   \code{\link[partykit]{mob}} for the underlying MOB framework
#'
#' @export
#' @importFrom partykit mob mob_control nodeids
#' @importFrom stats model.frame model.response na.omit p.adjust formula
longitudinal_grmtree <- function(formula, data, n_items = NULL,
                                 na.action = na.omit,
                                 control = grmtree.control(),
                                 mtry = NULL, ...) {
  if (!inherits(formula, "formula")) stop("'formula' must be a formula")
  if (!is.data.frame(data)) stop("'data' must be a data.frame")
  cl <- match.call(expand.dots = TRUE)
  mf <- model.frame(formula, data = data, na.action = na.action)
  y <- model.response(mf)
  if (!is.matrix(y)) {
    stop("Response variable must be a wide-format matrix with 2*n_items columns.")
  }
  if (is.null(n_items)) {
    if (ncol(y) %% 2 != 0) {
      stop("Odd number of columns (", ncol(y), "). Specify n_items.")
    }
    n_items <- ncol(y) / 2
    message("Auto-detected n_items = ", n_items)
  } else if (ncol(y) != 2 * n_items) {
    stop("Expected ", 2 * n_items, " columns, found ", ncol(y))
  }
  if (n_items < 2) stop("Need at least 2 items per time point.")
  # Fitting function closure
  long_grmfit_wrapper <- function(y, x = NULL, start = NULL, weights = NULL,
                                  offset = NULL, ..., estfun = FALSE,
                                  object = FALSE) {
    longitudinal_grmfit(y = y, x = x, start = start, weights = weights,
                        offset = offset, estfun = estfun, object = object, ...)
  }
  p_adjust <- if (!is.null(control$p_adjust)) control$p_adjust else "none"
  alpha <- control$alpha
  minbucket <- control$minsize
  if (p_adjust %in% c("holm", "BH", "BY", "hochberg", "hommel")) {
    initial_alpha <- control$initial_alpha
    ctrl_full <- partykit::mob_control(
      minbucket = minbucket, bonferroni = FALSE,
      alpha = initial_alpha, ytype = "matrix"
    )
    rval <- partykit::mob(
      formula = formula, data = data,
      fit = long_grmfit_wrapper,
      control = ctrl_full, na.action = na.action
    )
    if (exists(".adjust_and_prune_tree")) {
      rval <- .adjust_and_prune_tree(rval, p_adjust, alpha)
    }
  } else {
    rval <- partykit::mob(
      formula = formula, data = data,
      fit = long_grmfit_wrapper,
      control = control, na.action = na.action
    )
  }
  rval$info$p_adjust <- p_adjust
  rval$info$call <- cl
  rval$info$n_items <- n_items
  rval$info$model_type <- "longitudinal_grm"
  class(rval) <- c("longitudinal_grmtree", "grmtree", "modelparty", "party")
  return(rval)
}

#' Prepare Wide-Format Longitudinal PROM Data
#'
#' Constructs a wide-format data frame suitable for
#' \code{\link{longitudinal_grmtree}} from separate Time 1 and Time 2 item
#' response columns. The function creates a matrix-valued column containing
#' the concatenated T1 and T2 responses, along with any specified covariates.
#'
#' @param data A data frame containing item response columns for both time
#'   points and any covariates.
#' @param items_t1 Character vector of column names for Time 1 item responses,
#'   in the order they should appear in the response matrix.
#' @param items_t2 Character vector of column names for Time 2 item responses,
#'   in the same order as \code{items_t1}. Must have the same length.
#' @param covariates Optional character vector of column names for covariates
#'   to include in the output data frame (e.g., \code{c("age", "sex")}).
#' @param id Optional character string specifying the subject ID column name.
#'   If provided, the ID column is included in the output.
#'
#' @return A data frame with:
#'   \describe{
#'     \item{Covariates}{All columns specified in \code{covariates}}
#'     \item{\code{resp_wide}}{A matrix-valued column with
#'       \code{2 * length(items_t1)} columns. The first half contains T1
#'       responses (named \code{Item1_T1}, ..., \code{ItemM_T1}) and the
#'       second half contains T2 responses (named \code{Item1_T2}, ...,
#'       \code{ItemM_T2}).}
#'   }
#'   Rows with any missing values in the response matrix are removed, with
#'   a message indicating how many rows were dropped.
#'
#' @examples
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix from separate T1 and T2 columns
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Check structure
#'   str(ld$resp_wide)  # 1500 x 16 matrix
#'
#' @seealso \code{\link{longitudinal_grmtree}} for fitting the tree
#'
#' @export
#' @importFrom stats complete.cases
prepare_longitudinal_data <- function(data, items_t1, items_t2,
                                      covariates = NULL, id = NULL) {
  if (length(items_t1) != length(items_t2)) {
    stop("items_t1 and items_t2 must have the same length.")
  }
  all_cols <- c(items_t1, items_t2, covariates, id)
  missing <- setdiff(all_cols, names(data))
  if (length(missing) > 0) stop("Columns not found: ", paste(missing, collapse = ", "))
  resp_mat <- as.matrix(data[, c(items_t1, items_t2)])
  colnames(resp_mat) <- c(paste0("Item", seq_along(items_t1), "_T1"),
                          paste0("Item", seq_along(items_t2), "_T2"))
  out <- data.frame(row.names = 1:nrow(data))
  if (!is.null(covariates)) for (cov in covariates) out[[cov]] <- data[[cov]]
  if (!is.null(id)) out[[id]] <- data[[id]]
  out$resp_wide <- resp_mat
  complete <- complete.cases(resp_mat)
  if (sum(!complete) > 0) {
    message("Removed ", sum(!complete), " incomplete rows. Remaining: ", sum(complete))
    out <- out[complete, ]
  }
  return(out)
}

################################################################################
##
## Tests for generate_node_scores_dataset() (unified grmtree + longitudinal)
##
## The unified version:
##   - Signature: generate_node_scores_dataset(object, data = NULL, method)
##   - With data = NULL: returns only `node` + score column(s)
##   - With data = <df>: returns all original columns + node + score column(s)
##   - Cross-sectional -> `factor_score`; longitudinal -> `Theta_T1`, `Theta_T2`
##
################################################################################

library(testthat)
library(mirt)

# ==============================================================================
# 1. Cross-sectional grmtree
# ==============================================================================
test_that("generate_node_scores_dataset works for cross-sectional grmtree", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])
  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))

  # With data supplied: all original columns + node + factor_score
  result <- suppressMessages(
    generate_node_scores_dataset(tree, data = asti)
  )
  expect_s3_class(result, "data.frame")
  expect_true(all(c("node", "factor_score") %in% names(result)))
  expect_equal(nrow(result), nrow(asti))
  # Original covariates are preserved and in order
  expect_true("gender" %in% names(result))
  expect_equal(result$gender, asti$gender)

  # With data = NULL: minimal output (node + factor_score only)
  minimal <- suppressMessages(generate_node_scores_dataset(tree))
  expect_true(all(c("node", "factor_score") %in% names(minimal)))
  expect_equal(nrow(minimal), nrow(asti))
})

test_that("generate_node_scores_dataset accepts all scoring methods", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])
  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))

  for (m in c("EAP", "MAP", "ML", "WLE")) {
    res <- suppressMessages(
      generate_node_scores_dataset(tree, data = asti, method = m)
    )
    expect_s3_class(res, "data.frame")
    expect_true("factor_score" %in% names(res))
  }
})

# ==============================================================================
# 2. Input validation
# ==============================================================================
test_that("generate_node_scores_dataset validates its inputs", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])
  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))

  # Wrong object class
  expect_error(generate_node_scores_dataset(list()),
               "must be a grmtree or longitudinal_grmtree object")
  # Invalid scoring method
  expect_error(generate_node_scores_dataset(tree, method = "invalid"),
               "must be one of: 'EAP', 'MAP', 'ML', or 'WLE'")
})

# ==============================================================================
# 3. Longitudinal grmtree (skipped on CRAN for time)
# ==============================================================================
test_that("generate_node_scores_dataset works for longitudinal_grmtree", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("partykit")

  df  <- make_tiny_long(n = 250, n_items = 4, seed = 7)
  it1 <- paste0("I", 1:4, "_T1")
  it2 <- paste0("I", 1:4, "_T2")
  ld  <- prepare_longitudinal_data(df, items_t1 = it1, items_t2 = it2,
                                   covariates = c("grp", "age"))

  tree <- longitudinal_grmtree(resp_wide ~ grp + age, data = ld, n_items = 4,
                               control = grmtree.control(minbucket = 80))

  # With data supplied: original columns + node + Theta_T1 + Theta_T2
  result <- suppressMessages(
    generate_node_scores_dataset(tree, data = ld)
  )
  expect_s3_class(result, "data.frame")
  expect_true(all(c("node", "Theta_T1", "Theta_T2") %in% names(result)))
  expect_false("factor_score" %in% names(result))  # longitudinal, not x-sectional
  expect_equal(nrow(result), nrow(ld))

  # With data = NULL: minimal output (node + the two thetas)
  minimal <- suppressMessages(generate_node_scores_dataset(tree))
  expect_true(all(c("node", "Theta_T1", "Theta_T2") %in% names(minimal)))
})

library(testthat)
library(mirt)

# ==============================================================================
# longitudinal_grmtree()  input validation (fast)
# ==============================================================================

test_that("longitudinal_grmtree validates its inputs", {
  df <- make_tiny_long()
  it1 <- paste0("I", 1:4, "_T1")
  it2 <- paste0("I", 1:4, "_T2")
  ld  <- prepare_longitudinal_data(df, items_t1 = it1, items_t2 = it2,
                                   covariates = c("grp", "age"))

  # Not a formula
  expect_error(
    longitudinal_grmtree("resp_wide ~ grp", data = ld, n_items = 4),
    "must be a formula"
  )
  # Not a data.frame
  expect_error(
    longitudinal_grmtree(resp_wide ~ grp, data = ld$resp_wide, n_items = 4),
    "must be a data.frame"
  )
  # Wrong n_items given the matrix width
  expect_error(
    longitudinal_grmtree(resp_wide ~ grp, data = ld, n_items = 6),
    "Expected 12 columns"
  )
})

library(testthat)
library(mirt)


# ==============================================================================
# prepare_longitudinal_data()  (fast, no fitting)
# ==============================================================================

test_that("prepare_longitudinal_data builds the wide matrix correctly", {
  df <- make_tiny_long()
  it1 <- paste0("I", 1:4, "_T1")
  it2 <- paste0("I", 1:4, "_T2")

  ld <- prepare_longitudinal_data(df, items_t1 = it1, items_t2 = it2,
                                  covariates = c("grp", "age"))

  expect_s3_class(ld, "data.frame")
  expect_true("resp_wide" %in% names(ld))
  expect_true(is.matrix(ld$resp_wide))
  expect_equal(ncol(ld$resp_wide), 8)          # 2 * n_items
  expect_true(all(c("grp", "age") %in% names(ld)))
})

test_that("prepare_longitudinal_data validates inputs", {
  df <- make_tiny_long()
  it1 <- paste0("I", 1:4, "_T1")
  it2 <- paste0("I", 1:4, "_T2")

  # Unequal item vector lengths
  expect_error(
    prepare_longitudinal_data(df, items_t1 = it1[1:3], items_t2 = it2),
    "same length"
  )
  # Missing columns
  expect_error(
    prepare_longitudinal_data(df, items_t1 = c(it1, "nope"),
                              items_t2 = c(it2, "nope2")),
    "Columns not found"
  )
})

test_that("prepare_longitudinal_data drops incomplete response rows", {
  df <- make_tiny_long()
  it1 <- paste0("I", 1:4, "_T1")
  it2 <- paste0("I", 1:4, "_T2")
  df$I1_T1[1:5] <- NA

  expect_message(
    ld <- prepare_longitudinal_data(df, items_t1 = it1, items_t2 = it2),
    "incomplete rows"
  )
  expect_equal(nrow(ld), nrow(df) - 5)
})

library(testthat)
library(mirt)

# ==============================================================================
# END-TO-END smoke test (slow-ish; skipped on CRAN)
# ==============================================================================
test_that("longitudinal_grmtree + rs_characterize run end-to-end", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("partykit")

  df  <- make_tiny_long(n = 250, n_items = 4, seed = 42)
  it1 <- paste0("I", 1:4, "_T1")
  it2 <- paste0("I", 1:4, "_T2")
  ld  <- prepare_longitudinal_data(df, items_t1 = it1, items_t2 = it2,
                                   covariates = c("grp", "age"))

  # Fit tree (no true DIF -> typically no split, which is fine)
  tree <- longitudinal_grmtree(resp_wide ~ grp + age, data = ld, n_items = 4,
                               control = grmtree.control(minbucket = 80))

  expect_s3_class(tree, "longitudinal_grmtree")
  expect_true("longitudinal_grmtree" %in% class(tree))
  expect_equal(tree$info$n_items, 4)

  # Phase 2 runs even on a single (root) node
  rs <- rs_characterize(tree, verbose = FALSE)
  expect_s3_class(rs, "rs_characterization")
  expect_true(is.data.frame(rs$global))
  expect_true(all(c("Node", "LRT_chi2", "LRT_p", "RS_detected") %in%
                    names(rs$global)))

  # Extraction functions return the unique (T1) items only
  th <- threshpar_longitudinal_grmtree(tree)
  expect_s3_class(th, "data.frame")
  n_term <- length(partykit::nodeids(tree, terminal = TRUE))
  expect_equal(nrow(th), n_term * 4)          # n_items per node, not 2*n_items

  dp <- discrpar_longitudinal_grmtree(tree)
  expect_true("Discrimination" %in% names(dp))

  lp <- latentpar_longitudinal_grmtree(tree)
  expect_true(all(c("mu_T2", "sigma2_T2", "cor_T1_T2") %in% names(lp)))

  # Factor scores return two columns (Theta_T1, Theta_T2)
  fs <- fscores_longitudinal_grmtree(tree)
  first <- fs[[1]]
  expect_true(all(c("Theta_T1", "Theta_T2") %in% names(first)))
})

#' Internal Function: Apply Function to Models in Nodes
#'
#' Applies a function to the GRM models in specified nodes of a tree.
#' This is an internal helper function and not meant to be called directly.
#'
#' @param object A `grmtree` object.
#' @param node Vector of node IDs (default: all nodes).
#' @param FUN Function to apply to each model.
#' @param drop Logical indicating whether to drop list structure if length 1.
#' @param ... Additional arguments passed to FUN.
#'
#' @return List of results (or single result if drop=TRUE and length(node)=1).
#'
#' @keywords internal
apply_to_models <- function(object, node = NULL, FUN = NULL, drop = FALSE, ...) {
  # Validate inputs
  if (!inherits(object, "grmtree")) {
    stop("'object' must be a grmtree object")
  }
  
  if (is.null(node)) {
    node <- partykit::nodeids(object, terminal = FALSE)
  }
  
  if (is.null(FUN)) {
    FUN <- function(object, ...) object
  }
  
  if (!is.function(FUN)) {
    stop("'FUN' must be a function")
  }
  
  # Apply function to models
  rval <- if ("object" %in% object$info$control$terminal) {
    tryCatch(
      partykit::nodeapply(object, node, function(n) FUN(partykit::info_node(n)$object, ...)),
      error = function(e) {
        stop("Failed to apply function to nodes: ", e$message)
      }
    )
  } else {
    tryCatch(
      lapply(partykit::refit.modelparty(object, node, drop = FALSE), FUN, ...),
      error = function(e) {
        stop("Failed to refit and apply function: ", e$message)
      }
    )
  }
  
  names(rval) <- node
  
  if (drop && length(node) == 1L) {
    rval <- rval[[1L]]
  }
  
  return(rval)
}

# ------------------------------------------------------------------------------
# Helper: tiny synthetic longitudinal dataset (fast; no real data needed)
# ------------------------------------------------------------------------------
make_tiny_long <- function(n = 250, n_items = 4, seed = 1) {
  set.seed(seed)
  a  <- runif(n_items, 1.5, 2.5)
  b  <- t(replicate(n_items, sort(rnorm(4))))
  d  <- -a * b
  th1 <- rnorm(n)
  th2 <- 0.58 * th1 + sqrt(1 - 0.58^2) * rnorm(n)
  r1 <- mirt::simdata(a = matrix(a, ncol = 1), d = d,
                      Theta = matrix(th1, ncol = 1), itemtype = "graded") + 1L
  r2 <- mirt::simdata(a = matrix(a, ncol = 1), d = d,
                      Theta = matrix(th2, ncol = 1), itemtype = "graded") + 1L
  df <- data.frame(r1, r2)
  names(df) <- c(paste0("I", 1:n_items, "_T1"), paste0("I", 1:n_items, "_T2"))
  df$grp <- factor(sample(c("A", "B"), n, replace = TRUE))
  df$age <- round(rnorm(n, 60, 8))
  df
}

#' Extract Item Parameters from GRM Tree
#'
#' Extracts both discrimination parameters and average threshold parameters for each item
#' from all terminal nodes of a graded response model tree. This provides a comprehensive
#' view of item characteristics across different nodes of the tree.
#'
#' @param object A `grmtree` object.
#' @param node Optional vector of node IDs to extract from. If NULL (default),
#'   extracts from all terminal nodes.
#' @param ... Additional arguments (currently unused).
#'
#' @return A data.frame with item parameters for each item in each node, with columns:
#'   \item{Node}{Node ID}
#'   \item{Item}{Item name}
#'   \item{Discrimination}{Discrimination parameter (a1)}
#'   \item{AvgThreshold}{Average of threshold parameters}
#'   \item{Thresholds}{All threshold parameters as a list column}
#'
#' @examples
#' \donttest{
#'   library(grmtree)
#'   library(hlt)
#'   data("asti", package = "hlt")
#'   asti$resp <- data.matrix(asti[, 1:4])
#'
#'   # Fit GRM tree with gender and group as partitioning variables
#'   tree <- grmtree(resp ~ gender + group,
#'           data = asti,
#'           control = grmtree.control(minbucket = 30))
#'
#'   # Get all item parameters
#'   items <- itempar_grmtree(tree)
#'   print(items)
#' }
#'
#' @seealso
#' \code{\link{grmtree}} fits a Graded Response Model Tree,
#' \code{\link{grmforest}} for GRM Forests,
#' \code{\link{fscores_grmtree}} for computing factor scores,
#' \code{\link{threshpar_grmtree}} for extracting threshold parameters,
#' \code{\link{discrpar_grmtree}} for extracting discrimination parameters
#'
#' @export
itempar_grmtree <- function(object, node = NULL, ...) {
  # Validate input
  if (!inherits(object, "grmtree")) {
    stop("'object' must be a grmtree object")
  }

  # Get terminal nodes if not specified
  if (is.null(node)) {
    node <- partykit::nodeids(object, terminal = TRUE)
    if (length(node) == 0) {
      stop("No terminal nodes found in tree")
    }
  }

  # Validate node IDs
  all_nodes <- partykit::nodeids(object)
  invalid_nodes <- setdiff(node, all_nodes)
  if (length(invalid_nodes) > 0) {
    stop("Invalid node IDs: ", paste(invalid_nodes, collapse = ", "))
  }

  # Extract item parameters for each node
  item_list <- lapply(node, function(n) {
    # Get model from node
    model <- tryCatch(
      partykit::nodeapply(object, ids = n, FUN = function(nd) nd$info$object)[[1]],
      error = function(e) {
        stop("Failed to extract model from node ", n, ": ", e$message)
      }
    )

    # Get coefficients
    coef_model <- tryCatch(
      mirt::coef(model, IRTpars=T, simplify=TRUE),
      error = function(e) {
        stop("Failed to extract coefficients from node ", n, ": ", e$message)
      }
    )

    # Extract discrimination parameters
    if (!"a" %in% colnames(coef_model$items)) {
      stop("No discrimination parameters found in node ", n)
    }
    discr <- coef_model$items[, "a"]

    # Extract threshold parameters
    thresh_cols <- grep("^b", colnames(coef_model$items))
    if (length(thresh_cols) == 0) {
      stop("No threshold parameters found in node ", n)
    }
    thresholds <- coef_model$items[, thresh_cols, drop = FALSE]

    # Calculate average thresholds
    avg_thresh <- rowMeans(thresholds, na.rm = TRUE)

    # Create output data frame
    items <- rownames(coef_model$items)
    item_df <- data.frame(
      Node = n,
      Item = items,
      Discrimination = discr,
      AvgThreshold = avg_thresh,
      stringsAsFactors = FALSE
    )

    # Add thresholds as list column
    item_df$Thresholds <- lapply(seq_len(nrow(thresholds)), function(i) {
      unname(thresholds[i, ])
    })

    return(item_df)
  })

  # Combine all nodes' results
  items <- do.call("rbind", item_list)

  # Reset row names
  rownames(items) <- NULL

  return(items)
}

library(testthat)
library(mirt)
library(hlt)


## Test that itempar_grmtree works with valid inputs
test_that("itempar_grmtree works with valid inputs", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  ## ASTI data
  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))

  # Default usage
  expect_silent(items <- itempar_grmtree(tree))
  expect_s3_class(items, "data.frame")
  expect_true(nrow(items) > 0)
  expect_equal(names(items),
               c("Node", "Item", "Discrimination", "AvgThreshold", "Thresholds"))

  # Check parameter values
  expect_true(all(items$Discrimination > 0))
  expect_true(is.numeric(items$AvgThreshold)) # Thresholds typically can be positive or negative in GRM

  # Check thresholds list column
  expect_true(is.list(items$Thresholds))
  expect_equal(length(items$Thresholds[[1]]), 2) # n_categories - 1 thresholds

  # Specific nodes
  nodes <- partykit::nodeids(tree, terminal = TRUE)
  if (length(nodes) > 1) {
    expect_silent(items_sub <- itempar_grmtree(tree, node = nodes[1:2]))
    expect_equal(unique(items_sub$Node), nodes[1:2])
  }
})


## Test that itempar_grmtree handles errors appropriately
test_that("itempar_grmtree handles errors appropriately", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))

  # Invalid object
  expect_error(itempar_grmtree(list()), "must be a grmtree object")

  # Invalid nodes
  nodes <- partykit::nodeids(tree, terminal = TRUE)
  expect_error(itempar_grmtree(tree, node = 999), "Invalid node IDs")

  # Create a tree with a single non-terminal node
  bad_tree <- tree
  bad_tree$node <- list(
    id = 1L,
    split = NULL,
    kids = NULL,
    surrogates = NULL,
    info = list()
  )
  class(bad_tree$node) <- "partynode"
  expect_error(itempar_grmtree(bad_tree), "No discrimination parameters found in node")

})

## Test that item parameters are consistent with thresholds
test_that("item parameters are consistent with thresholds", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))
  items <- itempar_grmtree(tree)
  thresholds <- threshpar_grmtree(tree)

  # Check average thresholds match
  for (i in seq_len(nrow(items))) {
    node <- items$Node[i]
    item <- items$Item[i]
    calc_avg <- mean(unlist(items$Thresholds[i]))
    expect_equal(items$AvgThreshold[i], calc_avg)

    # Cross-check with threshpar output
    thresh_sub <- thresholds[thresholds$Node == node & thresholds$Item == item, ]
    thresh_values <- unlist(thresh_sub[, grep("^b", names(thresh_sub))])
    expect_equal(mean(thresh_values), items$AvgThreshold[i], tolerance = 1e-6)
  }
})

#' Plot Variable Importance
#'
#' Creates a bar plot of variable importance scores with options for both
#' ggplot2 and base R graphics.
#'
#' @param x A `varimp` object from `varimp()`.
#' @param top_n Number of top variables to display (NULL for all).
#' @param use_ggplot Logical indicating whether to use ggplot2 (if available).
#' @param ... Additional arguments passed to plotting functions.
#'
#' @return Invisibly returns the input object.
#'
#' @examples
#' \donttest{
#'  library(grmtree)
#'  library(hlt)
#'  data("asti", package = "hlt")
#'  asti$resp <- data.matrix(asti[, 1:4])
#'
#'  # Fit forest with default parameters
#'  forest <- grmforest(resp ~ gender + group, data = asti)
#'  imp <- varimp(forest)
#'  plot(imp)
#'  plot(imp, top_n = 1) ## select top 1 importance variable
#'  plot(imp, use_ggplot = FALSE) # Use base R graphics
#' }
#'
#' @seealso \code{\link{varimp}} calculates the variable importance for GRM
#' Forest, \code{\link{grmforest}} for GRM Forests,
#' \code{\link{grmforest.control}} creates a control object for `grmforest`,
#' \code{\link{plot.grmtree}} creates plot for the `grmtree` object
#'
#' @export
#' @importFrom graphics barplot par text abline
#' @importFrom rlang .data

plot.varimp <- function(x, top_n = NULL, use_ggplot = TRUE, ...) {
  if (!inherits(x, "varimp")) {
    stop("'x' must be a varimp object from varimp()")
  }

  # Convert to named numeric vector preserving class
  x_vec <- unclass(x)

  # Handle zero-length case
  if (length(x_vec) == 0) {
    stop("No importance values to plot")
  }

  # Handle top_n selection
  if (!is.null(top_n)) {
    if (!is.numeric(top_n) || length(top_n) != 1 || top_n < 1) {
      stop("'top_n' must be NULL or a positive integer")
    }
    top_n <- min(top_n, length(x_vec))
    x_vec <- x_vec[seq_len(top_n)]
  }

  # Calculate smart xlim that works for all cases
  x_range <- range(x_vec)
  if (all(x_vec >= 0)) {
    # All positive values - extend upper limit only
    xlim <- c(0, x_range[2] * 1.1)
  } else if (all(x_vec <= 0)) {
    # All negative values - extend lower limit only
    xlim <- c(x_range[1] * 1.1, 0)
  } else {
    # Mixed positive and negative
    xlim <- x_range * 1.1
  }

  if (use_ggplot) {
    # ggplot2 version
    imp_data <- data.frame(
      Variable = factor(names(x_vec), levels = names(x_vec)[order(x_vec)]),
      Importance = as.numeric(x_vec)
    )

    p <- ggplot2::ggplot(imp_data, ggplot2::aes(x = .data$Importance,
                                                y = stats::reorder(.data$Variable, .data$Importance))) +
      ggplot2::geom_col(fill = ifelse(imp_data$Importance >= 0, "skyblue", "salmon"),
                        width = 0.7, alpha = 0.8) +
      ggplot2::labs(
        title = "Variable Importance",
        x = "Importance Score (log-likelihood difference)",
        y = NULL
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        plot.title = ggplot2::element_text(hjust = 0.5, face = "bold"),
        panel.grid.major.y = ggplot2::element_blank()
      ) +
      ggplot2::xlim(xlim)  # Use calculated xlim

    if (length(x_vec) <= 20) {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = round(.data$Importance, 2)),
          hjust = ifelse(imp_data$Importance >= 0, -0.2, 1.2),
          size = 3
        )
    }

    print(p)

  } else {
    # Base R version
    op <- graphics::par(mar = c(5, max(8, max(nchar(names(x_vec))) / 1.5), 4, 2) + 0.1)
    on.exit(graphics::par(op))

    colors <- ifelse(x_vec >= 0, "skyblue", "salmon")

    bp <- graphics::barplot(
      as.numeric(x_vec),
      names.arg = names(x_vec),
      horiz = TRUE,
      las = 1,
      col = colors,
      main = "Variable Importance",
      xlab = "Importance Score (log-likelihood difference)",
      xlim = xlim,  # Use calculated xlim
      ...
    )

    graphics::text(
      x = ifelse(x_vec >= 0, x_vec, x_vec) + (ifelse(x_vec >= 0, 1, -1) * 0.02 * diff(x_range)),
      y = bp,
      labels = round(x_vec, 2),
      pos = ifelse(x_vec >= 0, 4, 2),
      cex = 0.8
    )

    graphics::abline(v = 0, lty = 2, col = "gray")
  }

  invisible(x)
}

# Register the method if not already registered
if (!"plot.varimp" %in% methods("plot")) {
  registerS3method("plot", "varimp", plot.varimp)
}

################################################################################
##
## rs_characterize(): Post-hoc Response Shift Characterization
##
## After fitting longitudinal_grmtree(), this function tests for actual
## response shift WITHIN each terminal node by comparing the constrained
## model (item parameters equal across time) to an unconstrained model
## (item parameters free across time).
##
## The tree splits identify subgroups with DIFFERENT measurement models
## (analogous to DIF). This function then asks: within each subgroup,
## do item parameters CHANGE over time (i.e., response shift)?
##
## Author: Olayinka Arimoro
## Date: February 2026
##
################################################################################

#' Characterize Response Shift Within Terminal Nodes of a Longitudinal GRM Tree
#'
#' After fitting a \code{\link{longitudinal_grmtree}} (Phase 1), this function
#' performs Phase 2: post-hoc response shift (RS) characterization within each
#' terminal node. For each subgroup identified by the tree, it compares the
#' constrained model (no RS: item parameters equal across time) to an
#' unconstrained model (item parameters free across time) using likelihood
#' ratio tests (LRTs), then identifies which specific items exhibit response
#' shift and classifies the type of shift.
#'
#' @param object A \code{longitudinal_grmtree} object fitted by
#'   \code{\link{longitudinal_grmtree}}.
#' @param node Optional integer vector of terminal node IDs to analyze. If
#'   \code{NULL} (default), all terminal nodes are analyzed.
#' @param item_level Logical. If \code{TRUE} (default), perform item-level RS
#'   testing by sequentially freeing one item at a time from the constrained
#'   model. Only performed for nodes where the omnibus test is significant.
#' @param alpha Numeric significance level for RS tests. Default is 0.05.
#'   Used for both omnibus and item-level significance decisions (after any
#'   p-value adjustments).
#' @param p_adjust Character string specifying the method for adjusting
#'   item-level p-values \strong{within each node}. Controls the error rate
#'   when testing multiple items for RS within a single subgroup. Options:
#'   \code{"bonferroni"} (default), \code{"holm"}, \code{"BH"}
#'   (Benjamini-Hochberg), \code{"BY"}, \code{"fdr"}, \code{"hochberg"},
#'   \code{"hommel"}, or \code{"none"}. See \code{\link[stats]{p.adjust}}.
#' @param global_p_adjust Character string specifying the method for adjusting
#'   omnibus RS p-values \strong{across terminal nodes}. Controls the
#'   family-wise error rate when testing RS in multiple subgroups. Options
#'   are the same as \code{p_adjust}. Default is \code{"bonferroni"}.
#'   The \code{RS_detected} column in the global results is updated based
#'   on adjusted p-values. Only applied when there are 2 or more nodes.
#' @param rs_threshold Numeric threshold for classifying the magnitude of
#'   parameter change in RS type determination, on the IRT parameter scale.
#'   Default is 0.3. A discrimination difference exceeding this threshold
#'   indicates reprioritization; a maximum threshold difference exceeding
#'   this indicates recalibration. Larger values produce more conservative
#'   classifications (fewer items classified as showing meaningful RS).
#'   Items that are statistically significant but below this threshold are
#'   classified as \code{"Significant (small effect)"}.
#' @param verbose Logical. If \code{TRUE} (default), print progress messages
#'   including log-likelihoods, LRT statistics, and RS classifications for
#'   each node during processing.
#' @param ... Additional arguments passed to \code{\link[mirt]{mirt}} during
#'   model fitting.
#'
#' @return A list of class \code{"rs_characterization"} containing:
#'   \describe{
#'     \item{\code{global}}{A \code{data.frame} with one row per terminal node
#'       and columns:
#'       \describe{
#'         \item{\code{Node}}{Terminal node ID}
#'         \item{\code{n}}{Sample size in the node}
#'         \item{\code{LL_constrained}}{Log-likelihood of the constrained
#'           (no-RS) model}
#'         \item{\code{LL_unconstrained}}{Log-likelihood of the unconstrained
#'           (RS-allowed) model}
#'         \item{\code{LRT_chi2}}{Likelihood ratio test statistic:
#'           \eqn{-2(LL_{con} - LL_{uncon})}}
#'         \item{\code{LRT_df}}{Degrees of freedom (number of freed
#'           constraints)}
#'         \item{\code{LRT_p}}{Raw p-value from chi-squared distribution}
#'         \item{\code{LRT_p_adj}}{Adjusted p-value (present only if
#'           \code{global_p_adjust != "none"} and there are 2+ nodes)}
#'         \item{\code{RS_detected}}{Logical: whether RS is detected at
#'           the specified \code{alpha} level (uses adjusted p if available)}
#'         \item{\code{pseudo_R2}}{McFadden's pseudo R-squared:
#'           \eqn{1 - LL_{uncon}/LL_{con}}}
#'         \item{\code{AIC_diff}}{AIC difference:
#'           \eqn{AIC_{con} - AIC_{uncon}}. Positive values indicate the
#'           unconstrained model fits better after parsimony penalty.}
#'         \item{\code{mu_T2}}{Latent mean shift at T2 from the constrained
#'           model. Positive = improvement, negative = decline.}
#'         \item{\code{sigma2_T2}}{Latent variance at T2}
#'         \item{\code{cor_T1_T2}}{Correlation between latent traits at T1
#'           and T2, reflecting stability of individual differences}
#'         \item{\code{converged_constrained}}{Logical: whether the
#'           constrained model converged}
#'         \item{\code{converged_unconstrained}}{Logical: whether the
#'           unconstrained model converged}
#'       }
#'     }
#'     \item{\code{item_level}}{A \code{data.frame} with item-level RS test
#'       results (one row per item per node where omnibus RS was detected),
#'       with columns:
#'       \describe{
#'         \item{\code{Node}}{Terminal node ID}
#'         \item{\code{Item}}{Item number (1 to \code{n_items})}
#'         \item{\code{LRT_chi2}}{LRT statistic for freeing this item}
#'         \item{\code{LRT_df}}{Degrees of freedom (1 discrimination +
#'           thresholds for this item)}
#'         \item{\code{LRT_p}}{Raw p-value}
#'         \item{\code{LRT_p_adj}}{Adjusted p-value (if \code{p_adjust !=
#'           "none"})}
#'         \item{\code{RS_type}}{Classification: \code{"Recalibration"},
#'           \code{"Reprioritization"}, \code{"Both"},
#'           \code{"Significant (small effect)"}, or \code{"None"}}
#'       }
#'       \code{NULL} if no nodes showed significant omnibus RS or
#'       \code{item_level = FALSE}.
#'     }
#'     \item{\code{parameters}}{A named list (one element per node) of lists
#'       containing \code{constrained} and \code{unconstrained} item parameter
#'       matrices (IRT parameterization with discrimination \code{a1}/\code{a2}
#'       and thresholds \code{b1}, ..., \code{bK}).}
#'     \item{\code{n_items}}{Number of items per time point.}
#'     \item{\code{alpha}}{Significance level used.}
#'     \item{\code{p_adjust}}{Item-level p-value adjustment method.}
#'     \item{\code{global_p_adjust}}{Omnibus p-value adjustment method.}
#'     \item{\code{rs_threshold}}{RS classification threshold.}
#'   }
#'
#' @details
#' ## The Two-Phase Framework
#'
#' The Longitudinal GRMTree operates as an integrated two-phase method:
#'
#' \strong{Phase 1 (\code{\link{longitudinal_grmtree}}):} Embeds a constrained
#' longitudinal GRM (item parameters equal across T1 and T2) within
#' model-based recursive partitioning. The tree identifies patient subgroups
#' requiring different constrained measurement models. This ensures that
#' subsequent RS testing is conducted within measurement-homogeneous groups.
#'
#' \strong{Phase 2 (this function):} Within each terminal node, relaxes the
#' equality constraint and tests whether item parameters actually change over
#' time. This separates between-group measurement heterogeneity (captured by
#' the tree) from within-group temporal change (response shift).
#'
#' ## Omnibus RS Test
#'
#' For each terminal node, the function fits two models to the node's data:
#' \enumerate{
#'   \item \strong{Constrained model:} Item parameters are equal across T1
#'     and T2 (no response shift). This model has the same structure as the
#'     model fitted during Phase 1.
#'   \item \strong{Unconstrained model:} Item parameters are free to differ
#'     between T1 and T2 (response shift allowed). All equality constraints
#'     on discrimination and threshold parameters are removed.
#' }
#'
#' The likelihood ratio test statistic is:
#' \deqn{\chi^2 = -2(LL_{constrained} - LL_{unconstrained})}
#'
#' with degrees of freedom equal to the number of freed constraints
#' (typically \eqn{M \times K} for \eqn{M} items with \eqn{K-1} thresholds
#' each, plus \eqn{M} discrimination parameters).
#'
#' ## Item-Level RS Testing
#'
#' When the omnibus test is significant, each item is tested individually by
#' freeing only that item's constraints while keeping all other items
#' constrained. The LRT compares the fully constrained model to the partially
#' constrained model with one item freed. This identifies which specific
#' items contribute to the overall response shift.
#'
#' ## RS Type Classification
#'
#' For items showing statistically significant RS (after p-value adjustment),
#' the type is classified by examining the freed item's T1 vs T2 parameters
#' from the partially constrained model:
#'
#' \describe{
#'   \item{\strong{Recalibration}}{Maximum absolute threshold difference
#'     exceeds \code{rs_threshold}. Interpretation: the patient's internal
#'     standards for endorsing response categories changed over time
#'     (Sprangers & Schwartz, 1999).}
#'   \item{\strong{Reprioritization}}{Absolute discrimination difference
#'     exceeds \code{rs_threshold}. Interpretation: the item's relationship
#'     to the underlying latent construct changed over time --- the item
#'     became more or less central to the construct.}
#'   \item{\strong{Both}}{Both discrimination and threshold changes exceed
#'     \code{rs_threshold}.}
#'   \item{\strong{Significant (small effect)}}{The LRT is significant but
#'     neither parameter change exceeds \code{rs_threshold}. The RS is
#'     statistically detectable but small in magnitude.}
#' }
#'
#' ## Hierarchical P-Value Correction
#'
#' The function implements a two-level correction strategy:
#' \enumerate{
#'   \item \strong{Across nodes (omnibus):} \code{global_p_adjust} controls
#'     the family-wise error rate for the question "in which subgroups does
#'     RS occur?" Default is Bonferroni.
#'   \item \strong{Within nodes (item-level):} \code{p_adjust} controls the
#'     error rate for the question "which items exhibit RS within a given
#'     subgroup?" Default is Bonferroni; FDR (Benjamini-Hochberg) is
#'     recommended for exploratory analyses.
#' }
#'
#' ## Effect Size Measures
#'
#' \describe{
#'   \item{McFadden's pseudo R-squared}{\eqn{1 - LL_{uncon}/LL_{con}}.
#'     Inherently small (~0.02) in IRT models because item parameters
#'     account for only a fraction of the total likelihood. Values should
#'     be compared across nodes rather than interpreted in absolute terms.}
#'   \item{AIC difference}{\eqn{AIC_{con} - AIC_{uncon}}. Positive values
#'     indicate the unconstrained model fits better even after penalizing
#'     for additional parameters. Negative values suggest the constrained
#'     model is preferred (response shift is not substantial enough to
#'     justify the additional parameters).}
#' }
#'
#' @references
#' Sprangers, M. A., & Schwartz, C. E. (1999). Integrating response shift
#' into health-related quality of life research. \emph{Social Science &
#' Medicine}, 48(11), 1507--1515.
#'
#' Benjamini, Y., & Hochberg, Y. (1995). Controlling the false discovery
#' rate: a practical and powerful approach to multiple testing. \emph{Journal
#' of the Royal Statistical Society: Series B}, 57(1), 289--300.
#'
#' @author Olayinka Imisioluwa Arimoro \email{olayinka.arimoro@ucalgary.ca},
#'   Lisa M. Lix, Tolulope T. Sajobi
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # After fitting the tree (Phase 1)
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Phase 2: characterize response shift within each subgroup
#'   rs <- rs_characterize(ltree, p_adjust = "fdr",
#'     global_p_adjust = "bonferroni")
#'   print(rs)
#'
#'   # Phase 2: Characterize RS
#'   # Default: Bonferroni at both levels, threshold = 0.3
#'   rs <- rs_characterize(ltree)
#'
#'   # Recommended: FDR for item-level, Bonferroni for omnibus
#'   rs <- rs_characterize(ltree,
#'     p_adjust = "fdr",
#'     global_p_adjust = "bonferroni",
#'     rs_threshold = 0.3
#'   )
#'
#'   # No omnibus correction (each node tested independently)
#'   rs <- rs_characterize(ltree, global_p_adjust = "none")
#'
#'   # More conservative RS classification
#'   rs <- rs_characterize(ltree, rs_threshold = 0.5)
#'
#'   # View results
#'   print(rs)
#'
#'   # Access components
#'   rs$global         # Omnibus results per node
#'   rs$item_level     # Item-level results
#'   rs$parameters     # Raw parameter matrices
#'
#' @seealso
#'   \code{\link{longitudinal_grmtree}} for Phase 1 (tree fitting),
#'   \code{\link{print.rs_characterization}} for the print method,
#'   \code{\link{prepare_longitudinal_data}} for data preparation,
#'   \code{\link[stats]{p.adjust}} for p-value adjustment methods
#'
#' @export
#' @importFrom mirt mirt mirt.model coef logLik extract.mirt
#' @importFrom partykit nodeids data_party
#' @importFrom stats pchisq na.omit p.adjust formula AIC

rs_characterize <- function(object, node = NULL,
                            item_level = TRUE, alpha = 0.05,
                            p_adjust = "bonferroni",
                            global_p_adjust = "bonferroni",
                            rs_threshold = 0.3,
                            verbose = TRUE, ...) {

  # ---- Input validation ----
  if (!inherits(object, "longitudinal_grmtree")) {
    stop("'object' must be a longitudinal_grmtree object")
  }
  valid_methods <- c("none", "bonferroni", "holm", "BH", "BY", "fdr",
                     "hochberg", "hommel")
  if (!p_adjust %in% valid_methods) {
    stop("'p_adjust' must be a valid p.adjust method or 'none'")
  }
  if (!global_p_adjust %in% valid_methods) {
    stop("'global_p_adjust' must be a valid p.adjust method or 'none'")
  }

  n_items <- object$info$n_items
  if (is.null(n_items)) {
    stop("Tree object does not contain n_items. Was it fit with longitudinal_grmtree()?")
  }

  # Get terminal node IDs
  if (is.null(node)) {
    node <- partykit::nodeids(object, terminal = TRUE)
  }

  # ---- Identify the response variable name from the formula ----
  tree_formula <- object$info$call$formula
  if (is.null(tree_formula)) {
    tree_formula <- formula(object)
  }
  resp_name <- as.character(tree_formula[[2]])  # LHS of formula

  # ---- Build model specification strings ----
  model_str_base <- paste0(
    'Theta_T1 = 1-', n_items, '\n',
    'Theta_T2 = ', n_items + 1, '-', 2 * n_items, '\n',
    'COV = Theta_T1*Theta_T2\n',
    'MEAN = Theta_T2'
  )

  # ---- Process each terminal node ----
  global_results <- list()
  item_results <- list()
  param_list <- list()

  for (nd in node) {

    if (verbose) message("Processing node ", nd, "...")

    # ---- Extract data using data_party() ----
    node_data <- tryCatch(
      partykit::data_party(object, id = nd),
      error = function(e) {
        warning("Failed to extract data for node ", nd, ": ", e$message)
        NULL
      }
    )

    if (is.null(node_data) || nrow(node_data) == 0) {
      global_results[[as.character(nd)]] <- data.frame(
        Node = nd, n = 0,
        LL_constrained = NA, LL_unconstrained = NA,
        LRT_chi2 = NA, LRT_df = NA, LRT_p = NA,
        RS_detected = NA, pseudo_R2 = NA, AIC_diff = NA,
        mu_T2 = NA, sigma2_T2 = NA, cor_T1_T2 = NA,
        converged_constrained = NA, converged_unconstrained = NA,
        stringsAsFactors = FALSE
      )
      next
    }

    # Extract the response matrix by name (e.g., "resp_wide")
    y_node <- node_data[[resp_name]]
    if (!is.matrix(y_node)) y_node <- as.matrix(y_node)
    n_node <- nrow(y_node)

    if (verbose) message("  n = ", n_node)

    # ---- Detect thresholds for this subgroup ----
    n_cats_per_col <- apply(y_node, 2, function(col) length(unique(na.omit(col))))

    # ---- Fit CONSTRAINED model (no RS) ----
    constraint_parts <- character(0)
    for (i in 1:n_items) {
      j <- i + n_items
      constraint_parts <- c(constraint_parts, paste0("(", i, ", a1, ", j, ", a2)"))
      n_thresh_i <- min(n_cats_per_col[i], n_cats_per_col[j]) - 1
      if (n_thresh_i > 0) {
        for (d in 1:n_thresh_i) {
          constraint_parts <- c(constraint_parts, paste0("(", i, ", ", j, ", d", d, ")"))
        }
      }
    }
    all_constraints <- paste(constraint_parts, collapse = ", ")
    model_constrained_str <- paste0(model_str_base, '\nCONSTRAIN = ', all_constraints)

    fit_constrained <- tryCatch({
      mirt::mirt(
        data = y_node,
        model = mirt::mirt.model(model_constrained_str),
        itemtype = 'graded',
        SE = FALSE,
        verbose = FALSE,
        technical = list(NCYCLES = 1000)
      )
    }, error = function(e) {
      warning("Constrained model failed in node ", nd, ": ", e$message)
      NULL
    })

    # ---- Fit UNCONSTRAINED model (RS allowed) ----
    fit_unconstrained <- tryCatch({
      mirt::mirt(
        data = y_node,
        model = mirt::mirt.model(model_str_base),
        itemtype = 'graded',
        SE = FALSE,
        verbose = FALSE,
        technical = list(NCYCLES = 1000)
      )
    }, error = function(e) {
      warning("Unconstrained model failed in node ", nd, ": ", e$message)
      NULL
    })

    if (is.null(fit_constrained) || is.null(fit_unconstrained)) {
      global_results[[as.character(nd)]] <- data.frame(
        Node = nd, n = n_node,
        LL_constrained = NA, LL_unconstrained = NA,
        LRT_chi2 = NA, LRT_df = NA, LRT_p = NA,
        RS_detected = NA, pseudo_R2 = NA, AIC_diff = NA,
        mu_T2 = NA, sigma2_T2 = NA, cor_T1_T2 = NA,
        converged_constrained = NA, converged_unconstrained = NA,
        stringsAsFactors = FALSE
      )
      next
    }

    # ---- Omnibus LRT: Constrained vs Unconstrained ----
    ll_con <- as.numeric(logLik(fit_constrained))
    ll_uncon <- as.numeric(logLik(fit_unconstrained))
    n_constraints <- length(constraint_parts)
    lrt_chi2 <- -2 * (ll_con - ll_uncon)
    lrt_df <- n_constraints
    lrt_p <- pchisq(lrt_chi2, df = lrt_df, lower.tail = FALSE)

    # Effect sizes for omnibus RS test
    pseudo_r2 <- 1 - (ll_uncon / ll_con)  # McFadden's pseudo R²
    # AIC = -2*LL + 2*k; compute from extract.mirt to get npar safely
    npar_con <- mirt::extract.mirt(fit_constrained, 'nest')
    npar_uncon <- mirt::extract.mirt(fit_unconstrained, 'nest')
    aic_con <- -2 * ll_con + 2 * npar_con
    aic_uncon <- -2 * ll_uncon + 2 * npar_uncon
    aic_diff <- aic_con - aic_uncon  # Positive = unconstrained fits better

    # Extract latent trait parameters from constrained model
    coefs_con <- mirt::coef(fit_constrained, simplify = TRUE)
    mu_T2 <- coefs_con$means[2]
    cov_mat <- coefs_con$cov
    sigma2_T2 <- cov_mat[2, 2]
    cor_T1_T2 <- cov_mat[1, 2] / sqrt(cov_mat[1, 1] * cov_mat[2, 2])

    # Convergence check
    conv_con <- fit_constrained@OptimInfo$converged
    conv_uncon <- fit_unconstrained@OptimInfo$converged

    # Check for extreme parameters (potential estimation problems)
    coefs_uncon_items <- mirt::coef(fit_unconstrained, IRTpars = TRUE, simplify = TRUE)$items
    extreme_discr <- any(abs(coefs_uncon_items[, grep("^a", colnames(coefs_uncon_items))]) > 10,
                         na.rm = TRUE)
    if (extreme_discr && verbose) {
      warning("Node ", nd, ": Extreme discrimination parameters detected (>10). ",
              "Check for sparse categories or separation.")
    }

    global_results[[as.character(nd)]] <- data.frame(
      Node = nd,
      n = n_node,
      LL_constrained = round(ll_con, 2),
      LL_unconstrained = round(ll_uncon, 2),
      LRT_chi2 = round(lrt_chi2, 3),
      LRT_df = lrt_df,
      LRT_p = lrt_p,
      RS_detected = lrt_p < alpha,
      pseudo_R2 = round(pseudo_r2, 4),
      AIC_diff = round(aic_diff, 2),
      mu_T2 = round(mu_T2, 3),
      sigma2_T2 = round(sigma2_T2, 3),
      cor_T1_T2 = round(cor_T1_T2, 3),
      converged_constrained = conv_con,
      converged_unconstrained = conv_uncon,
      stringsAsFactors = FALSE
    )

    if (verbose) {
      message("  Constrained LL = ", round(ll_con, 2),
              ", Unconstrained LL = ", round(ll_uncon, 2))
      message("  LRT chi2 = ", round(lrt_chi2, 3),
              ", df = ", lrt_df,
              ", p = ", formatC(lrt_p, format = "e", digits = 3))
      message("  RS detected: ", lrt_p < alpha,
              "  (pseudo R2 = ", round(pseudo_r2, 4),
              ", AIC diff = ", round(aic_diff, 1), ")")
      message("  mu_T2 = ", round(mu_T2, 3),
              ", cor(T1,T2) = ", round(cor_T1_T2, 3))
    }

    # ---- Store parameters for comparison ----
    coefs_con_items <- mirt::coef(fit_constrained, IRTpars = TRUE, simplify = TRUE)$items

    param_list[[as.character(nd)]] <- list(
      constrained = coefs_con_items,
      unconstrained = coefs_uncon_items
    )

    # ---- Item-level RS testing ----
    if (item_level && lrt_p < alpha) {

      if (verbose) message("  Performing item-level RS testing...")

      item_rs <- lapply(1:n_items, function(m) {

        # Free item m: remove its constraints from the list
        j <- m + n_items

        # Build constraints excluding item m
        partial_parts <- character(0)
        for (i in 1:n_items) {
          if (i == m) next  # Skip this item
          jj <- i + n_items
          partial_parts <- c(partial_parts, paste0("(", i, ", a1, ", jj, ", a2)"))
          n_thresh_i <- min(n_cats_per_col[i], n_cats_per_col[jj]) - 1
          if (n_thresh_i > 0) {
            for (d in 1:n_thresh_i) {
              partial_parts <- c(partial_parts, paste0("(", i, ", ", jj, ", d", d, ")"))
            }
          }
        }

        if (length(partial_parts) == 0) return(NULL)

        partial_constraints <- paste(partial_parts, collapse = ", ")
        model_partial_str <- paste0(model_str_base, '\nCONSTRAIN = ', partial_constraints)

        fit_partial <- tryCatch({
          mirt::mirt(
            data = y_node,
            model = mirt::mirt.model(model_partial_str),
            itemtype = 'graded',
            SE = FALSE,
            verbose = FALSE,
            technical = list(NCYCLES = 1000)
          )
        }, error = function(e) {
          warning("Partial model (free item ", m, ") failed in node ", nd)
          NULL
        })

        if (is.null(fit_partial)) {
          return(data.frame(
            Node = nd, Item = m,
            LRT_chi2 = NA, LRT_df = NA, LRT_p = NA,
            RS_type = NA,
            stringsAsFactors = FALSE
          ))
        }

        # LRT: constrained (all equal) vs partial (item m free)
        ll_partial <- as.numeric(logLik(fit_partial))
        n_freed <- 1 + min(n_cats_per_col[m], n_cats_per_col[j]) - 1
        # freed: 1 discrimination + thresholds for item m
        item_chi2 <- -2 * (ll_con - ll_partial)
        item_p <- pchisq(item_chi2, df = n_freed, lower.tail = FALSE)

        # Determine RS type by comparing T1 vs T2 parameters
        rs_type <- "None"
        if (item_p < alpha) {
          coefs_partial <- mirt::coef(fit_partial, IRTpars = TRUE, simplify = TRUE)$items

          # Compare discrimination for item m
          a_t1 <- coefs_partial[m, "a1"]       # T1 item discrimination
          a_t2 <- coefs_partial[j, "a2"]       # T2 item discrimination
          discr_diff <- abs(a_t1 - a_t2)

          # Compare thresholds for item m
          d_names_t1 <- grep("^b\\d+$", colnames(coefs_partial), value = TRUE)
          thresh_t1 <- coefs_partial[m, d_names_t1]
          thresh_t2 <- coefs_partial[j, d_names_t1]
          thresh_diff <- max(abs(thresh_t1 - thresh_t2), na.rm = TRUE)

          # Classify RS type based on parameter differences
          has_reprioritization <- discr_diff > rs_threshold
          has_recalibration <- thresh_diff > rs_threshold

          if (has_reprioritization && has_recalibration) {
            rs_type <- "Both"
          } else if (has_reprioritization) {
            rs_type <- "Reprioritization"
          } else if (has_recalibration) {
            rs_type <- "Recalibration"
          } else {
            rs_type <- "Significant (small effect)"
          }
        }

        data.frame(
          Node = nd,
          Item = m,
          LRT_chi2 = round(item_chi2, 3),
          LRT_df = n_freed,
          LRT_p = round(item_p, 6),
          RS_type = rs_type,
          stringsAsFactors = FALSE
        )
      })

      item_results[[as.character(nd)]] <- do.call(rbind, item_rs)

      if (verbose) {
        item_df <- item_results[[as.character(nd)]]
        sig_items <- item_df[item_df$LRT_p < alpha & !is.na(item_df$LRT_p), ]
        if (nrow(sig_items) > 0) {
          message("  Items with RS (unadjusted): ",
                  paste(sig_items$Item, " (", sig_items$RS_type, ")",
                        sep = "", collapse = ", "))
        } else {
          message("  No individual items reached significance")
        }
      }
    }
  }

  # ---- Compile results ----
  global_df <- do.call(rbind, global_results)
  item_level_df <- if (length(item_results) > 0) do.call(rbind, item_results) else NULL

  # Apply p-value adjustment to omnibus RS tests across nodes
  if (global_p_adjust != "none" && nrow(global_df) > 1) {
    non_na <- !is.na(global_df$LRT_p)
    global_df$LRT_p_adj <- NA_real_
    global_df$LRT_p_adj[non_na] <- p.adjust(global_df$LRT_p[non_na],
                                            method = global_p_adjust)
    # Update RS_detected based on adjusted p-values
    global_df$RS_detected <- global_df$LRT_p_adj < alpha
    global_df$RS_detected[is.na(global_df$LRT_p_adj)] <- NA
    if (verbose) {
      message("\nOmnibus p-values adjusted across ", sum(non_na),
              " nodes using '", global_p_adjust, "' method.")
    }
  }

  # Apply p-value adjustment to item-level tests within each node
  if (!is.null(item_level_df) && p_adjust != "none") {
    for (nd_id in unique(item_level_df$Node)) {
      nd_rows <- which(item_level_df$Node == nd_id)
      raw_p <- item_level_df$LRT_p[nd_rows]
      item_level_df$LRT_p_adj[nd_rows] <- p.adjust(raw_p, method = p_adjust)
    }
    if (verbose) {
      message("\nItem-level p-values adjusted using '", p_adjust, "' method.")
    }
  }

  out <- list(
    global = global_df,
    item_level = item_level_df,
    parameters = param_list,
    n_items = n_items,
    alpha = alpha,
    p_adjust = p_adjust,
    global_p_adjust = global_p_adjust,
    rs_threshold = rs_threshold
  )
  class(out) <- "rs_characterization"
  return(out)
}


#' Print Method for Response Shift Characterization Results
#'
#' Displays a formatted summary of response shift characterization results
#' from \code{\link{rs_characterize}}, including omnibus RS test results per
#' terminal node and item-level RS testing results with significance markers.
#'
#' @param x An \code{rs_characterization} object returned by
#'   \code{\link{rs_characterize}}.
#' @param ... Additional arguments (currently unused).
#'
#' @return Invisibly returns \code{x}. Called for its side effect of printing
#'   formatted output to the console.
#'
#' @details
#' The output is organized in two sections:
#'
#' \strong{Omnibus RS Test:} For each terminal node, displays the LRT
#' statistic, degrees of freedom, raw and adjusted p-values (if applicable),
#' RS detection status, effect sizes (pseudo R-squared and AIC difference),
#' latent mean shift, T1-T2 correlation, and convergence warnings.
#'
#' \strong{Item-Level RS Testing:} For nodes where omnibus RS was detected,
#' displays per-item LRT statistics, raw and adjusted p-values, RS type
#' classification, and significance markers (\code{***} for items significant
#' after adjustment). Also reports the p-value adjustment method and RS
#' classification threshold used.
#'
#' @seealso \code{\link{rs_characterize}} for computing the results
#' @export
print.rs_characterization <- function(x, ...) {

  cat("=== Response Shift Characterization ===\n\n")

  g <- x$global

  cat("--- Omnibus RS Test (Constrained vs Unconstrained) ---\n")
  cat("H0: Item parameters are equal across T1 and T2\n")
  cat("H1: Item parameters differ across T1 and T2\n")
  has_global_adj <- "LRT_p_adj" %in% names(g)
  if (has_global_adj) {
    cat(sprintf("Omnibus p-values adjusted across nodes using '%s' method\n",
                x$global_p_adjust))
  }
  cat("\n")

  # Print global results nicely
  for (i in 1:nrow(g)) {
    cat(sprintf("Node %d (n = %d):\n", g$Node[i], g$n[i]))
    if (has_global_adj && !is.na(g$LRT_p_adj[i])) {
      cat(sprintf("  LRT: chi2 = %.3f, df = %d, p = %s, p_adj = %s\n",
                  as.numeric(g$LRT_chi2[i]), g$LRT_df[i],
                  formatC(g$LRT_p[i], format = "e", digits = 3),
                  formatC(g$LRT_p_adj[i], format = "e", digits = 3)))
    } else {
      cat(sprintf("  LRT: chi2 = %.3f, df = %d, p = %s\n",
                  as.numeric(g$LRT_chi2[i]), g$LRT_df[i],
                  formatC(g$LRT_p[i], format = "e", digits = 3)))
    }
    cat(sprintf("  RS detected: %s\n", g$RS_detected[i]))
    if ("pseudo_R2" %in% names(g)) {
      cat(sprintf("  Effect size: pseudo R2 = %.4f, AIC diff = %.2f\n",
                  as.numeric(g$pseudo_R2[i]), as.numeric(g$AIC_diff[i])))
    }
    cat(sprintf("  Latent mean shift (mu_T2): %.3f\n", as.numeric(g$mu_T2[i])))
    cat(sprintf("  Correlation (T1, T2): %.3f\n", as.numeric(g$cor_T1_T2[i])))
    if (!is.na(g$converged_constrained[i]) && !g$converged_constrained[i]) {
      cat("  WARNING: Constrained model did not converge\n")
    }
    if (!is.na(g$converged_unconstrained[i]) && !g$converged_unconstrained[i]) {
      cat("  WARNING: Unconstrained model did not converge\n")
    }
    cat("\n")
  }

  # Print item-level results if present
  if (!is.null(x$item_level)) {
    cat("--- Item-Level RS Testing ---\n")
    cat("(Items freed one at a time from the constrained model)\n")
    has_adj <- "LRT_p_adj" %in% colnames(x$item_level)
    if (has_adj) {
      cat(sprintf("P-values adjusted using '%s' method\n", x$p_adjust))
    }
    cat(sprintf("RS classification threshold: %.2f\n", x$rs_threshold))
    cat("\n")

    for (nd in unique(x$item_level$Node)) {
      cat(sprintf("Node %d:\n", nd))
      node_items <- x$item_level[x$item_level$Node == nd, ]

      # Use adjusted p-values for significance marking if available
      p_for_sig <- if (has_adj) node_items$LRT_p_adj else node_items$LRT_p
      node_items$Sig <- ifelse(p_for_sig < x$alpha & !is.na(p_for_sig),
                               " ***", "")

      for (j in 1:nrow(node_items)) {
        if (has_adj) {
          cat(sprintf("  Item %2d: chi2 = %7.3f, df = %d, p = %.6f, p_adj = %.6f  %-25s%s\n",
                      node_items$Item[j],
                      node_items$LRT_chi2[j],
                      node_items$LRT_df[j],
                      node_items$LRT_p[j],
                      node_items$LRT_p_adj[j],
                      node_items$RS_type[j],
                      node_items$Sig[j]))
        } else {
          cat(sprintf("  Item %2d: chi2 = %7.3f, df = %d, p = %.6f  %-25s%s\n",
                      node_items$Item[j],
                      node_items$LRT_chi2[j],
                      node_items$LRT_df[j],
                      node_items$LRT_p[j],
                      node_items$RS_type[j],
                      node_items$Sig[j]))
        }
      }
      cat("\n")
    }
  }

  invisible(x)
}

#' Internal Function: Fit Graded Response Model
#'
#' Fits a graded response model (GRM) to item response data. This is an internal
#' function called by \code{grmtree} and is not intended to be used directly.
#'
#' @param y A matrix of item responses (rows = persons, columns = items)
#' @param x Optional predictor matrix (not currently used)
#' @param start Optional starting values (passed to mirt)
#' @param weights Optional case weights (not currently used)
#' @param offset Optional offset (not currently used)
#' @param estfun Logical indicating whether to compute empirical estimating
#'   functions
#' @param object Logical indicating whether to return the full mirt model object
#' @param ... Additional arguments passed to \code{mirt::mirt}
#'
#' @return A list containing: \item{coefficients}{Item parameter estimates}
#'   \item{objfun}{Negative log-likelihood value} \item{estfun}{Empirical
#'   estimating functions if requested} \item{object}{Full mirt model object if
#'   requested}
#'
#' @importFrom mirt mirt logLik
#' @keywords internal
grmfit <- function(y, x = NULL, start = NULL, weights = NULL, offset = NULL, ...,
                   estfun = FALSE, object = FALSE) {



  # Validate input response matrix
  if (!is.matrix(y)) {
    y <- as.matrix(y)
  }

  if (ncol(y) < 2) {
    stop("Response matrix must have at least 2 columns (items). Found: ", ncol(y))
  }

  # Fit GRM model
  fit <- mirt::mirt(y, model = 1, itemtype = 'graded', SE = TRUE, verbose = FALSE, ...)

  # Extract coefficients
  coefs <- mirt::coef(fit, IRTpars = TRUE, simplify = TRUE)

  # Validate coefficients were returned

  if (is.null(coefs)) {
    stop("Model fitting failed - no coefficients returned")
  }

  # Return list
  list(
    coefficients = coefs,
    objfun = -as.numeric(mirt::logLik(fit)),
    estfun = if (estfun) mirt::estfun.AllModelClass(fit) else NULL,
    object = if (object) fit else NULL
  )
}


#' Fit a Graded Response Model Tree for Differential Item Functioning Detection
#'
#' This function implements a tree-based graded response model (GRM) using
#' model-based recursive partitioning to detect and account for differential
#' item functioning (DIF) in polytomous items. The GRMTree combines the
#' statistical framework of item response theory with recursive partitioning to
#' identify heterogeneous subgroups in the population where item parameters
#' (discrimination and thresholds) vary systematically across covariates.
#'
#' @param formula A formula specifying the model structure with the response
#'   matrix on the left and partitioning variables on the right (e.g.,
#'   `response_matrix ~ age + gender`).
#' @param data A data frame containing the variables in the model.
#' @param na.action How to handle missing values (default: `na.omit`).
#' @param control A list of control parameters created by `grmtree.control()`.
#' @param mtry Number of variables randomly sampled as candidates at each split.
#'   If NULL, all variables are considered.
#' @param ... Additional arguments passed to the fitting function.
#'
#' @return An object of class `grmtree` inheriting from `modelparty` containing
#'   the fitted tree structure.
#'
#' @details
#'
#' The algorithm works by first estimating a global GRM for the entire sample,
#' then recursively testing for parameter instability with respect to available
#' covariates. When significant DIF is detected, the sample is partitioned into
#' homogeneous subgroups, each with their own set of item parameters. This
#' approach allows for the identification of complex interaction effects and
#' provides interpretable tree structures that visualize how item functioning
#' varies across different patient subgroups.
#'
#' GRMTree is particularly useful in health outcomes research where
#' patient-reported outcome measures may function differently across diverse
#' demographic, clinical, or socioeconomic subgroups. The resulting tree
#' diagrams facilitate the development of personalized assessment strategies and
#' can inform targeted interventions by identifying specific patient
#' characteristics associated with differential item interpretation.
#'
#' ## Conventional Graded Response Model (GRM)
#'
#' Let \eqn{Y_{im}} denote the response of the \eqn{i^{th}} (\eqn{i=1,\ldots,N})
#' individual to the \eqn{m^{th}} (\eqn{m = 1,2,\ldots,M}) item. The graded
#' response model is described as:
#'
#' \deqn{P(Y_{im} \geq j | \tau_{mj}, \lambda_m, \theta_i) =
#' \frac{\exp(-(\tau_{mj} - \lambda_m \theta_i))}{1 + \exp(-(\tau_{mj} -
#' \lambda_m \theta_i))}}
#'
#' where:
#' - \eqn{P(Y_{im} \geq j | \tau_{mj}, \lambda_m, \theta_i)} is the probability that
#' individual \eqn{i}'s response is in category \eqn{j} or higher on item
#' \eqn{m},
#' - \eqn{\tau_{mj}} is the threshold parameter between categories \eqn{j-1} and
#' \eqn{j} for item \eqn{m},
#' - \eqn{\lambda_m} is the discrimination parameter for item \eqn{m},
#' - \eqn{\theta_i \sim N(0,1)} is the latent trait score for individual \eqn{i}.
#'
#' This parametrization is equivalent to the conventional IRT formulation where
#' item discrimination is \eqn{a_m = \lambda_m} and item difficulty is
#' \eqn{b_{mj} = \tau_{mj} / \lambda_m}.
#'
#' ## Graded Response Model Tree (GRMTree) Implementation
#'
#' The GRMTree is a hybrid model that integrates the GRM with model-based
#' recursive partitioning to detect and account for differential item
#' functioning (DIF) across subgroups defined by covariates. The algorithm
#' proceeds through the following steps:
#'
#' **Step 1: Global Model Estimation**
#'
#' Estimate the GRM item parameters \eqn{(\hat{\tau}_{mj}, \hat{\lambda}_m)}
#' jointly for all individuals in the study cohort at the root node via maximum
#' likelihood estimation:
#'
#' \deqn{\hat{\beta}_{\text{global}} = \arg\max_{\beta} \sum_{i=1}^N \log
#' L(\beta; \mathbf{y}_i)}
#'
#' where \eqn{\beta = (a_1, \ldots, a_J, b_{11}, \ldots, b_{J,m-1})} contains
#' all item parameters (discrimination and difficulty), providing a baseline
#' model assuming parameter invariance.
#'
#' **Step 2: Parameter Stability Testing**
#'
#' For each available covariate \eqn{X_p} (\eqn{p = 1, \ldots, P}), assess the
#' stability of the item parameters by conducting score-based structural change
#' tests. This involves: 1. Calculating the score function contributions
#' \eqn{s(\hat{\beta}; y_i, x_i)} for each individual, 2. Ordering these scores
#' with respect to each covariate \eqn{X_p}, 3. Testing the null hypothesis
#' \eqn{H_0: \mathbb{E}[s(\hat{\beta}; y_i, x_i)] = 0} for all \eqn{i} against
#' the alternative that scores fluctuate systematically with \eqn{X_p},
#' indicating parameter instability (DIF).
#'
#' **Step 3: Recursive Partitioning**
#'
#' If significant instability is detected (\eqn{p < \alpha_{\text{adj}}}):
#' - **Covariate Selection:** Identify the covariate \eqn{X_p^*} with the most
#' significant instability (smallest adjusted p-value),
#' - **Split Point Determination:** Find the optimal cut-point \eqn{c^*} that
#' maximizes the partitioned log-likelihood: \deqn{\ell_{\text{left}}(\beta) +
#' \ell_{\text{right}}(\beta) = \sum_{i: X_{pi} \leq c^*} \log L(\beta; y_i) +
#' \sum_{i: X_{pi} > c^*} \log L(\beta; y_i)} over all possible cut-points on
#' \eqn{X_p^*},
#' - **Sample Splitting:** Partition the sample into two child nodes based on the
#' rule \eqn{X_p^* \leq c^*}.
#'
#' **Step 4: Recursive Application & Stopping Criteria**
#'
#' Repeat Steps 1-3 recursively within each resulting child node until one of
#' the following stopping criteria is met:
#'
#' 1. **No Significant Instability:** No covariate shows significant parameter
#' instability after multiple testing correction (\eqn{\alpha_{\text{adj}} =
#' \alpha/m}, where multiple adjustment methods can be applied, including
#' Bonferroni, Holm, Benjamini-Hochberg, etc.).
#'
#' 2. **Minimum Node Size:** The subsample size falls below a prespecified
#' minimum (e.g., \eqn{n < 10 \times} the number of item parameters).
#'
#' ## Formal GRMTree Structure
#'
#' The final GRMTree provides a piecewise GRM where each terminal node
#' represents a subgroup with homogeneous item parameters, explicitly modeling
#' the detected DIF structure within the data. The resulting GRMTree model can
#' be expressed as a mixture of subgroup-specific GRMs:
#'
#' \deqn{P(Y_{ij} = k | \theta_i, \mathbf{x}_i) = \sum_{b=1}^B I(\mathbf{x}_i
#' \in \mathcal{X}_b) \cdot P_b(Y_{ij} = k | \theta_i)}
#'
#' where:
#' - \eqn{B} is the number of terminal nodes,
#' - \eqn{\mathcal{X}_b} is the covariate subspace defining terminal node \eqn{b},
#' - \eqn{P_b} is the node-specific GRM with parameters \eqn{\beta_b},
#' - \eqn{I(\cdot)} is the indicator function.
#'
#' Each terminal node \eqn{b} contains a complete GRM with:
#' - **Node-specific item parameters:** \eqn{\beta_b = (a_{1b}, \ldots, a_{Jb}, b_{11b}, \ldots, b_{J,m-1,b})}
#' - **Local ability distribution:** \eqn{\theta_i | \mathbf{x}_i \in \mathcal{X}_b \sim N(0, 1)}
#'
#' This approach allows **differential item functioning (DIF)** to be detected
#' and modeled explicitly through the tree structure, where item parameters can
#' vary across subgroups defined by covariates, while maintaining the
#' conditional distribution of the latent trait within each subgroup.
#'
#' ## Post-Hoc Multiple Comparison Adjustments
#'
#' For holm, BH, BY, hochberg, and
#' hommel methods, the algorithm employs a two-stage approach: (1) build a tree
#' using \code{initial_alpha} as the splitting threshold, (2) apply global
#' p-value adjustment across all splits, and (3) prune splits that do not meet
#' the adjusted threshold \code{alpha}. This differs from Bonferroni correction,
#' which is applied locally during tree construction via \code{partykit::mob}.
#' The choice of \code{initial_alpha} represents a statistical trade-off. The
#' default (\code{min(3 * alpha, 0.20)}) provides a balance between power and
#' computational efficiency. Note that global post-hoc adjustments in
#' hierarchical tree structures may be conservative compared to per-node
#' adjustments, as they account for all tests performed during tree exploration.
#' This global adjustment approach controls the tree-wide error rate and may be
#' conservative (Type I error often <3%). This conservativeness ensures strong
#' control of family-wise error rate or false discovery rate across all splits
#' in the tree. Users requiring less conservative control may prefer the
#' Bonferroni method, which applies per-node adjustment during tree
#' construction.
#'
#' @references ## Methodological Foundations
#'
#'   Samejima, F. (1969). Estimation of latent ability using a response pattern
#'   of graded scores. Psychometrika Monograph Supplement, 34, 100-114.
#'
#'   Strobl, C., Kopf, J., & Zeileis, A. (2015). Rasch trees: A new method for
#'   detecting differential item functioning in the Rasch model. Psychometrika,
#'   80(2), 289-316.
#'
#'   Komboz, B., Strobl, C., & Zeileis, A. (2018). Tree-based global model tests
#'   for polytomous Rasch models. Educational and psychological measurement,
#'   78(1), 128-166. https://doi.org/10.1177/0013164416664394
#'
#'   Arimoro, O. I., Lix, L. M., Patten, S. B., Sawatzky, R., Sebille, V., Liu,
#'   J., Wiebe, S., Josephson, C. B., & Sajobi, T. T. (2025). Tree-based latent
#'   variable model for assessing differential item functioning in
#'   patient-reported outcome measures: a simulation study. Quality of Life
#'   Research. https://doi.org/10.1007/s11136-025-04018-6
#'
#'   ## Applied Examples
#'
#'   Arimoro, O. I., Josephson, C. B., James, M. T., Patten, S. B., Wiebe, S.,
#'   Lix, L. M., & Sajobi, T. T. (2024). Screening for depression in patients
#'   with epilepsy: same questions but different meaning to different patients.
#'   Quality of Life Research, 33(12), 3409-3419.
#'   https://doi.org/10.1007/s11136-024-03782-1
#'
#' @author Olayinka Imisioluwa Arimoro \email{olayinka.arimoro@ucalgary.ca},
#'   Lisa M. Lix, Tolulope T. Sajobi
#'
#' @examples
#' \donttest{
#'   library(grmtree)
#'   library(hlt)
#'
#'   # Prepare the asti data (from the hlt package)
#'   data("asti", package = "hlt")
#'   asti$resp <- data.matrix(asti[, 1:4])
#'
#'   # Fit GRM tree with gender and group as partitioning variables
#'   tree <- grmtree(resp ~ gender + group,
#'           data = asti,
#'           control = grmtree.control(minbucket = 30))
#'
#'   ## Print and plot the tree
#'   print(tree)
#'   plot(tree)
#'
#'   # Extract item parameters for specific subgroups
#'   discr_params <- discrpar_grmtree(tree)
#'   threshold_params <- threshpar_grmtree(tree)
#' }
#'
#' @seealso \code{\link{print.grmtree}} prints the detailed summary results of
#'   the `grmtree` object, \code{\link{grmtree.control}} creates a control
#'   object for `grmtree`, \code{\link{plot.grmtree}} creates plot for the
#'   `grmtree` object, \code{\link{grmforest}} for GRM Forests,
#'   \code{\link{varimp}} calculates the variable importance for GRM Forest,
#'   \code{\link{fscores_grmtree}} for computing factor scores,
#'   \code{\link{threshpar_grmtree}} for extracting threshold parameters,
#'   \code{\link{discrpar_grmtree}} for extracting discrimination parameters,
#'   \code{\link{itempar_grmtree}} for extracting item parameters
#'
#' @export
#' @importFrom partykit mob nodeids nodeprune
#' @importFrom strucchange sctest
#' @importFrom stats model.frame model.response na.omit p.adjust
grmtree <- function(formula, data, na.action = na.omit,
                    control = grmtree.control(), mtry = NULL, ...) {

  # Validate inputs
  if (!inherits(formula, "formula")) {
    stop("'formula' must be a valid formula object")
  }
  if (!is.data.frame(data)) {
    stop("'data' must be a data.frame")
  }

  # Keep call for later reference
  cl <- match.call(expand.dots = TRUE)

  # Parse the formula and handle missing values
  mf <- model.frame(formula, data = data, na.action = na.action)

  # Extract response matrix
  y <- model.response(mf)
  if (!is.matrix(y)) {
    stop("Response variable must be a matrix. Did you provide a multi-item response matrix?")
  }

  # Get p_adjust method from control
  p_adjust <- if (!is.null(control$p_adjust)) control$p_adjust else "none"
  alpha <- control$alpha
  minbucket <- control$minsize  # mob_control stores minbucket as minsize

  # For BH, Holm, BY, hochberg, hommel - use post-hoc pruning approach
  if (p_adjust %in% c("holm", "BH", "BY", "hochberg", "hommel")) {

    # Use documented initial_alpha
    initial_alpha <- control$initial_alpha

    ctrl_full <- partykit::mob_control(
      minbucket = minbucket,
      bonferroni = FALSE,
      alpha = initial_alpha,
      ytype = "matrix"
    )

    rval <- partykit::mob(
      formula = formula,
      data = data,
      fit = grmfit,
      control = ctrl_full,
      na.action = na.action
    )

    # Apply p-value adjustment and prune
    rval <- .adjust_and_prune_tree(rval, p_adjust, alpha)

  } else {
    # For "none" and "bonferroni" - use mob directly
    rval <- partykit::mob(
      formula = formula,
      data = data,
      fit = grmfit,
      control = control,
      na.action = na.action
    )
  }

  # Store p_adjust method and original call
  rval$info$p_adjust <- p_adjust
  rval$info$call <- cl
  class(rval) <- c("grmtree", "modelparty", "party")

  return(rval)
}

library(testthat)
library(hlt)
library(mirt)

## Test that grmtree works as expected
test_that("grmtree produces correct object structure", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  ## ASTI data
  data("asti", package = "hlt")
  ## Create response as a matrix
  asti$resp <- data.matrix(asti[, 1:4])

  # Basic functionality
  expect_silent({
    tree <- grmtree(resp ~ gender + group, data = asti,
                    control = grmtree.control(minbucket = 30))
  })

  expect_s3_class(tree, "grmtree")
  expect_true("grmtree" %in% class(tree))

  # Check response matrix requirement
  expect_error(
    grmtree(group ~ gender, data = asti),
    "Response variable must be a matrix"
  )
})


## Test that grmtree.control handles invalid inputs
test_that("grmtree.control handles invalid inputs", {
  # Valid inputs
  expect_silent(grmtree.control(minbucket = 10, alpha = 0.01))

  # Invalid minbucket
  expect_error(grmtree.control(minbucket = 0), "must be at least 1")
  expect_error(grmtree.control(minbucket = "a"), "must be numeric")

  # Invalid alpha
  expect_error(grmtree.control(alpha = 0), "must be between 0 and 1")
  expect_error(grmtree.control(alpha = 1.1), "must be between 0 and 1")

  # Invalid p_adjust
  expect_error(grmtree.control(p_adjust = "invalid"),
               "must be one of")
})

test_that("grmtree.control handles initial_alpha correctly", {
  # Default initial_alpha
  ctrl <- grmtree.control(p_adjust = "BH", alpha = 0.05)
  expect_equal(ctrl$initial_alpha, 0.15)  # min(3*0.05, 0.20)

  # Custom initial_alpha
  ctrl <- grmtree.control(p_adjust = "BH", alpha = 0.05, initial_alpha = 0.10)
  expect_equal(ctrl$initial_alpha, 0.10)

  # Invalid initial_alpha (too small)
  expect_error(
    grmtree.control(p_adjust = "BH", alpha = 0.05, initial_alpha = 0.03),
    "initial_alpha"
  )

  # Invalid initial_alpha (equal to alpha)
  expect_error(
    grmtree.control(p_adjust = "BH", alpha = 0.05, initial_alpha = 0.05),
    "initial_alpha"
  )
})

library(testthat)
library(mirt)
library(hlt)

## Test that plot.grmtree works correctly
test_that("plot.grmtree works correctly", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  ## ASTI data
  data("asti", package = "hlt")
  ## Create response as a matrix
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti)

  # Test basic plot types
  expect_silent(plot(tree, type = "regions"))
  expect_silent(plot(tree, type = "profile"))
  expect_message(plot(tree, type = "histogram"), "Processing node")

  # Test profile plot variants
  expect_silent(plot(tree, type = "profile", what = "item"))
  expect_silent(plot(tree, type = "profile", what = "threshold"))
  expect_silent(plot(tree, type = "profile", what = "discrimination"))

  # Test parameter variations
  expect_silent(plot(tree, tnex = 3))
  expect_silent(plot(tree, drop_terminal = FALSE))
  expect_silent(plot(tree, spacing = 0.2))

  # Test error handling - need to force dispatch to plot.grmtree
  expect_error(plot.grmtree("not_a_tree"), "must be of class 'grmtree'")
  expect_error(plot(tree, type = "invalid"), "should be one of")
  expect_error(plot(tree, what = "invalid"), "should be one of")
  expect_error(plot(tree, tnex = -1), "positive numeric value")
  expect_error(plot(tree, spacing = -0.1), "non-negative numeric value")

  # Test with empty tree (should fail gracefully)
  empty_tree <- structure(
    list(
      node = structure(list(id = 1L), class = "partynode"),
      fitted = data.frame("(fitted)" = factor(1), "(response)" = matrix(0, 1, 4)),
      terms = terms(resp ~ gender + group),
      data = asti,
      info = list()
    ),
    class = c("grmtree", "constparty", "party")
  )
  expect_error(plot(empty_tree, type = "regions"), "Failed to create regions plot")
})

## Test that the lot.grmtree argument validation works
test_that("plot.grmtree argument validation works", {
  # Test input object validation separately
  expect_error(plot.grmtree(), "missing")
  expect_error(plot.grmtree("not_a_tree"), "must be of class 'grmtree'")

  # Improved validation helper function
  validate_plot_args <- function(type = c("regions", "profile", "histogram"),
                                 what = c("item", "threshold", "discrimination"),
                                 tnex = 2L,
                                 drop_terminal = TRUE,
                                 spacing = 0.1) {
    # Input validation checks
    type <- match.arg(type)
    what <- match.arg(what)

    if (!is.numeric(tnex) || length(tnex) != 1 || tnex <= 0 || !is.finite(tnex)) {
      stop("'tnex' must be a single positive numeric value.")
    }

    if (!is.logical(drop_terminal) || length(drop_terminal) != 1 || is.na(drop_terminal)) {
      stop("'drop_terminal' must be a single logical value.")
    }

    if (!is.numeric(spacing) || length(spacing) != 1 || spacing < 0 || !is.finite(spacing)) {
      stop("'spacing' must be a single non-negative numeric value.")
    }

    return(TRUE)
  }

  # Test the validation logic
  expect_silent(validate_plot_args())  # Test default args work

  # Test invalid inputs
  expect_error(
    validate_plot_args(type = "invalid"),
    "should be one of"
  )

  expect_error(
    validate_plot_args(what = "invalid"),
    "should be one of"
  )

  expect_error(
    validate_plot_args(tnex = "a"),
    "must be a single positive numeric value"
  )

  expect_error(
    validate_plot_args(drop_terminal = NA),
    "must be a single logical value"
  )

  expect_error(
    validate_plot_args(spacing = -0.1),
    "must be a single non-negative numeric value"
  )

  # Additional edge cases
  expect_error(
    validate_plot_args(tnex = NA_real_),
    "must be a single positive numeric value"
  )

  expect_error(
    validate_plot_args(spacing = Inf),
    "must be a single non-negative numeric value"
  )
})

library(testthat)
library(hlt)
library(mirt)


## Test that discrpar_grmtree works as expected
test_that("discrpar_grmtree works with valid inputs", {

  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  ## Asti data
  data("asti", package = "hlt")
  ## Create response as a matrix
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))

  # Default usage
  expect_silent(discr <- discrpar_grmtree(tree))
  expect_s3_class(discr, "data.frame")
  expect_true(nrow(discr) > 0)
  expect_equal(names(discr), c("Node", "Item", "Discrimination"))

  # Check discrimination values
  expect_true(all(discr$Discrimination > 0))

  # Specific nodes
  nodes <- partykit::nodeids(tree, terminal = TRUE)
  if (length(nodes) > 1) {
    expect_silent(discr_sub <- discrpar_grmtree(tree, node = nodes[1:2]))
    expect_equal(unique(discr_sub$Node), nodes[1:2])
  }
})


## Test that discrpar_grmtree handles errors appropriately
test_that("discrpar_grmtree handles errors appropriately", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))

  # 1. Test invalid object type
  expect_error(discrpar_grmtree(list()), "must be a grmtree object")

  # 2. Test invalid nodes
  nodes <- partykit::nodeids(tree, terminal = TRUE)
  expect_error(discrpar_grmtree(tree, node = 999), "Invalid node IDs")

  # Create a tree with a single non-terminal node
  bad_tree <- tree
  bad_tree$node <- list(
    id = 1L,
    split = NULL,
    kids = NULL,
    surrogates = NULL,
    info = list()
  )
  class(bad_tree$node) <- "partynode"
  expect_error(discrpar_grmtree(bad_tree), "No discrimination parameters found in node")

})


## Test that discrimination parameters are reasonable
test_that("discrimination parameters are reasonable", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  ## Asti data
  data("asti", package = "hlt")
  ## Create response as a matrix
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti,
                  control = grmtree.control(minbucket = 30))
  discr <- discrpar_grmtree(tree)

  # Typical discrimination values in educational testing
  expect_true(all(discr$Discrimination > 0.2))
  expect_true(all(discr$Discrimination < 4))

  # Check items are correctly labeled
  expect_true(all(discr$Item %in% paste0("resp",colnames(asti[, 1:4]))))
})

#' Extract Threshold Parameters from Longitudinal GRM Tree
#'
#' Extracts threshold (difficulty) parameters for each unique item from all
#' terminal nodes of a longitudinal GRM tree. Only T1 item parameters are
#' returned since the constrained model enforces equal thresholds across T1 and
#' T2 within each node.
#'
#' @param object A \code{longitudinal_grmtree} object.
#' @param node Optional vector of node IDs. If NULL (default), extracts from all
#'   terminal nodes.
#' @param clean_names Logical. If TRUE (default), clean item names by removing
#'   common prefixes (e.g., "resp_wideMOS_") and suffixes (e.g., "_BL", "_T1").
#' @param ... Additional arguments (currently unused).
#'
#' @return A data.frame with columns:
#'   \describe{
#'     \item{Node}{Terminal node ID}
#'     \item{Item}{Item name (cleaned if \code{clean_names = TRUE})}
#'     \item{b1, b2, ..., bK}{Threshold parameters for each category boundary}
#'   }
#'   Contains \code{n_items} rows per node (not \code{2 * n_items}).
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Phase 1: fit the longitudinal GRM tree
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Print the threshold parameters
#'   thresholds <- threshpar_longitudinal_grmtree(ltree)
#'   print(thresholds)
#'
#'   # Using the raw names
#'   threshpar_longitudinal_grmtree(ltree, clean_names = FALSE)  # raw names
#'
#' @seealso \code{\link{longitudinal_grmtree}} for Phase 1 (tree fitting),
#'   \code{\link{discrpar_longitudinal_grmtree}} for extracting discrimination
#'   parameters for longitudinal GRMTree,
#'   \code{\link{itempar_longitudinal_grmtree}} for extracting item parameters
#'   for longitudinal GRMTree
#'
#' @export
#' @importFrom partykit nodeids nodeapply
#' @importFrom mirt coef
threshpar_longitudinal_grmtree <- function(object, node = NULL,
                                           clean_names = TRUE, ...) {
  if (!inherits(object, "longitudinal_grmtree")) {
    stop("'object' must be a longitudinal_grmtree object")
  }

  n_items <- object$info$n_items
  if (is.null(n_items)) {
    stop("Tree object does not contain n_items.")
  }

  if (is.null(node)) {
    node <- partykit::nodeids(object, terminal = TRUE)
    if (length(node) == 0) stop("No terminal nodes found")
  }

  thresholds <- do.call("rbind", lapply(node, function(n) {
    model <- partykit::nodeapply(
      object, ids = n, FUN = function(nd) nd$info$object
    )[[1]]

    coef_model <- mirt::coef(model, IRTpars = TRUE, simplify = TRUE)
    all_items <- coef_model$items
    thresh_cols <- grep("^b", colnames(all_items))

    if (length(thresh_cols) == 0) {
      stop("No threshold parameters found in node ", n)
    }

    # T1 items only (rows 1:n_items)
    thresh <- all_items[1:n_items, thresh_cols, drop = FALSE]
    items <- rownames(all_items)[1:n_items]

    if (clean_names) {
      items <- gsub("^.*MOS_|^.*resp_wide|^.*resp", "", items)
      items <- gsub("_BL$|_Pre$|_T1$|_baseline$", "", items)
    }

    thresh_df <- as.data.frame(thresh)
    thresh_df$Item <- items
    thresh_df$Node <- n
    thresh_df <- thresh_df[, c("Node", "Item", colnames(thresh)), drop = FALSE]
    return(thresh_df)
  }))

  rownames(thresholds) <- NULL
  return(thresholds)
}

library(testthat)
library(mirt)


# ==============================================================================
# Extraction functions  input validation (fast)
# ==============================================================================
test_that("extraction functions reject non-longitudinal objects", {
  expect_error(threshpar_longitudinal_grmtree(list()),
               "must be a longitudinal_grmtree object")
  expect_error(discrpar_longitudinal_grmtree(list()),
               "must be a longitudinal_grmtree object")
  expect_error(itempar_longitudinal_grmtree(list()),
               "must be a longitudinal_grmtree object")
  expect_error(fscores_longitudinal_grmtree(list()),
               "must be a longitudinal_grmtree object")
  expect_error(latentpar_longitudinal_grmtree(list()),
               "must be a longitudinal_grmtree object")
})

test_that("fscores_longitudinal_grmtree rejects bad scoring methods", {
  stub <- structure(list(info = list(n_items = 4)),
                    class = c("longitudinal_grmtree", "grmtree",
                              "modelparty", "party"))
  expect_error(fscores_longitudinal_grmtree(stub, method = "invalid"),
               "method must be one of")
})

test_that("plot.longitudinal_grmtree rejects non-longitudinal objects", {
  expect_error(plot.longitudinal_grmtree(list()),
               "must be of class 'longitudinal_grmtree'")
})

#' Extract Latent Trait Parameters from Longitudinal GRM Tree
#'
#' Extracts the latent trait distribution parameters from each terminal node of
#' a longitudinal GRM tree: the T2 latent mean shift (\eqn{\mu_{T2}}), the T2
#' latent variance (\eqn{\sigma^2_{T2}}), and the T1-T2 correlation.
#'
#' @param object A \code{longitudinal_grmtree} object.
#' @param node Optional vector of node IDs. If NULL, all terminal nodes.
#' @param ... Additional arguments (currently unused).
#'
#' @return A data.frame with columns:
#'   \describe{
#'     \item{Node}{Terminal node ID}
#'     \item{n}{Sample size in the node}
#'     \item{mu_T2}{Latent mean at T2 (positive = improvement)}
#'     \item{sigma2_T2}{Latent variance at T2}
#'     \item{cor_T1_T2}{Correlation between T1 and T2 latent traits}
#'   }
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Phase 1: fit the longitudinal GRM tree
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Print latent trait summary
#'   latent <- latentpar_longitudinal_grmtree(ltree)
#'   print(latent)
#'
#' @seealso \code{\link{longitudinal_grmtree}} for Phase 1 (tree fitting),
#'   \code{\link{fscores_longitudinal_grmtree}}, for computing factor scores for
#'   longitudinal GRMTree
#'
#' @export
#' @importFrom mirt coef
#' @importFrom partykit nodeids nodeapply info_node
latentpar_longitudinal_grmtree <- function(object, node = NULL, ...) {

  if (!inherits(object, "longitudinal_grmtree")) {
    stop("'object' must be a longitudinal_grmtree object")
  }

  if (is.null(node)) {
    node <- partykit::nodeids(object, terminal = TRUE)
  }

  results <- lapply(node, function(nd) {
    model <- partykit::nodeapply(
      object, ids = nd, FUN = function(n) n$info$object
    )[[1]]
    if (is.null(model)) {
      warning("No model object in node ", nd)
      return(data.frame(Node = nd, n = NA, mu_T2 = NA,
                        sigma2_T2 = NA, cor_T1_T2 = NA))
    }
    coefs <- mirt::coef(model, simplify = TRUE)
    means <- coefs$means
    cov_mat <- coefs$cov
    nobs <- partykit::nodeapply(
      object, ids = nd,
      FUN = function(n) partykit::info_node(n)$nobs
    )[[1]]
    data.frame(
      Node = nd,
      n = nobs,
      mu_T2 = round(means[2], 4),
      sigma2_T2 = round(cov_mat[2, 2], 4),
      cor_T1_T2 = round(cov_mat[1, 2] / sqrt(cov_mat[1, 1] * cov_mat[2, 2]), 4),
      stringsAsFactors = FALSE
    )
  })

  do.call(rbind, results)
}

#' Control Parameters for GRM Trees
#'
#' Creates a control object for `grmtree` containing various parameters that
#' control the tree growing process.
#'
#' @param minbucket Minimum number of observations in a terminal node (default:
#'   20).
#' @param p_adjust Method for p-value adjustment. One of: "none", "bonferroni",
#'   "holm", "BH", "BY", "hochberg", or "hommel" (default: "none").
#' @param alpha Significance level for splitting (default: 0.05).
#' @param ... Additional arguments passed to `partykit::mob_control()`.
#' @param initial_alpha For post-hoc adjustment methods (holm, BH, BY, hochberg,
#'   hommel), the significance threshold for initial tree construction before
#'   pruning. Must satisfy \code{alpha < initial_alpha < 1}. Default is
#'   \code{min(3 * alpha, 0.20)}. Lower values produce more conservative results
#'   but run faster; higher values provide more power but require more
#'   computation and may increase Type I error. Ignored for "none" and
#'   "bonferroni" methods.
#'
#' @return A list of control parameters with class `grmtree_control`.
#'
#' @examples
#' # Use Bonferroni correction with alpha = 0.01
#' ctrl <- grmtree.control(p_adjust = "bonferroni", alpha = 0.01)
#'
#' @seealso \code{\link{grmtree}} fits a Graded Response Model Tree
#'
#' @export
#' @importFrom partykit mob_control

grmtree.control <- function(minbucket = 20, p_adjust = "none", alpha = 0.05,
                            initial_alpha = NULL, ...) {


  # Validate minbucket
  if (!is.numeric(minbucket)) {
    stop("'minbucket' must be numeric")
  }
  if (minbucket < 1) {
    stop("'minbucket' must be at least 1")
  }

  # Validate alpha
  if (!is.numeric(alpha)) {
    stop("'alpha' must be numeric")
  }
  if (alpha <= 0 || alpha >= 1) {
    stop("'alpha' must be between 0 and 1")
  }

  # Validate p_adjust method
  p_adjust_methods <- c("none", "bonferroni", "holm", "BH", "BY", "hochberg", "hommel")
  if (!p_adjust %in% p_adjust_methods) {
    stop("'p_adjust' must be one of: ", paste(p_adjust_methods, collapse = ", "))
  }

  # Set default initial_alpha
  if (is.null(initial_alpha)) {
    if (p_adjust %in% c("holm", "BH", "BY", "hochberg", "hommel")) {
      initial_alpha <- min(3 * alpha, 0.20)  # Heuristic: 3x target, capped
    } else {
      initial_alpha <- alpha  # Not used for "none" and "bonferroni"
    }
  }

  # Validate initial_alpha
  if (p_adjust %in% c("holm", "BH", "BY", "hochberg", "hommel")) {
    if (!is.numeric(initial_alpha) || initial_alpha <= alpha || initial_alpha >= 1) {
      stop("'initial_alpha' must satisfy: alpha < initial_alpha < 1")
    }
  }

  # Create control object
  control <- tryCatch(
    partykit::mob_control(
      minbucket = minbucket,
      bonferroni = (p_adjust == "bonferroni"),
      alpha = alpha,
      ytype = "matrix",
      ...
    ),
    error = function(e) {
      stop("Error creating control parameters: ", e$message)
    }
  )

  # Add custom p_adjust method
  control$p_adjust <- p_adjust
  control$initial_alpha <- initial_alpha
  class(control) <- c("grmtree_control", class(control))
  return(control)
}

# Internal helper function - adjust p-values and prune tree (CORRECTED)
# @keywords internal
.adjust_and_prune_tree <- function(tree, method, alpha, verbose = FALSE) {

  all_nodes <- partykit::nodeids(tree)
  terminal_nodes <- partykit::nodeids(tree, terminal = TRUE)
  inner_nodes <- setdiff(all_nodes, terminal_nodes)

  if (length(inner_nodes) == 0) return(tree)

  # ============================================================
  # Step 1: Collect ALL p-values from ALL covariates at ALL nodes
  # ============================================================
  # This is the key fix. Previously, we took min(p-values) per node
  # first, which discarded the within-node multiplicity before
  # adjustment. Now we keep every individual covariate test.

  pval_node_ids  <- integer(0)
  pval_cov_names <- character(0)
  pval_raw       <- numeric(0)

  for (node_id in inner_nodes) {
    tryCatch({
      test_matrix <- strucchange::sctest(tree, node = node_id)

      if (!is.null(test_matrix) && "p.value" %in% rownames(test_matrix)) {
        pvals <- test_matrix["p.value", ]
        pvals <- pvals[!is.na(pvals)]

        if (length(pvals) > 0) {
          # Replace exact 0 with machine epsilon
          pvals[pvals == 0] <- .Machine$double.eps

          pval_node_ids  <- c(pval_node_ids,  rep(node_id, length(pvals)))
          pval_cov_names <- c(pval_cov_names, names(pvals))
          pval_raw       <- c(pval_raw,       unname(pvals))
        }
      }
    }, error = function(e) NULL)
  }

  if (length(pval_raw) == 0) return(tree)

  # ============================================================
  # Step 2: Apply GLOBAL p-value adjustment across ALL tests
  # ============================================================
  # With P covariates and K inner nodes, we now adjust K*P values.
  # This properly accounts for BOTH within-node multiplicity
  # (testing multiple covariates) and across-node multiplicity
  # (testing at multiple nodes in the tree).

  pval_adj <- stats::p.adjust(pval_raw, method = method)

  if (verbose) {
    cat(sprintf("  Post-hoc adjustment (%s): %d total tests across %d nodes\n",
                method, length(pval_raw), length(inner_nodes)))
  }

  # ============================================================
  # Step 3: For each node, check if the split survives
  # ============================================================
  # MOB split on the covariate with the smallest raw p-value.
  # Since p.adjust preserves ordering, that covariate also has
  # the smallest adjusted p-value at its node. So min(adj_p)
  # per node is equivalent to checking the split variable.

  nodes_to_prune <- integer(0)

  for (node_id in inner_nodes) {
    idx <- which(pval_node_ids == node_id)
    if (length(idx) > 0) {
      min_adj_p <- min(pval_adj[idx])
      if (min_adj_p >= alpha) {
        nodes_to_prune <- c(nodes_to_prune, node_id)
      }

      if (verbose) {
        cat(sprintf("    Node %d: min raw p = %.6f, min adj p = %.6f -> %s\n",
                    node_id, min(pval_raw[idx]), min_adj_p,
                    ifelse(min_adj_p < alpha, "KEEP", "PRUNE")))
      }
    }
  }

  if (length(nodes_to_prune) == 0) return(tree)

  # ============================================================
  # Step 4: Prune non-significant nodes (deepest first)
  # ============================================================
  nodes_to_prune <- sort(nodes_to_prune, decreasing = TRUE)

  for (node_id in nodes_to_prune) {
    current_inner <- setdiff(partykit::nodeids(tree),
                             partykit::nodeids(tree, terminal = TRUE))
    if (node_id %in% current_inner) {
      tree <- .prune_single_node(tree, node_id)
    }
  }

  return(tree)
}

# Internal helper function - prune a single node
# @keywords internal
.prune_single_node <- function(tree, node_id) {

  current_terminals <- partykit::nodeids(tree, terminal = TRUE)

  # Find which terminal nodes are under this node
  terminals_under_node <- c()
  for (term_id in current_terminals) {
    parent_path <- .get_parent_path(tree, term_id)
    if (node_id %in% parent_path) {
      terminals_under_node <- c(terminals_under_node, term_id)
    }
  }

  # Keep all terminals NOT under this node, plus this node itself
  keep_terminals <- setdiff(current_terminals, terminals_under_node)
  keep_terminals <- c(keep_terminals, node_id)
  keep_terminals <- sort(unique(keep_terminals))

  tryCatch({
    partykit::nodeprune(tree, ids = keep_terminals)
  }, error = function(e) {
    tree
  })
}

# Internal helper function - get parent path for a node
# @keywords internal
.get_parent_path <- function(tree, node_id) {
  path <- c(node_id)
  current <- node_id

  while (current > 1) {
    for (possible_parent in partykit::nodeids(tree)) {
      node <- tree[[possible_parent]]
      if (!is.null(node$kids)) {
        kid_ids <- sapply(node$kids, function(k) k$id)
        if (current %in% kid_ids) {
          path <- c(possible_parent, path)
          current <- possible_parent
          break
        }
      }
    }
    if (current == path[1]) break
  }

  return(path)
}

#' Plot Method for GRM Tree Objects
#'
#' Visualizes a GRM (Graded Response Model) tree with different types of
#' terminal node plots. This function extends \code{plot.modelparty} from the
#' partykit package with specialized visualizations for GRM trees.
#'
#' @param x A GRM tree object of class 'grmtree'.
#' @param type Type of terminal node plot to display:
#'   \describe{
#'     \item{"regions"}{Threshold regions plot (default)}
#'     \item{"profile"}{Item parameter profile plot}
#'     \item{"histogram"}{Histogram of factor scores with normal curve}
#'   }
#' @param what Type of parameters to plot when \code{type = "profile"}:
#'   \describe{
#'     \item{"item"}{Both discrimination and threshold parameters (default)}
#'     \item{"threshold"}{Only threshold parameters}
#'     \item{"discrimination"}{Only discrimination parameters}
#'   }
#' @param tnex Numeric scaling factor for terminal node extension (default: 2).
#' @param drop_terminal Logical indicating whether to drop terminal node IDs
#'   (default: TRUE).
#' @param spacing Numeric value controlling spacing between elements (default:
#'   0.1).
#' @param ... Additional arguments passed to the terminal panel functions.
#'
#' @return Invisibly returns the GRM tree object. Primarily called for its side
#'   effect of producing a plot.
#'
#' @details The function provides three visualization types:
#' \itemize{
#'   \item \strong{Regions plot}: Shows threshold parameters as colored regions, useful
#'     for visualizing the difficulty parameters across items and nodes.
#'   \item \strong{Profile plot}: Displays either item parameters (discrimination and
#'     average thresholds), just thresholds, or just discrimination parameters as
#'     line plots across items.
#'   \item \strong{Histogram}: Shows the distribution of factor scores in each node
#'     with an overlaid normal curve.
#' }
#'
#' @examples
#' \donttest{
#'
#' library(grmtree)
#' library(hlt)
#' data("asti", package = "hlt")
#' asti$resp <- data.matrix(asti[, 1:4])
#'
#' # Fit GRM tree with gender and group as partitioning variables
#' tree <- grmtree(resp ~ gender + group,
#'           data = asti,
#'           control = grmtree.control(minbucket = 30))
#'
#' # Default regions plot
#' plot(tree)
#'
#' # Profile plot showing item parameters
#' plot(tree, type = "profile")
#'
#' # Profile plot showing only thresholds
#' plot(tree, type = "profile", what = "threshold")
#'
#' # Histograms of factor scores
#' plot(tree, type = "histogram")
#' }
#'
#' @seealso \code{\link[partykit]{plot.modelparty}} for the underlying plotting
#'   infrastructure, \code{\link{grmtree}} for creating GRM tree objects,
#'   \code{\link{plot.varimp}} creates a bar plot of variable importance scores
#'
#' @method plot grmtree
#' @import grid
#' @export



plot.grmtree <- function(x, type = c("regions", "profile", "histogram"),
                         what = c("item", "threshold", "discrimination"),
                         tnex = 2L, drop_terminal = TRUE, spacing = 0.1, ...) {

  # Input validation
  if (!inherits(x, "grmtree")) {
    stop("The input object must be of class 'grmtree'.")
  }

  type <- match.arg(type)
  what <- match.arg(what)

  if (!is.numeric(tnex) || length(tnex) != 1 || tnex <= 0) {
    stop("'tnex' must be a single positive numeric value.")
  }

  if (!is.logical(drop_terminal) || length(drop_terminal) != 1) {
    stop("'drop_terminal' must be a single logical value.")
  }

  if (!is.numeric(spacing) || length(spacing) != 1 || spacing < 0) {
    stop("'spacing' must be a single non-negative numeric value.")
  }

  # Select appropriate terminal panel function
  terminal_panel <- switch(
    type,
    regions = {
      tryCatch(
        node_regionplot_grmtree(x, spacing = spacing, ...),
        error = function(e) {
          stop("Failed to create regions plot: ", e$message)
        }
      )
    },
    profile = {
      tryCatch(
        node_profileplot_grmtree(x, what = what, spacing = spacing, ...),
        error = function(e) {
          stop("Failed to create profile plot: ", e$message)
        }
      )
    },
    histogram = {
      tryCatch(
        node_histogram_grmtree(x, spacing = spacing, ...),
        error = function(e) {
          stop("Failed to create histogram plot: ", e$message)
        }
      )
    }
  )

  # Call partykit's plot.modelparty with our terminal panel
  tryCatch(
    {
      partykit::plot.modelparty(
        x,
        terminal_panel = terminal_panel,
        tnex = tnex,
        drop_terminal = drop_terminal,
        ...
      )
    },
    error = function(e) {
      stop("Plotting failed: ", e$message)
    }
  )

  # Invisibly return the tree object
  invisible(x)
}

# Function to create a histogram and normal curve for each terminal node with added horizontal spacing
node_histogram_grmtree <- function(grmtree_obj, fill = "lightgray", border = "black", spacing = 0.1, ...) {
  # Extract factor scores for all nodes in the tree
  scores_list <- fscores_grmtree(grmtree_obj)

  function(node) {
    # Get the node ID and corresponding factor scores
    node_id <- partykit::id_node(node)
    factor_scores <- scores_list[[as.character(node_id)]]
    if (is.null(factor_scores) || length(factor_scores) == 0) return()

    # Create histogram data for the factor scores
    hist_data <- graphics::hist(factor_scores, plot = FALSE, ...) # Generate histogram bins without plotting

    # Calculate mean and standard deviation for the normal curve
    mean_score <- mean(factor_scores)
    sd_score <- stats::sd(factor_scores)

    # Create x values for the normal curve
    x_vals <- seq(min(hist_data$breaks), max(hist_data$breaks), length.out = 100)

    # Scale the normal curve to match histogram height and width
    normal_curve <- stats::dnorm(x_vals, mean = mean_score, sd = sd_score) *
      length(factor_scores) * diff(hist_data$breaks[1:2])  # Correct scaling for bin width

    # Adjust horizontal spacing for this node
    x_shift <- node_id * spacing

    # Set up x and y scales based on histogram data
    x_scale <- range(hist_data$breaks) + c(-1, 1) * x_shift  # Shift x-range for spacing
    y_scale <- c(0, max(hist_data$counts, max(normal_curve))) # Ensure y-scale fits all elements
    grid::pushViewport(grid::viewport(layout = grid::grid.layout(1, 1), name = paste0("node_", node_id)))
    grid::pushViewport(grid::viewport(xscale = x_scale, yscale = y_scale))

    # Draw histogram bars
    for (i in seq_along(hist_data$counts)) {
      grid::grid.rect(
        x = mean(hist_data$breaks[i:(i + 1)]),  # Center of each bar
        y = hist_data$counts[i] / 2,  # Center the height
        width = diff(hist_data$breaks[i:(i + 1)]),  # Bar width
        height = hist_data$counts[i],  # Bar height
        gp = grid::gpar(fill = fill, col = border),  # Fill and border colors
        default.units = "native"
      )
    }

    # Add the normal curve
    grid::grid.lines(
      x = x_vals,
      y = normal_curve,
      gp = grid::gpar(col = "blue", lwd = 2), # Line color and width
      default.units = "native"
    )

    ## Draw axes
    # Add x-axis
    grid::grid.xaxis(
      at = hist_data$breaks,  # Use histogram breaks for x-axis ticks
      label = round(hist_data$breaks, 2),  # Round labels for readability
      gp = grid::gpar(cex = 0.95),  # Adjust font size
      main = TRUE,  # Place the x-axis at the bottom
      edits = grid::gEdit(gPath = "labels", y = unit(-0.75, "lines"))  # Move labels closer to the axis
    )

    # Add node name and sample size below the histogram
    grid::grid.text(
      label = paste0("Node ", node_id, " (n = ", length(factor_scores), ")"),
      x = grid::unit(0.5, "npc"),  # Centered horizontally
      y = grid::unit(1, "npc") - unit(-0.9, "lines"),  # Position at the top of the plot
      gp = grid::gpar(fontface = "bold", cex = 1.0)  # Bold font with slightly reduced size
    )

    # Pop the viewport for this node
    grid::popViewport(2)
  }
}

node_regionplot_grmtree <- function(mobobj, names = FALSE, abbreviate = TRUE, type = c("mode", "median", "mean"),
                                     ref = NULL, ylim = NULL, off = 0.1, col_fun = grDevices::gray.colors, bg = "white",
                                     ylines = 2, spacing = 0.2) {
  ## Check input
  stopifnot(!is.null(mobobj))
  stopifnot(off >= 0)
  type <- match.arg(type)

  # Internal threshold extraction
  grm_threshpar_list <- function(object, node = NULL, ...) {
    if (is.null(node)) {
      node <- partykit::nodeids(object, terminal = TRUE)
    }
    # Extract thresholds for each node
    delta_lst <- lapply(node, function(n) {
      # Extract the model for the given node
      model <- partykit::nodeapply(object, ids = n, FUN = function(nd) nd$info$object)[[1]]

      # Extract threshold coefficients
      coef_model <- mirt::coef(model, IRTpars=T, simplify=TRUE)
      thresh <- coef_model$items[, grep("b", colnames(coef_model$items))]

      # Convert to a list of vectors (one per item)
      thresh_list <- split(thresh, seq(nrow(thresh)))
      names(thresh_list) <- rownames(thresh)
      return(thresh_list)
    })
    # Name the list by node IDs
    names(delta_lst) <- node
    return(delta_lst)
  }

  ## Setup parameters
  node <- partykit::nodeids(mobobj, terminal = TRUE)
  delta_lst <- grm_threshpar_list(mobobj, node = node)
  m <- max(sapply(delta_lst, length))
  xi <- 0:m + c(0:(m - 1), m - 1) * off
  xlim <- c(xi[1], xi[m + 1])

  ## Setup axis range
  if (is.null(ylim)) {
    ylim <- grDevices::extendrange(unlist(lapply(delta_lst, function(x) unlist(x)), use.names = FALSE), f = 0.25)
  }

  ## Labeling
  if (isTRUE(names)) {
    names <- lapply(delta_lst, names)
  } else if (is.character(names)) {
    names <- split(rep(names, length(node)), f = rep(1:length(node), each = length(names)))
  } else {
    ncf <- lapply(delta_lst, length)
    names <- lapply(ncf, function(m) {
      lab <- rep("", m)
      lab[c(1, m)] <- c(1, m)
      pr <- pretty(1:m, n = 4)
      pr <- pr[pr > 1 & pr < m]
      lab[pr] <- pr
      lab
    })
    abbreviate <- FALSE
  }

  ## Abbreviation
  if (is.logical(abbreviate)) {
    nlab <- max(unlist(lapply(names, function(j) nchar(j))))
    abbreviate <- if (abbreviate) as.numeric(cut(nlab, c(-Inf, 1.5, 4.5, 7.5, Inf))) else nlab
  }
  names <- lapply(names, function(j) abbreviate(j, abbreviate))

  ## Label for extraction
  names(names) <- names(delta_lst) <- node

  ## Panel function
  panelfun <- function(node) {
    ## Select node ID, coefficients, identified items, x-position vector, and label
    id <- as.character(partykit::id_node(node))
    delta_unsorted <- delta_lst[[id]]
    lab <- paste("node", id, sep = "")
    namesi <- names[[id]]

    ## Sort thresholds for display
    delta_sorted <- lapply(delta_unsorted, sort)



    ## Original panel setup (your preferred layout)
    top.vp <- grid::viewport(layout = grid::grid.layout(nrow = 2, ncol = 1,
                                            widths = unit(1, "null"),
                                            heights = unit(c(1, 1), c("lines", "null"))),
                       width = unit(0.9, "npc"),
                       height = unit(1, "npc") - unit(3, "lines"),
                       name = paste(lab, "_effects", sep = ""))
    grid::pushViewport(top.vp)
    grid::grid.rect(gp = gpar(fill = bg, col = 0), name = paste(lab, "_border", sep = ""))

    ## main title
    grid::pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 1, name = paste(lab, "_title_vp", sep = "")))
    grid::grid.text(paste("Node ", id, " (n = ", partykit::info_node(node)$nobs, ")", sep = ""), name = paste(lab, "_title", sep = ""))
    grid::upViewport()

    ## Plot area
    grid::pushViewport(viewport(layout.pos.col = 1, layout.pos.row = 2, name = lab))
    lab <- paste(lab, "_plot", sep = "")

    ## Setup plotting area (3x3 grid)
    wcol <- c(ylines, 1, 1)
    hrow <- c(0.5, 1, 1)

    top.vp <- grid::viewport(layout = grid.layout(nrow = 3, ncol = 3, widths = unit(wcol, c("lines", "null", "lines")), heights = unit(hrow, c("lines", "null", "lines"))), name = paste(lab, "_top_vp", sep = ""))
    bmargin.vp <- grid::viewport(layout.pos.row = 3, layout.pos.col = 2, name = paste(lab, "_bottom-margin_vp", sep = ""))
    lmargin.vp <- grid::viewport(layout.pos.row = 2, layout.pos.col = 1, name = paste(lab, "_left-margin_vp", sep = ""))
    rmargin.vp <- grid::viewport(layout.pos.row = 2, layout.pos.col = 3, name = paste(lab, "_right-margin_vp", sep = ""))
    plot.vp <- grid::viewport(layout.pos.row = 2, layout.pos.col = 2, name = paste(lab, "_vp", sep = ""), xscale = xlim, yscale = ylim)
    grid::pushViewport(top.vp)
    grid::pushViewport(plot.vp)

    ## plot rectangles per item
    for (j in seq_along(delta_sorted)) {
      ncat <- length(delta_sorted[[j]]) + 1
      grid::grid.rect(x = rep.int(xi[j], ncat), y = c(ylim[1], delta_sorted[[j]]), width = rep.int(1, ncat),
                height = diff.default(c(ylim[1], delta_sorted[[j]], ylim[2])), just = c("left", "bottom"),
                gp = grid::gpar(fill = col_fun(ncat)), default.units = "native", name = paste(lab, "_item", j, "_rect", sep = ""))
    }



    ## add box and axis
    grid::grid.rect(name = paste(lab, "_plot-box", sep = ""), gp = gpar(fill = NA))
    grid::grid.xaxis(at = (xi[-(m+1)] + 0.5), label = namesi, main = TRUE, name = paste(lab, "_xaxis-bottom", sep = ""))
    grid::grid.yaxis(main = TRUE, name = paste(lab, "_yaxis-left", sep = ""))
    grid::upViewport()

    ## add descriptions
    grid::pushViewport(lmargin.vp)
    grid::upViewport(2)

    ## go back to uper vp
    grid::upViewport(2)
  }

  ## Return panel function
  return(panelfun)
}
class(node_regionplot_grmtree) <- "grapcon_generator"


## Profile plot
node_profileplot_grmtree <- function(mobobj, what = c("item", "threshold", "discrimination"),
                                     parg = list(type = NULL, ref = NULL, alias = TRUE, logit = FALSE),
                                     id = TRUE, names = FALSE, abbreviate = TRUE, index = TRUE,
                                     ref = TRUE, col = "black", border = col, linecol = "black",
                                     refcol = "lightgray", bg = "white", cex = 0.5, pch = 21,
                                     xscale = NULL, yscale = NULL, ylines = 2, ...) {
  ## Check input
  what <- match.arg(what)
  refpar <- parg$ref
  alias <- if (is.null(parg$alias)) TRUE else parg$alias
  addargs <- list(...)
  logit <- parg$logit

  ## Node IDs (terminal nodes only)
  node <- partykit::nodeids(mobobj, terminal = TRUE)

  ## Get all coefficients (item, thresholds, or discrimination)
  cf <- lapply(node, function(n) {
    model <- partykit::nodeapply(mobobj, ids = n, FUN = function(nd) nd$info$object)[[1]]
    coef_model <- mirt::coef(model, IRTpars=T, simplify=TRUE)
    if (what == "threshold") {
      coef_model$items[, grep("b", colnames(coef_model$items))]  # Thresholds
    } else if (what == "discrimination") {
      coef_model$items[, "a"]  # Discrimination (use "a" instead of "a1")
    } else if (what == "item") {
      # Compute item parameters (discrimination and average thresholds)
      thresholds <- coef_model$items[, grep("b", colnames(coef_model$items))]
      avg_thresholds <- rowMeans(thresholds, na.rm = TRUE)
      item_params <- cbind(coef_model$items[, "a"], avg_thresholds)  # Discrimination and average thresholds
      colnames(item_params) <- c("Discrimination", "AvgThreshold")
      item_params
    }
  })
  names(cf) <- as.character(node)  # Ensure names are character

  ## Labeling
  if (isTRUE(names)) {
    nms <- lapply(cf, rownames)
  } else if (is.character(names)) {
    nms <- split(rep(names, length(node)), f = rep(1:length(node), each = length(names)))
  } else {
    ncf <- lapply(cf, NROW)
    nms <- lapply(ncf, function(m) {
      lab <- rep("", m)
      lab[c(1, m)] <- c(1, m)
      pr <- pretty(1:m, n = 4)
      pr <- pr[pr > 1 & pr < m]
      lab[pr] <- pr
      lab
    })
    abbreviate <- FALSE
  }

  ## Abbreviation
  if (is.logical(abbreviate)) {
    nlab <- max(unlist(lapply(nms, function(j) nchar(j))))
    abbreviate <- if (abbreviate) as.numeric(cut(nlab, c(-Inf, 1.5, 4.5, 7.5, Inf))) else nlab
  }
  nms <- lapply(nms, function(j) abbreviate(j, abbreviate))

  ## Axis scale
  if (is.null(xscale)) xscale <- c(0.5, max(sapply(cf, NROW)) + 0.5)  # Add padding for first/last item
  rg <- range(unlist(cf)[is.finite(unlist(cf))], na.rm = TRUE)
  r <- diff(rg)
  if (!r) r <- 1
  if (is.null(yscale)) yscale <- rg + c(-0.1, 0.1) * r

  ## Panel function for profile plots in nodes
  panelfun <- function(node) {
    ## Node index (ensure it's a terminal node)
    idn <- as.character(partykit::id_node(node))
    if (!idn %in% names(cf)) {
      stop("Node ID ", idn, " not found in the coefficients list. Ensure terminal nodes are used.")
    }

    ## Get coefficients and labels
    cfi <- cf[[idn]]
    nmsi <- nms[[idn]]

    ## Viewport setup
    top_vp <- grid::viewport(layout = grid::grid.layout(nrow = 2, ncol = 3,
                                            widths = unit(c(ylines, 1, 2), c("lines", "null", "lines")),
                                            heights = unit(c(1, 1), c("lines", "null"))),
                       width = unit(0.9, "npc"), height = unit(1, "npc") - unit(3, "lines"),
                       name = paste("node_profileplot", idn, sep = ""))
    grid::pushViewport(top_vp)
    grid::grid.rect(gp = gpar(fill = bg, col = 0))

    ## Main title
    top <- viewport(layout.pos.col = 2, layout.pos.row = 1)
    grid::pushViewport(top)
    mainlab <- paste(ifelse(id, paste("Node", idn, "(n = "), ""),
                     partykit::info_node(node)$nobs, ifelse(id, ")", ""), sep = "")
    grid::grid.text(mainlab)
    grid::popViewport()

    ## Actual plot
    plot_vpi <- viewport(layout.pos.col = 2, layout.pos.row = 2, xscale = xscale, yscale = yscale,
                         name = paste("node_profileplot", idn, "plot", sep = ""))
    grid::pushViewport(plot_vpi)

    ## Plot thresholds, discrimination, or item parameters
    if (what == "threshold") {
      for (j in 1:ncol(cfi)) {
        grid::grid.lines(1:nrow(cfi), cfi[, j], gp = gpar(col = linecol, lty = j), default.units = "native")
      }
    } else if (what == "discrimination") {
      grid::grid.lines(1:length(cfi), cfi, gp = gpar(col = linecol), default.units = "native")
      grid::grid.points(1:length(cfi), cfi, gp = gpar(col = border, fill = col, cex = cex), pch = pch, default.units = "native")
    } else if (what == "item") {
      # Plot discrimination and average thresholds
      grid::grid.lines(1:nrow(cfi), cfi[, "Discrimination"], gp = gpar(col = linecol, lty = 1), default.units = "native")
      grid::grid.lines(1:nrow(cfi), cfi[, "AvgThreshold"], gp = gpar(col = linecol, lty = 2), default.units = "native")
      grid::grid.points(1:nrow(cfi), cfi[, "Discrimination"], gp = gpar(col = border, fill = col, cex = cex), pch = pch, default.units = "native")
      grid::grid.points(1:nrow(cfi), cfi[, "AvgThreshold"], gp = gpar(col = border, fill = col, cex = cex), pch = pch + 1, default.units = "native")

      ## Add a compact legend in the top-right corner with line symbols and vertical spacing
      legend_vp <- grid::viewport(x = unit(0.95, "npc"), y = unit(1.0, "npc"),
                            width = unit(0.33, "npc"), height = unit(0.08, "npc"),  # Increased height for spacing
                            just = c("right", "top"))
      grid::pushViewport(legend_vp)
      grid::grid.rect(gp = gpar(fill = "white", col = "black", alpha = 0.8))  # Semi-transparent background

      # Add line symbols and labels with vertical spacing
      grid::grid.lines(x = unit(c(0.1, 0.3), "npc"), y = unit(0.75, "npc"),
                 gp = gpar(col = linecol, lty = 1), default.units = "npc")
      grid::grid.text("Discrim", x = unit(0.35, "npc"), y = unit(0.75, "npc"),
                just = "left", gp = gpar(col = linecol, fontface = "bold"))

      grid::grid.lines(x = unit(c(0.1, 0.3), "npc"), y = unit(0.3, "npc"),
                 gp = gpar(col = linecol, lty = 2), default.units = "npc")
      grid::grid.text("Threshold", x = unit(0.35, "npc"), y = unit(0.3, "npc"),
                just = "left", gp = gpar(col = linecol, fontface = "bold"))

      grid::popViewport()
    }

    ## Add axes
    grid::grid.xaxis(at = 1:length(nmsi), label = nmsi)
    grid::grid.yaxis(at = c(ceiling(yscale[1] * 100)/100, floor(yscale[2] * 100)/100))  # Improved y-axis
    grid::grid.rect(gp = gpar(fill = "transparent"))

    grid::upViewport(2)
  }

  return(panelfun)
}
class(node_profileplot_grmtree) <- "grapcon_generator"

#' Extract Discrimination Parameters from Longitudinal GRM Tree
#'
#' Extracts discrimination (slope) parameters for each unique item from all
#' terminal nodes of a longitudinal GRM tree. The longitudinal model uses
#' columns "a1" (T1 loading) and "a2" (T2 loading); this function extracts the
#' T1 discrimination since a1 = a2 under the equality constraint.
#'
#' @param object A \code{longitudinal_grmtree} object.
#' @param node Optional vector of node IDs. If NULL (default), extracts from all
#'   terminal nodes.
#' @param clean_names Logical. If TRUE (default), clean item names.
#' @param ... Additional arguments (currently unused).
#'
#' @return A data.frame with columns:
#'   \describe{
#'     \item{Node}{Terminal node ID}
#'     \item{Item}{Item name}
#'     \item{Discrimination}{Discrimination parameter (a1 from T1 items)}
#'   }
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Phase 1: fit the longitudinal GRM tree
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Print the discrimination parameters
#'   discr <- discrpar_longitudinal_grmtree(ltree)
#' print(discr)
#'
#' @seealso \code{\link{longitudinal_grmtree}} for Phase 1 (tree fitting),
#'   \code{\link{threshpar_longitudinal_grmtree}} for extracting threshold
#'   parameters for longitudinal GRMTree,
#'   \code{\link{itempar_longitudinal_grmtree}} for extracting item parameters
#'   for longitudinal GRMTree
#'
#' @export
#' @importFrom partykit nodeids nodeapply
#' @importFrom mirt coef
discrpar_longitudinal_grmtree <- function(object, node = NULL,
                                          clean_names = TRUE, ...) {
  if (!inherits(object, "longitudinal_grmtree")) {
    stop("'object' must be a longitudinal_grmtree object")
  }

  n_items <- object$info$n_items
  if (is.null(n_items)) stop("Tree object does not contain n_items.")

  if (is.null(node)) {
    node <- partykit::nodeids(object, terminal = TRUE)
    if (length(node) == 0) stop("No terminal nodes found")
  }

  discr_list <- lapply(node, function(n) {
    model <- partykit::nodeapply(
      object, ids = n, FUN = function(nd) nd$info$object
    )[[1]]

    coef_model <- mirt::coef(model, IRTpars = TRUE, simplify = TRUE)
    all_items <- coef_model$items

    # Longitudinal model has "a1" and "a2" columns, not "a"
    if ("a1" %in% colnames(all_items)) {
      discr <- all_items[1:n_items, "a1"]
    } else if ("a" %in% colnames(all_items)) {
      discr <- all_items[1:n_items, "a"]
    } else {
      stop("No discrimination parameters (a1 or a) found in node ", n)
    }

    items <- rownames(all_items)[1:n_items]
    if (clean_names) {
      items <- gsub("^.*MOS_|^.*resp_wide|^.*resp", "", items)
      items <- gsub("_BL$|_Pre$|_T1$|_baseline$", "", items)
    }

    data.frame(
      Node = n,
      Item = items,
      Discrimination = discr,
      row.names = NULL,
      stringsAsFactors = FALSE
    )
  })

  discrimination <- do.call("rbind", discr_list)
  rownames(discrimination) <- NULL
  return(discrimination)
}

#' Compute Latent Factor Scores for Longitudinal GRM Tree
#'
#' Computes latent factor scores (theta) for both T1 and T2 within each terminal
#' node of a longitudinal GRM tree. Unlike the cross-sectional
#' \code{\link{fscores_grmtree}} which returns a single theta per person, this
#' function returns two scores per person: \eqn{\theta_{T1}} and
#' \eqn{\theta_{T2}}.
#'
#' @param object A \code{longitudinal_grmtree} object.
#' @param method Scoring method: "EAP" (default), "MAP", "ML", or "WLE".
#' @param ... Additional arguments (currently unused).
#'
#' @return A named list (one element per terminal node). Each element is a
#'   \code{data.frame} with columns:
#'   \describe{
#'     \item{Theta_T1}{Estimated latent trait at Time 1}
#'     \item{Theta_T2}{Estimated latent trait at Time 2}
#'   }
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Phase 1: fit the longitudinal GRM tree
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Print the factor scores
#'   scores <- fscores_longitudinal_grmtree(ltree)
#'   # Scores for node 2
#'   head(scores[["2"]])
#'
#' @seealso \code{\link{longitudinal_grmtree}} for fitting the tree,
#'   \code{\link{generate_node_scores_dataset}} to generate node assignment and
#'   factor scores
#'
#' @export
#' @importFrom mirt fscores
#' @importFrom partykit nodeids nodeapply
fscores_longitudinal_grmtree <- function(object, method = "EAP", ...) {

  if (!inherits(object, "longitudinal_grmtree")) {
    stop("'object' must be a longitudinal_grmtree object")
  }
  if (!method %in% c("EAP", "MAP", "ML", "WLE")) {
    stop("method must be one of: 'EAP', 'MAP', 'ML', or 'WLE'")
  }

  terminal_nodes <- partykit::nodeids(object, terminal = TRUE)
  scores_list <- vector("list", length(terminal_nodes))
  names(scores_list) <- as.character(terminal_nodes)

  for (node_id in terminal_nodes) {
    message("Processing node: ", node_id)

    # Extract fitted model
    node_model <- tryCatch(
      partykit::nodeapply(
        object, ids = node_id,
        FUN = function(nd) nd$info$object
      )[[1]],
      error = function(e) NULL
    )
    if (is.null(node_model)) {
      warning("No model object in node ", node_id)
      next
    }

    # Compute factor scores — returns N x 2 matrix for two-factor model
    node_scores <- suppressWarnings(tryCatch({
      result <- mirt::fscores(node_model, method = method)
      if (is.matrix(result) && ncol(result) >= 2) {
        df <- data.frame(
          Theta_T1 = result[, 1],
          Theta_T2 = result[, 2]
        )
      } else if (is.matrix(result) && ncol(result) == 1) {
        df <- data.frame(Theta_T1 = result[, 1], Theta_T2 = NA)
        warning("Only one factor score column returned for node ", node_id)
      } else {
        df <- data.frame(Theta_T1 = result, Theta_T2 = NA)
      }
      df
    }, error = function(e) {
      warning("Factor score computation failed for node ", node_id, ": ", e$message)
      NULL
    }))

    if (!is.null(node_scores)) {
      scores_list[[as.character(node_id)]] <- node_scores
    }
  }

  scores_list <- Filter(Negate(is.null), scores_list)
  if (length(scores_list) == 0) {
    stop("No factor scores were successfully computed for any node.")
  }
  return(scores_list)
}

################################################################################
##
## Plot Methods for Longitudinal GRMTree
##
## Custom plotting functions for longitudinal_grmtree objects.
## The key difference from grmtree plots: the constrained longitudinal model
## has 2*n_items columns but item parameters are equal across T1 and T2,
## so we only display the first n_items (T1) to avoid redundancy.
##
## Author: Olayinka Arimoro
## Date: March 2026
##
################################################################################

#' Plot Method for Longitudinal GRM Tree Objects
#'
#' Visualizes a longitudinal GRM tree with threshold region plots in terminal
#' nodes. Unlike the cross-sectional \code{\link[grmtree]{plot.grmtree}},
#' this method displays only the unique item parameters (T1 items), since
#' the constrained longitudinal model enforces equal parameters across T1
#' and T2 within each node.
#'
#' @param x A \code{longitudinal_grmtree} object.
#' @param type Type of terminal node plot. Currently only \code{"regions"}
#'   is supported for the longitudinal model.
#' @param tnex Numeric scaling factor for terminal node extension (default: 2).
#' @param drop_terminal Logical indicating whether to drop terminal node IDs
#'   (default: TRUE).
#' @param names Logical or character vector. If \code{TRUE}, use item names
#'   from the response matrix. If a character vector, use as custom labels.
#'   If \code{FALSE} (default), use numeric indices 1 through n_items.
#' @param abbreviate Logical or numeric. If \code{TRUE}, abbreviate item
#'   names. If numeric, abbreviate to that many characters.
#' @param ... Additional arguments passed to the terminal panel function.
#'
#' @return Invisibly returns the tree object. Called for its side effect of
#'   producing a plot.
#'
#' @details
#' The region plot displays threshold parameters as colored horizontal bands
#' for each item within each terminal node. Darker shading represents lower
#' response categories and lighter shading represents higher categories.
#' The height of each band corresponds to the range of the latent trait
#' over which that response category is most likely.
#'
#' Because the constrained longitudinal GRM enforces
#' \eqn{a_{m,T1} = a_{m,T2}} and \eqn{b_{k,m,T1} = b_{k,m,T2}}, the T1
#' and T2 item parameters are identical. The plot therefore shows only the
#' \code{n_items} unique items rather than all \code{2 * n_items} columns
#' in the response matrix.
#'
#' @examplesIf interactive()
#' library(grmtree)
#'
#' # Load the synthetic longitudinal data
#' data("grmtree_long_data", package = "grmtree")
#'
#'   # Prepare the wide-format response matrix
#'   items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
#'                 "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
#'   ld <- prepare_longitudinal_data(
#'     data = grmtree_long_data,
#'     items_t1 = items_t1,
#'     items_t2 = paste0(items_t1, "_year1"),
#'     covariates = c("sex", "age", "residency", "job",
#'                    "education", "comorbidity_count", "ever_smoker")
#'   )
#'
#'   # Phase 1: fit the longitudinal GRM tree
#'   ltree <- longitudinal_grmtree(
#'     resp_wide ~ sex + age + residency + job +
#'       education + comorbidity_count + ever_smoker,
#'     data = ld, n_items = 8,
#'     control = grmtree.control(minbucket = 200)
#'   )
#'
#'   # Region plot with numeric labels
#'   plot(ltree)
#'
#'   # Region plot with item names
#'   plot(ltree, names = TRUE)
#'
#'   # Custom labels
#'   plot(ltree, names = c("Listen", "Info", "Crisis",
#'     "Confide", "Advice", "Fears", "Personal", "Understand"))
#'
#' @seealso \code{\link{longitudinal_grmtree}} for fitting the tree,
#'   \code{\link[grmtree]{plot.grmtree}} for cross-sectional tree plots
#'
#' @method plot longitudinal_grmtree
#' @import grid
#' @export
plot.longitudinal_grmtree <- function(x,
                                      type = "regions",
                                      tnex = 2L,
                                      drop_terminal = TRUE,
                                      names = FALSE,
                                      abbreviate = TRUE,
                                      ...) {

  if (!inherits(x, "longitudinal_grmtree")) {
    stop("The input object must be of class 'longitudinal_grmtree'.")
  }

  type <- match.arg(type, choices = c("regions"))

  n_items <- x$info$n_items
  if (is.null(n_items)) {
    stop("Tree object does not contain n_items. Was it fit with longitudinal_grmtree()?")
  }

  # Build the terminal panel
  terminal_panel <- node_regionplot_longitudinal(
    x, n_items = n_items, names = names, abbreviate = abbreviate, ...
  )

  # Use partykit's plot infrastructure
  partykit::plot.modelparty(
    x,
    terminal_panel = terminal_panel,
    tnex = tnex,
    drop_terminal = drop_terminal,
    ...
  )

  invisible(x)
}


#' Region Plot Panel for Longitudinal GRM Tree Nodes
#'
#' Internal function that generates the terminal node panel function for
#' region plots of longitudinal GRM trees. Extracts only T1 item thresholds
#' (items 1 through n_items) since T2 items have identical constrained
#' parameters.
#'
#' @param mobobj A \code{longitudinal_grmtree} object.
#' @param n_items Number of items per time point.
#' @param names Logical, character vector, or FALSE for labeling.
#' @param abbreviate Logical or numeric for name abbreviation.
#' @param ylim Optional y-axis limits.
#' @param off Offset between items (default: 0.1).
#' @param col_fun Color function for response categories (default:
#'   \code{gray.colors}).
#' @param bg Background color (default: "white").
#' @param ylines Width of y-axis margin in lines (default: 2).
#' @param ... Additional arguments (unused).
#'
#' @return A panel function suitable for use with
#'   \code{\link[partykit]{plot.modelparty}}.
#'
#' @keywords internal
node_regionplot_longitudinal <- function(mobobj, n_items, names = FALSE,
                                         abbreviate = TRUE,
                                         ylim = NULL, off = 0.1,
                                         col_fun = grDevices::gray.colors,
                                         bg = "white", ylines = 2, ...) {

  stopifnot(!is.null(mobobj))
  stopifnot(off >= 0)

  # ---- Extract T1-only thresholds for each terminal node ----
  node_ids <- partykit::nodeids(mobobj, terminal = TRUE)

  delta_lst <- lapply(node_ids, function(n) {
    model <- partykit::nodeapply(
      mobobj, ids = n, FUN = function(nd) nd$info$object
    )[[1]]

    coef_model <- mirt::coef(model, IRTpars = TRUE, simplify = TRUE)
    all_items <- coef_model$items
    thresh_cols <- grep("^b", colnames(all_items))

    # Take only T1 items (rows 1:n_items)
    thresh <- all_items[1:n_items, thresh_cols, drop = FALSE]

    # Convert to list of vectors (one per item)
    thresh_list <- split(thresh, seq_len(nrow(thresh)))

    # Get clean item names from T1 rows
    raw_names <- rownames(all_items)[1:n_items]
    names(thresh_list) <- raw_names

    return(thresh_list)
  })
  names(delta_lst) <- as.character(node_ids)

  # ---- Setup x-axis parameters ----
  m <- n_items
  xi <- 0:m + c(0:(m - 1), m - 1) * off
  xlim <- c(xi[1], xi[m + 1])

  # ---- Setup y-axis range ----
  if (is.null(ylim)) {
    all_vals <- unlist(lapply(delta_lst, function(x) unlist(x)), use.names = FALSE)
    ylim <- grDevices::extendrange(all_vals[is.finite(all_vals)], f = 0.25)
  }

  # ---- Setup labels ----
  if (isTRUE(names)) {
    # Use item names from the model, cleaned up
    lab_list <- lapply(delta_lst, function(dl) {
      raw <- names(dl)
      # Remove common prefixes like "resp_wideMOS_" or "mos_resp_wideMOS_"
      cleaned <- gsub("^.*MOS_", "", raw)
      cleaned <- gsub("_BL$|_Pre$|_T1$", "", cleaned)
      cleaned
    })
  } else if (is.character(names)) {
    # User-provided labels
    lab_list <- lapply(node_ids, function(n) names)
    names(lab_list) <- as.character(node_ids)
  } else {
    # Numeric labels
    lab_list <- lapply(node_ids, function(n) {
      lab <- rep("", m)
      lab[c(1, m)] <- c(1, m)
      pr <- pretty(1:m, n = 4)
      pr <- pr[pr > 1 & pr < m]
      lab[pr] <- pr
      lab
    })
    names(lab_list) <- as.character(node_ids)
    abbreviate <- FALSE
  }

  # ---- Abbreviate if needed ----
  if (is.logical(abbreviate) && abbreviate) {
    nlab <- max(unlist(lapply(lab_list, nchar)))
    abbr_len <- as.numeric(cut(nlab, c(-Inf, 1.5, 4.5, 7.5, Inf)))
    lab_list <- lapply(lab_list, function(j) abbreviate(j, abbr_len))
  } else if (is.numeric(abbreviate)) {
    lab_list <- lapply(lab_list, function(j) abbreviate(j, abbreviate))
  }

  # ---- Panel function ----
  panelfun <- function(node) {
    id <- as.character(partykit::id_node(node))
    delta_unsorted <- delta_lst[[id]]
    namesi <- lab_list[[id]]
    lab <- paste("node", id, sep = "")

    # Sort thresholds within each item for display
    delta_sorted <- lapply(delta_unsorted, sort)

    # ---- Viewport setup ----
    top.vp <- grid::viewport(
      layout = grid::grid.layout(
        nrow = 2, ncol = 1,
        widths = grid::unit(1, "null"),
        heights = grid::unit(c(1, 1), c("lines", "null"))
      ),
      width = grid::unit(0.9, "npc"),
      height = grid::unit(1, "npc") - grid::unit(3, "lines"),
      name = paste(lab, "_effects", sep = "")
    )
    grid::pushViewport(top.vp)
    grid::grid.rect(gp = grid::gpar(fill = bg, col = 0),
                    name = paste(lab, "_border", sep = ""))

    # ---- Title ----
    grid::pushViewport(grid::viewport(
      layout.pos.col = 1, layout.pos.row = 1,
      name = paste(lab, "_title_vp", sep = "")
    ))
    grid::grid.text(
      paste("Node ", id, " (n = ", partykit::info_node(node)$nobs, ")", sep = ""),
      name = paste(lab, "_title", sep = "")
    )
    grid::upViewport()

    # ---- Plot area ----
    grid::pushViewport(grid::viewport(
      layout.pos.col = 1, layout.pos.row = 2,
      name = lab
    ))
    lab <- paste(lab, "_plot", sep = "")

    wcol <- c(ylines, 1, 1)
    hrow <- c(0.5, 1, 1)

    top.vp2 <- grid::viewport(
      layout = grid::grid.layout(
        nrow = 3, ncol = 3,
        widths = grid::unit(wcol, c("lines", "null", "lines")),
        heights = grid::unit(hrow, c("lines", "null", "lines"))
      ),
      name = paste(lab, "_top_vp", sep = "")
    )
    plot.vp <- grid::viewport(
      layout.pos.row = 2, layout.pos.col = 2,
      name = paste(lab, "_vp", sep = ""),
      xscale = xlim, yscale = ylim
    )
    grid::pushViewport(top.vp2)
    grid::pushViewport(plot.vp)

    # ---- Draw region rectangles ----
    for (j in seq_along(delta_sorted)) {
      ncat <- length(delta_sorted[[j]]) + 1
      grid::grid.rect(
        x = rep.int(xi[j], ncat),
        y = c(ylim[1], delta_sorted[[j]]),
        width = rep.int(1, ncat),
        height = diff.default(c(ylim[1], delta_sorted[[j]], ylim[2])),
        just = c("left", "bottom"),
        gp = grid::gpar(fill = col_fun(ncat)),
        default.units = "native",
        name = paste(lab, "_item", j, "_rect", sep = "")
      )
    }

    # ---- Axes and box ----
    grid::grid.rect(name = paste(lab, "_plot-box", sep = ""),
                    gp = grid::gpar(fill = NA))
    grid::grid.xaxis(
      at = (xi[-(m + 1)] + 0.5),
      label = namesi,
      main = TRUE,
      name = paste(lab, "_xaxis-bottom", sep = "")
    )
    grid::grid.yaxis(main = TRUE, name = paste(lab, "_yaxis-left", sep = ""))
    grid::upViewport()

    # ---- Clean up viewports ----
    grid::pushViewport(grid::viewport(
      layout.pos.row = 2, layout.pos.col = 1,
      name = paste(lab, "_left-margin_vp", sep = "")
    ))
    grid::upViewport(2)
    grid::upViewport(2)
  }

  return(panelfun)
}
class(node_regionplot_longitudinal) <- "grapcon_generator"

#' Extract Threshold Parameters from GRM Tree
#'
#' Extracts threshold parameters for each item from all terminal nodes of a
#' graded response model tree. The thresholds represent the points on the latent
#' trait continuum where the probability of scoring in adjacent response
#' categories is equal.
#'
#' @param object A `grmtree` object.
#' @param node Optional vector of node IDs to extract from. If NULL (default),
#'   extracts from all terminal nodes.
#' @param ... Additional arguments (currently unused).
#'
#' @return A data.frame with threshold parameters for each item in each node,
#'   with columns: \item{Node}{Node ID} \item{Item}{Item name} \item{d1, d2,
#'   ...}{Threshold parameters for each category}
#'
#' @examples
#' \donttest{
#'   library(grmtree)
#'   library(hlt)
#'
#'   data("asti", package = "hlt")
#'   asti$resp <- data.matrix(asti[, 1:4])
#'
#'   # Fit GRM tree with gender and group as partitioning variables
#'   tree <- grmtree(resp ~ gender + group,
#'           data = asti,
#'           control = grmtree.control(minbucket = 30))
#'
#'   # Get all thresholds
#'   thresholds <- threshpar_grmtree(tree)
#'   print(thresholds)
#'
#' }
#'
#' @seealso \code{\link{grmtree}} fits a Graded Response Model Tree,
#' \code{\link{grmforest}} for GRM Forests, \code{\link{fscores_grmtree}} for
#' computing factor scores, \code{\link{discrpar_grmtree}} for extracting
#' discrimination parameters, \code{\link{itempar_grmtree}} for extracting item
#' parameters
#'
#' @export
#' @importFrom partykit nodeids nodeapply
#' @importFrom mirt coef
threshpar_grmtree <- function(object, node = NULL, ...) {
  # Validate input
  if (!inherits(object, "grmtree")) {
    stop("'object' must be a grmtree object")
  }

  # Get terminal nodes if not specified
  if (is.null(node)) {
    node <- partykit::nodeids(object, terminal = TRUE)
    if (length(node) == 0) {
      stop("No terminal nodes found in tree")
    }
  }

  # Validate node IDs
  all_nodes <- partykit::nodeids(object)
  invalid_nodes <- setdiff(node, all_nodes)
  if (length(invalid_nodes) > 0) {
    stop("Invalid node IDs: ", paste(invalid_nodes, collapse = ", "))
  }

  # Extract thresholds for each node
  thresholds <- do.call("rbind", lapply(node, function(n) {
    # Get model from node
    model <- tryCatch(
      partykit::nodeapply(object, ids = n, FUN = function(nd) nd$info$object)[[1]],
      error = function(e) {
        stop("Failed to extract model from node ", n, ": ", e$message)
      }
    )

    # Get coefficients
    coef_model <- tryCatch(
      mirt::coef(model, IRTpars=T, simplify = TRUE),
      error = function(e) {
        stop("Failed to extract coefficients from node ", n, ": ", e$message)
      }
    )

    # Extract thresholds
    thresh_cols <- grep("^b", colnames(coef_model$items))
    if (length(thresh_cols) == 0) {
      stop("No threshold parameters found in node ", n)
    }

    thresh <- coef_model$items[, thresh_cols, drop = FALSE]
    items <- rownames(coef_model$items)

    # Create output data frame
    thresh_df <- as.data.frame(thresh)
    thresh_df$Item <- items
    thresh_df$Node <- n

    # Reorder columns
    thresh_df <- thresh_df[, c("Node", "Item", colnames(thresh)), drop = FALSE]

    return(thresh_df)
  }))

  # Reset row names
  rownames(thresholds) <- NULL

  return(thresholds)
}


# Internal non-exported version of threshold extraction
grm_threshpar <- function(object, node = NULL, ...) {
  if (!inherits(object, "grmtree")) {
    stop("'object' must be a grmtree object")
  }

  if (is.null(node)) {
    node <- partykit::nodeids(object, terminal = TRUE)
  }

  thresholds <- tryCatch(
    do.call("rbind", partykit::nodeapply(object, ids = node, FUN = function(n) {
      model <- partykit::info_node(n)$object
      coef_model <- mirt::coef(model, IRTpars=T, simplify=TRUE)
      coef_model$items[, grep("^b", colnames(coef_model$items)), drop = FALSE]
    })),
    error = function(e) {
      stop("Failed to extract thresholds: ", e$message)
    }
  )

  return(thresholds)
}

library(testthat)
library(mirt)
library(hlt)

## Test that threshpar_grmtree works with valid inputs
test_that("threshpar_grmtree works with valid inputs", {

  cat("\n=== Starting threshpar test ===\n")
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  ## ASTI data
  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  ## Create the grmtree object
  tree <- grmtree(resp ~ gender + group, data = asti)

  # Default usage
  expect_silent(thresh <- threshpar_grmtree(tree))
  expect_s3_class(thresh, "data.frame")
  expect_true(nrow(thresh) > 0)
  expect_true(all(c("Node", "Item") %in% names(thresh)))

  # Specific nodes
  nodes <- partykit::nodeids(tree, terminal = TRUE)
  if (length(nodes) > 1) {
    expect_silent(thresh_sub <- threshpar_grmtree(tree, node = nodes[1:2]))
    expect_equal(unique(thresh_sub$Node), nodes[1:2])
  }
  cat("Current objects:", ls(), "\n")
})


## Test that threshpar_grmtree handles errors appropriately
test_that("threshpar_grmtree handles errors appropriately", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti[1:1000,])

  # Invalid object
  expect_error(threshpar_grmtree(list()), "must be a grmtree object")

  # Invalid nodes
  nodes <- partykit::nodeids(tree, terminal = TRUE)
  expect_error(threshpar_grmtree(tree, node = 999), "Invalid node IDs")

})

## Test that apply_to_models helper works correctly
test_that("apply_to_models helper works correctly", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")


  data(asti, package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti)

  # Default usage
  expect_silent(models <- apply_to_models(tree))
  expect_type(models, "list")
  expect_true(length(models) > 0)

  # With custom function
  expect_silent(coefs <- apply_to_models(tree, FUN = function(x) mirt::coef(x, simplify = TRUE)))
  expect_true(all(sapply(coefs, is.list)))

  # With specific nodes
  nodes <- partykit::nodeids(tree, terminal = TRUE)[1:2]
  expect_silent(sub_models <- apply_to_models(tree, node = nodes))
  expect_equal(length(sub_models), length(nodes))

  # Error cases
  expect_error(apply_to_models(list()), "must be a grmtree object")
  expect_error(apply_to_models(tree, FUN = "not_a_function"), "must be a function")
})


## Test that the internal grm_threshpar works
test_that("internal grm_threshpar works", {
  skip_if_not_installed("mirt")
  skip_if_not_installed("hlt")

  data(asti, package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  tree <- grmtree(resp ~ gender + group, data = asti)

  expect_silent(thresh <- grm_threshpar(tree))
  expect_true(is.matrix(thresh))
  expect_true(nrow(thresh) > 0)

  nodes <- partykit::nodeids(tree, terminal = TRUE)
  if (length(nodes) > 1) {
    expect_silent(thresh_sub <- grm_threshpar(tree, node = nodes[1:2]))
    expect_equal(nrow(thresh_sub), length(nodes[1:2]) * ncol(asti[, 1:4]))
  }
})

#' Medical Outcomes Study Social Support Survey (MOS-SS) Test Data
#'
#' A dataset containing sample responses to the MOS-SS emotional domain items
#' and demographic variables. This data is provided for testing and
#' demonstration purposes within the grmtree package. The items are numbered
#' 1-5, representing None of the time,	A little of the time,	Some of the
#' time,	Most of the time, All of the time, respectively.
#'
#' @format A tibble with 3,500 rows and 17 variables:
#' \describe{
#'   \item{MOS_Listen}{Someone you can count on to listen to you when you need to talk (1-5 Likert scale)}
#'   \item{MOS_Info}{Someone to give you information to help you understand a situation (1-5 Likert scale)}
#'   \item{MOS_Advice_Crisis}{Someone to give good advice about a crisis (1-5 Likert scale)}
#'   \item{MOS_Confide}{Someone to confide in or talk to about yourself or your problems (1-5 Likert scale)}
#'   \item{MOS_Advice_Want}{Someone whose advice you really want (1-5 Likert scale)}
#'   \item{MOS_Fears}{Someone to share private worries or fears (1-5 Likert scale)}
#'   \item{MOS_Personal}{Someone to turn to for suggestions about how to deal with a personal problem (1-5 Likert scale)}
#'   \item{MOS_Understand}{Someone who understands your problems (1-5 Likert scale)}
#'   \item{sex}{Gender (Male, Female)}
#'   \item{age}{Age in years (numeric)}
#'   \item{residency}{Residence location (rural, urban)}
#'   \item{depressed}{Depression status (No, Yes)}
#'   \item{bmi}{Body Mass Index (numeric)}
#'   \item{Education}{Education level (Primary/High school, College/University)}
#'   \item{job}{Employment status (Employed, Unemployed)}
#'   \item{smoker}{Smoking status (No, Yes)}
#'   \item{multimorbidity}{Number of chronic conditions (0, 1, 2+)}
#' }
#'
#' @examples
#' \donttest{
#'   library(dplyr)
#'
#'   # Load and take a glimpse at the data
#'   data(grmtree_data, package = "grmtree")
#'   glimpse(grmtree_data)
#' }
#' @source Simulated data generated for package testing and demonstration
"grmtree_data"

---
title: "Getting Started with the grmtree Package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with the grmtree Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

# Introduction

The `grmtree` package implements recursive partitioning for Graded Response Models (GRM), allowing researchers to test for differential item functioning (DIF) and identify heterogeneous subgroups in their data based on item response patterns and covariates. This vignette describes the implementation of the tree-based graded response model (GRMTree) to test for DIF on the sample Medical Outcomes Study Social Support Survey (MOS-SS). This vignette demonstrates:

* Data preparation and exploration

* Unidimensionality checking

* GRM fitting

* GRMTree construction and interpretation

# Install and Load required packages
To implement the tree-based GRM (GRMTree), you will install the following packages if not previously installed.
```{r, eval=F}
## Install packages from CRAN repository
install.packages(c("dplyr", "grmtree", "mirt", "psych"))
```

Once installed, load the packages as follows:
```{r, message=FALSE, warning=FALSE}
library(dplyr)        # For data manipulation
library(grmtree)      # For tree-based GRM DIF Test
library(mirt)         # For traditional GRM
library(psych)        # For psychometrics
```

# Import and explore the data

The data set used in this demonstration is a test/sample data for the package.

**Now, let's load this tidy data set into R.**

```{r, message=FALSE}
## Load the data
data("grmtree_data", package = "grmtree")

## Take a glimpse at the data
glimpse(grmtree_data)
```

# IRT Assumptions: Check for unidimensionality assumption

There are various ways to check for the unidimensionality assumption. Some including using exploratory factor analysis (through parallel analysis), scree plot of the eigen values, and contrasts from principal component analysis.

Here, we will use the scree plot of eigen values. Reeve, et al. suggested that a ratio of first-to-second eigenvalues greater than four is evidence of unidimensionality.

```{r, warning=FALSE}
## Create the response data (the 8 MOS-SS items)
response_data <- grmtree_data %>% 
  dplyr::select(MOS_Listen:MOS_Understand) %>% 
  mutate_at(vars(starts_with("MOS")), as.ordered)

## Create response as outcomes
response_data$resp <- data.matrix(response_data[, 1:8])
```

```{r, eval=FALSE}
## Calculate the polychoric correlation of items 
polycorr_mos <- polychoric(response_data$resp, global=FALSE)$rho

## Return the eigen values
polycorr_eigen_mos <- eigen(polycorr_mos)$values
round(polycorr_eigen_mos, 3)

## Ratio of first and second eigen value
mos_ratio <- round(round(polycorr_eigen_mos, 3)[1]/round(polycorr_eigen_mos, 3)[2],3)

## Print the result
mos_ratio

cat("The ratio of the first-to-second eigen value is", mos_ratio,
    "which is >4 suggesting that the unidimensionality assumption is satisfied", "\n")
```

Now, let's create the screen plot.

```{r, eval=FALSE}
## Scree plot of eigen values
plot(1:length(polycorr_eigen_mos), polycorr_eigen_mos, 
     type = "b", pch = 20, xlab = "", ylab = "Eigen values")
```

**Note:** The `type = "b"` argument means we desire that the plot creates both points and lines, 

Based on the scree plot, we assume that the MOS-SS emotional domain is unidimensional. This implies that one dominant latent trait is being measured and that this trait is the driving force for the responses observed for each item in the measure.

Other ways to check for unidimensionality of the MOS-SS emotional domain are shown below
```{r, eval=FALSE}
## Perform EFA with 1 factors
efa_mos <- fa(response_data$resp, nfactors = 1, rotate = "varimax", fm = "mle")

## Print the results
summary(efa_mos)
print(efa_mos, sort=TRUE)
efa_mos$loadings

## Visualizing factor structure using fa.diagram
fa.diagram(efa_mos, main = "EFA Factor Structure")

## Scree plot to visualize the number of factors
fa.parallel(response_data$resp, fa = "fa")
```


# Fit the graded response model (GRM)
Having established unidimensionality, let's create the GRM.
```{r, eval=FALSE}
## Create GRM
mos_grm <- mirt(data = response_data$resp, 
                model = 1,
                itemtype = "graded", SE = TRUE, method = "EM",
                verbose = FALSE)
```

**Note:** The argument `model = 1` means that default is 1, indicating that a unidimensional model will be fitted

## Fit Indices for the GRM
```{r, eval=FALSE}
## Get the coefficients
mos_coef <- coef(mos_grm, IRTpars = T, simplify = TRUE)
mos_coef

## Get the residuals
residuals(mos_grm, type = "Q3")

## Compute the M2 model fit statistic
M2(mos_grm, type = "C2")

## Infit and outfit statistics
mirt::itemfit(mos_grm, c('S_X2', 'infit'), method = 'EAP')

## Could also use method = 'ML'
```

## Visualization: Visualize the results for all the items

```{r, eval = FALSE}
## Plot the category response curves
plot(mos_grm, type = 'trace')

## Plot the test information
plot(mos_grm, type = 'info')
```

# Fit the tree-based graded response model (GRMTree)
Based on multiple fit indices including the Tucker–Lewis index (TLI) and Comparative fit index (CFI), we conclude that GRM has a good fit to the data.

Now, let's create the GRNTree using covariates to test for DIF. To implement the GRMTree, it follows this four-step process.

```{r grmtree-process, echo=FALSE, out.width="75%", fig.align='center', fig.cap="GRMTree implementation process showing the four-step recursive partitioning approach for detecting differential item functioning"}
knitr::include_graphics("GRMTree.png")
```

Before we proceed, let's put the variables in their proper formats:

* All variables to be included in the model except `age` and `BMI` will be converted to factor since they are categorical variables and the MOS-SS emotional domain items will be converted to ordinal variables as required for the GRMTree.

```{r, warning=FALSE}

## Prepare the data
resp.data <- grmtree_data %>% 
  mutate_at(vars(starts_with("MOS")), as.ordered) %>% 
  mutate_at(vars(c(sex, residency, depressed,
                   Education, job, smoker,
                   multimorbidity)), as.factor) 

## Explore the data
head(resp.data)

## Check the structure of the data
glimpse(resp.data)

## Create response as outcomes
resp.data$resp <- data.matrix(resp.data[, 1:8])

## GRMTree control parameters with Benjamini-Hochberg
grm_control <- grmtree.control(
  minbucket = 350,
  p_adjust = "BH", alpha = 0.05)
```


```{r, eval=FALSE}
## Fit the GRMTree model
mos_grmtree <- grmtree(resp ~ sex + age + bmi + Education + depressed +
                       residency + job + multimorbidity + smoker,  
                       data = resp.data,
                       control = grm_control)
```

**Note:** Unlike previous tree-based item response theory models, currently, the `grmtree` package implemented other post-hoc multiple adjustments methods and also the Bonferroni correction. The implemented post-hoc multiple adjustments methods include, "bonferroni" (Bonferroni), "holm" (Holm-Bonferroni), "BH" (Benjamini-Hochberg), "BY" (Benjamini-Yekutieli), "hochberg" (Hochberg), and "hommel" (Hommel).

See `?grmtree` and `?grmtree.control` for more information on other control arguments. Also see below an example of using other multiple comparison methods with the control parameters in the function.

```{r, eval=FALSE}
## Bonferroni correction
tree_bonf <- grmtree(response ~ covariate1 + covariate2, data = df, 
                    control = grmtree.control(p_adjust = "bonferroni"))

## Hommel
tree_bh <- grmtree(response ~ covariate1 + covariate2, data = df,
                  control = grmtree.control(p_adjust = "hommel"))

## Holm-Bonferroni
tree_holm <- grmtree(response ~ covariate1 + covariate2, data = df,
                    control = grmtree.control(p_adjust = "holm"))
```

## Print the tree
We can print the tree using the `print()` function. Note that using `print(mos_grmtree)` will print the tree with the item parameters. The code below also includes a way to print a simplified version of the tree showing only the nodes.

```{r, eval=FALSE}
## Print the tree model
print(mos_grmtree)
print(mos_grmtree, FUN = function(x) " *")
```

## Plot the GRMTree
Now, we can plot the tree using the `plot()` function. We have implemented several plotting options including plot the distribution of the factor scores in each node overlayed by a normal curve. See `?plot` for options of plots available.
```{r, eval=FALSE}
## Create the regions plot (by default)
plot(mos_grmtree)

## This also creates a regions plot
plot(mos_grmtree, type = "regions",  tnex = 2L)

## Create the histogram plot of the factor scores
plot(mos_grmtree, type = "histogram",  tnex = 2L)

## Create the profile plot with different options
plot(mos_grmtree, type = "profile", tnex = 2L, what = "threshold")
plot(mos_grmtree, type = "profile", tnex = 2L, what = "discrimination")
plot(mos_grmtree, type = "profile", tnex = 2L, what = "item")
```

**Note:** The `tnex = 2L` argument is a numeric value giving the terminal node extension in relation to the inner nodes. 

You may also rename a covariate in the plot. Let's assume we want to rename the variable that splits at the root node from `age` to `Age`.

```{r, eval=FALSE}
## Return the names of the covariates to know the position of age
names(mos_grmtree$data)

## Rename the age to Age (Uncomment the code below and change to the correct name)
#names(mos_grmtree$data)[3] <- "Age"

## Create the regions plot (by default)
plot(mos_grmtree)
```
More than one subgroup (terminal node) was identified, suggesting the presence of sample heterogeneity with respect to DIF on the MOS-SS emotional domain items. Specifically, the GRMTree identified three distinct subgroups defined by interactions among age, sex and smoking status as splitting variables. Subgroup 1 consists of female patients who are ≤69 years old, subgroup 2 consists of males who are ≤69 years, and subgroup 3 consists of patients who are > 69 years. In the figure above, the larger the sections of the bars for each response category, the more likely patients are to endorse that category. For a more detailed example on the interpretation of these types of plot, see the article below.

* Arimoro, O.I., Josephson, C.B., James, M.T. et al. Screening for depression in patients with epilepsy: same questions but different meaning to different patients. Qual Life Res 33, 3409–3419 (2024). https://doi.org/10.1007/s11136-024-03782-1.

## Extract item parameters for terminal nodes
```{r, eval=FALSE}
## Extract and print the threshold parameters
thresholds <- threshpar_grmtree(mos_grmtree)
print(thresholds)

## Extract and print the discrimination parameters
discriminations <- discrpar_grmtree(mos_grmtree)
print(discriminations)

## Extract and print the item parameters
itemspars <- itempar_grmtree(mos_grmtree)
print(itemspars)
```
**Note:** The `itempar_grmtree` extracts both discrimination parameters and average threshold parameters for each item from all terminal nodes of a graded response model tree. See `?itempar_grmtree` for more information.

# Conclusion

This vignette demonstrated basic usage of `grmtree` for identifying DIF in polytomous items. Key steps included checking dimensionality, fitting the GRM, and interpreting the resulting tree.

---
title: "GRM Forests for Robust DIF Detection"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{GRM Forests for Robust DIF Detection}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

# Introduction

GRM Forests extend GRM Trees by creating ensembles of trees to provide more robust variable importance measures. This vignette covers:

* GRM Forest implementation

* Variable importance and variable importance plot.

See the vignette on getting started with grmtree package for a more detailed walkthrough of the tree-based graded response theory model (GRMTree).

# Install and Load required packages
To implement the tree-based GRM (GRMTree), you will install the following packages if not previously installed.
```{r, eval=F}
## Install packages from CRAN repository
install.packages(c("dplyr", "grmtree"))
```

Once installed, load the packages as follows:
```{r, message=FALSE, warning=FALSE}
library(dplyr)        # For data manipulation
library(grmtree)      # For tree-based GRM DIF Test
```

# Import and prepare the data

The data set used in this demonstration is a test/sample data for the package.

```{r, message=FALSE}
## Load the data
data("grmtree_data", package = "grmtree")

## Take a glimpse at the data
glimpse(grmtree_data)

## Prepare the data
resp.data <- grmtree_data %>% 
  mutate_at(vars(starts_with("MOS")), as.ordered) %>% 
  mutate_at(vars(c(sex, residency, depressed,
                   Education, job, smoker,
                   multimorbidity)), as.factor) 

## Explore the data
head(resp.data)

## Check the structure of the data
glimpse(resp.data)

## Create response as outcomes
resp.data$resp <- data.matrix(resp.data[, 1:8])
```

# GRM Forests Implementation

## Define the forest control parameters
```{r}
## Get help on the control parameter
# ?grmforest.control

## GRMTree control parameters with Benjamini-Hochberg 
grm_control <- grmtree.control(
  minbucket = 350,
  p_adjust = "BH", alpha = 0.05)

## Define the forest control parameters
forest_control <- grmforest.control(
  n_tree = 3, # Number of trees (Reduced for vignette build time)
  sampling = "bootstrap",  # Bootstrap method; resampling also available
  sample_fraction = 0.632,
  mtry = sqrt(9),  # Usually the square root of the number of covariates
  control = grm_control,
  remove_dead_trees = TRUE, # Remove any null GRMTree
  seed = 123
)
```

## Grow the GRM Forest

```{r, eval=FALSE}
## Fit the GRM forest
mos_forest <- grmforest(
  resp ~ sex + age + bmi + Education + 
  residency + depressed + job + multimorbidity + smoker,  
  data = resp.data,
  control = forest_control
)

## Get the summary of the fitted forest
summary(mos_forest)
print(mos_forest)

## Plot a tree in the forest
plot(mos_forest$trees[[1]])
```

# Variable Importance

## Compute the variable importance of each covariate
```{r, eval=FALSE}
## Calculate the variable importance
importance <- varimp(mos_forest, seed = 123, verbose = T)

## Print the result of the variable importance
print(importance)
```

Example output:
```
           age         smoker            bmi multimorbidity            sex 
     403.07554      220.37908       39.02621       37.00120       32.06389 
     Education      residency      depressed            job 
       0.00000        0.00000        0.00000        0.00000 
```

## Plot the variable importance of each variable 

Here `plot.varimp` creates a bar plot of variable importance scores with options for both ggplot2 and base R graphics.
```{r, eval=FALSE}
## Plot the variable importance scores (ggplot is the default)
plot(importance)

## Plot onlt the top 5 importance variables
plot(importance, xlab = "", top_n = 5)

## Plot the base R version
plot(importance, use_ggplot = FALSE)

## Custom colors
plot(importance, col = c("green", "red"))

## Rename the variable names in the order from the variable importance result
names(importance) <- c("Age", "Smoking Status", "BMI",
                       "Multimorbidity", "Sex", "Education",
                       "Residency", "Depression", "Employment")

## Now create the plot with informative names
plot(importance)
```


# Conclusion

GRM Forests provide more stable DIF detection by aggregating across many trees. Key advantages include robust variable importance measures, reduced overfitting, and better handling of complex interactions.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmforest-control.R
\name{grmforest.control}
\alias{grmforest.control}
\title{Control Parameters for GRM Forest}
\usage{
grmforest.control(
  n_tree = 100,
  sampling = "bootstrap",
  sample_fraction = 0.632,
  mtry = NULL,
  remove_dead_trees = TRUE,
  control = grmtree.control(),
  alpha = 0.05,
  minbucket = 20,
  seed = NULL
)
}
\arguments{
\item{n_tree}{Number of trees in the forest (default: 100).}

\item{sampling}{Sampling method: "bootstrap" (with replacement) or
"subsample" (without replacement) (default: "bootstrap").}

\item{sample_fraction}{Fraction of data to sample for each tree (default:
0.632).}

\item{mtry}{Number of variables randomly sampled as candidates at each split.
If NULL, all variables are considered (default: NULL).}

\item{remove_dead_trees}{Logical indicating whether to remove trees that
encounter errors during fitting (default: TRUE).}

\item{control}{Control parameters for individual trees created by
\code{grmtree.control()}.}

\item{alpha}{Significance level for splitting (default: 0.05).}

\item{minbucket}{Minimum number of observations in terminal nodes (default:
20).}

\item{seed}{Random seed for reproducibility (default: NULL).}
}
\value{
A list of class \code{grmforest_control} containing: \item{n_tree}{Number
of trees} \item{sampling}{Sampling method} \item{sample_fraction}{Sample
fraction} \item{mtry}{Number of variables to try at each split}
\item{remove_dead_trees}{Whether to remove failed trees}
\item{control}{Tree control parameters} \item{seed}{Random seed}
}
\description{
Creates a control object for \code{grmforest} containing parameters that control
the forest growing process including sampling, tree growing, and error
handling.
}
\examples{
library(grmtree)
# Control with 50 trees using subsampling
ctrl <- grmforest.control(n_tree = 50, sampling = "subsample")

# Control with specific tree parameters
ctrl <- grmforest.control(
  control = grmtree.control(minbucket = 30, alpha = 0.01)
)

}
\seealso{
\code{\link{grmtree.control}} creates a control object for
\code{grmtree}, \code{\link{plot.grmtree}} creates plot for the \code{grmtree} object,
\code{\link{grmforest}} for GRM Forests,
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree-fscores.R
\name{fscores_grmtree}
\alias{fscores_grmtree}
\title{Compute Latent Factor Scores for Each Terminal Node in a GRM Tree}
\usage{
fscores_grmtree(grmtree_obj, method = "EAP")
}
\arguments{
\item{grmtree_obj}{A GRM tree object (from \code{grmtree()} function) containing
fitted models in its terminal nodes.}

\item{method}{Scoring method to use: "EAP" (default), "MAP", "ML", or "WLE".
See \code{mirt::fscores()} for details.}
}
\value{
A named list where each element contains the factor scores for a
terminal node. Names correspond to node IDs. Returns NULL for nodes where
computation fails. If no scores can be computed for any node, returns NULL
with a warning.
}
\description{
This function calculates latent factor scores for each terminal node in a GRM
tree object using specified scoring method (EAP, MAP, ML, or WLE).
}
\examples{
\donttest{
  library(grmtree)
  library(hlt)
  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Fit GRM tree with gender and group as partitioning variables
  tree <- grmtree(resp ~ gender + group,
          data = asti,
          control = grmtree.control(minbucket = 30))

# Compute EAP scores for all terminal nodes
node_scores <- fscores_grmtree(tree)

# Compute MAP scores instead
node_scores_map <- fscores_grmtree(tree, method = "MAP")
}

}
\seealso{
\code{\link[mirt]{fscores}} for factor scoring methods,
\code{\link{grmtree}} fits a Graded Response Model Tree,
\code{\link{grmforest}} for GRM Forests, \code{\link{threshpar_grmtree}} for
extracting threshold parameters, \code{\link{discrpar_grmtree}} for
extracting discrimination parameters, \code{\link{itempar_grmtree}} for
extracting item parameters, \code{\link{generate_node_scores_dataset}}
generates combined dataset with node assignments and factor scores
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long-grmtree-fscores.R
\name{fscores_longitudinal_grmtree}
\alias{fscores_longitudinal_grmtree}
\title{Compute Latent Factor Scores for Longitudinal GRM Tree}
\usage{
fscores_longitudinal_grmtree(object, method = "EAP", ...)
}
\arguments{
\item{object}{A \code{longitudinal_grmtree} object.}

\item{method}{Scoring method: "EAP" (default), "MAP", "ML", or "WLE".}

\item{...}{Additional arguments (currently unused).}
}
\value{
A named list (one element per terminal node). Each element is a
\code{data.frame} with columns:
\describe{
\item{Theta_T1}{Estimated latent trait at Time 1}
\item{Theta_T2}{Estimated latent trait at Time 2}
}
}
\description{
Computes latent factor scores (theta) for both T1 and T2 within each terminal
node of a longitudinal GRM tree. Unlike the cross-sectional
\code{\link{fscores_grmtree}} which returns a single theta per person, this
function returns two scores per person: \eqn{\theta_{T1}} and
\eqn{\theta_{T2}}.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Phase 1: fit the longitudinal GRM tree
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Print the factor scores
  scores <- fscores_longitudinal_grmtree(ltree)
  # Scores for node 2
  head(scores[["2"]])
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{longitudinal_grmtree}} for fitting the tree,
\code{\link{generate_node_scores_dataset}} to generate node assignment and
factor scores
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_nodescores_dataset.R
\name{generate_node_scores_dataset}
\alias{generate_node_scores_dataset}
\title{Generate Dataset with Node Assignments and Factor Scores}
\usage{
generate_node_scores_dataset(object, data = NULL, method = "EAP")
}
\arguments{
\item{object}{A \code{grmtree} or \code{longitudinal_grmtree} object.}

\item{data}{The original data frame used to fit the tree. If provided,
the output contains all columns from this data frame plus node and
factor score columns. If NULL (default), returns only model frame
variables (backward-compatible behavior).}

\item{method}{Scoring method: "EAP" (default), "MAP", "ML", or "WLE".}
}
\value{
A data.frame containing:
\describe{
\item{All original columns}{From \code{data} if provided}
\item{node}{Factor indicating terminal node membership}
\item{factor_score}{For cross-sectional grmtree: single latent score.
For longitudinal: Theta_T1.}
\item{Theta_T1}{(Longitudinal only) Latent trait at T1}
\item{Theta_T2}{(Longitudinal only) Latent trait at T2}
}
}
\description{
Creates a dataset by augmenting the original data with node assignments
and computed factor scores. Unlike the previous version which only returned
model frame variables, this version merges node and score information back
to the full original data frame.
}
\details{
The function works by:
\enumerate{
\item Predicting node membership for each observation using
\code{predict(object, type = "node")}
\item Computing factor scores within each terminal node using the
node-specific model
\item Merging the results back to the original data by row position
}

When \code{data} is provided, the function ensures that the output
contains all columns from the original data frame, not just the
variables used in the model formula. This is important when the
original data contains clinical variables, identifiers, or other
columns not used as partitioning variables.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
# Cross-sectional GRMTree
library(grmtree)
library(hlt)
data("asti", package = "hlt")
asti$resp <- data.matrix(asti[, 1:4])

  # Fit GRM tree with gender and group as partitioning variables
  tree <- grmtree(resp ~ gender + group,
          data = asti,
          control = grmtree.control(minbucket = 30))

# Generate combined dataset
scored_data <- generate_node_scores_dataset(tree, data = asti)

# Plot scores by node
boxplot(factor_score ~ node, data = scored_data)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{grmtree}} fits a Graded Response Model Tree,
\code{\link{grmforest}} for GRM Forests, \code{\link{fscores_grmtree}} for
computing factor scores, \code{\link{threshpar_grmtree}} for extracting
threshold parameters, \code{\link{discrpar_grmtree}} for extracting
discrimination parameters, \code{\link{itempar_grmtree}} for extracting item
parameters, \code{\link{longitudinal_grmtree}} for longitudinal GRMTree,
\code{\link{fscores_longitudinal_grmtree}}, for computing factor scores for longitudinal GRMTree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree-print.R
\name{print.grmtree}
\alias{print.grmtree}
\title{Print Method for GRM Tree Objects}
\usage{
\method{print}{grmtree}(
  x,
  title = "Graded Response Model Tree",
  objfun = "negative log-likelihood",
  ...
)
}
\arguments{
\item{x}{A GRM tree object of class 'grmtree'.}

\item{title}{Character string specifying the title for the print output
(default: "Graded Response Model Tree").}

\item{objfun}{Character string labeling the objective function
(default: "negative log-likelihood").}

\item{...}{Additional arguments passed to \code{print.modelparty}.}
}
\value{
Invisibly returns the GRM tree object. Primarily called for its side
effect of printing a formatted summary.
}
\description{
Displays a formatted summary of a GRM (Graded Response Model) tree object.
This function extends \code{print.modelparty} from the partykit package with
specialized formatting for GRM trees.
}
\details{
The print method provides a comprehensive summary of the GRM tree,
including:
\itemize{
\item Model formula used for fitting
\item Tree structure with node information
\item Item parameter estimates for each terminal node
\item Confidence intervals for parameters
\item Group parameters (mean and covariance)
\item Summary statistics (number of nodes, objective function value)
}
}
\examples{
\donttest{
library(grmtree)
library(hlt)
data("asti", package = "hlt")
asti$resp <- data.matrix(asti[, 1:4])

# Fit GRM tree
tree <- grmtree(resp ~ gender + group,
                data = asti,
                control = grmtree.control(minbucket = 30))

# Print the tree summary
print(tree)

# Alternative syntax (automatically calls print.grmtree)
tree
}

}
\seealso{
\code{\link[partykit]{print.modelparty}} for the underlying printing
infrastructure, \code{\link{grmtree}} for creating GRM tree objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmforest.R
\name{grmforest}
\alias{grmforest}
\title{Fit a Forest of Graded Response Model Trees for Ensemble-Based DIF Detection}
\usage{
grmforest(formula, data, control = grmforest.control(), ...)
}
\arguments{
\item{formula}{A formula specifying the model structure with the response
matrix on the left and partitioning variables on the right (e.g.,
\code{response_matrix ~ age + gender + education + clinical_variables}).}

\item{data}{A data frame containing the response matrix and partitioning
variables. The response matrix should contain polytomous items coded
as ordered factors.}

\item{control}{A control object created by \code{grmforest.control()}.}

\item{...}{Additional arguments passed to underlying \code{grmtree()} function.}
}
\value{
An object of class \code{grmforest} containing: \item{trees}{List of
fitted GRM trees} \item{oob_samples}{List of out-of-bag samples for each
tree} \item{formula}{The model formula} \item{data}{The original dataset}
\item{call}{The function call}
}
\description{
This function implements a forest of graded response model trees (GRM Forest)
using bootstrap aggregation (bagging) or random subsampling to enhance the
detection and analysis of differential item functioning (DIF) in polytomous items.
The GRM Forest approach combines the strengths of multiple GRMTrees to provide
more robust and stable DIF detection, particularly for complex datasets with
high-dimensional covariates or subtle DIF patterns.
}
\details{
The algorithm works by fitting multiple GRMTrees, each on a random sample of
the original data (either through bootstrap sampling or subsampling). For each
tree, approximately one-third of the observations are left out as out-of-bag
(OOB) samples, which are used for internal validation and variable importance
calculation. The ensemble approach reduces variance, minimizes overfitting,
and provides more reliable identification of covariates associated with DIF.

Key advantages of the GRM Forest approach include:
\itemize{
\item Enhanced stability in DIF detection across different sampling variations
\item Robust variable importance measures that quantify the relative contribution
of each covariate to DIF patterns
\item Reduced false positive rates through consensus-based detection
\item Ability to handle high-dimensional covariate spaces effectively
\item Internal validation through out-of-bag error estimation
}

The forest implementation supports both bootstrap aggregation (where samples
are drawn with replacement) and subsampling (without replacement), allowing
flexibility for different data characteristics and research objectives.
}
\examples{

\donttest{
  library(grmtree)
  library(hlt)
  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Fit forest with default parameters
  forest <- grmforest(resp ~ gender + group, data = asti)

  # Fit with custom control
  ctrl <- grmforest.control(n_tree = 20, sampling = "subsample")
  forest <- grmforest(resp ~ gender + group, data = asti, control = ctrl)
}

}
\seealso{
\code{\link{grmtree}} fits a Graded Response Model Tree,
\code{\link{grmtree.control}} creates a control object for \code{grmtree},
\code{\link{grmforest.control}} creates a control object for \code{grmforest},
\code{\link{varimp}} calculates the variable importance for GRM Forest,
\code{\link{plot.varimp}} creates a bar plot of variable importance scores
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree-control.R
\name{grmtree.control}
\alias{grmtree.control}
\title{Control Parameters for GRM Trees}
\usage{
grmtree.control(
  minbucket = 20,
  p_adjust = "none",
  alpha = 0.05,
  initial_alpha = NULL,
  ...
)
}
\arguments{
\item{minbucket}{Minimum number of observations in a terminal node (default:
20).}

\item{p_adjust}{Method for p-value adjustment. One of: "none", "bonferroni",
"holm", "BH", "BY", "hochberg", or "hommel" (default: "none").}

\item{alpha}{Significance level for splitting (default: 0.05).}

\item{initial_alpha}{For post-hoc adjustment methods (holm, BH, BY, hochberg,
hommel), the significance threshold for initial tree construction before
pruning. Must satisfy \code{alpha < initial_alpha < 1}. Default is
\code{min(3 * alpha, 0.20)}. Lower values produce more conservative results
but run faster; higher values provide more power but require more
computation and may increase Type I error. Ignored for "none" and
"bonferroni" methods.}

\item{...}{Additional arguments passed to \code{partykit::mob_control()}.}
}
\value{
A list of control parameters with class \code{grmtree_control}.
}
\description{
Creates a control object for \code{grmtree} containing various parameters that
control the tree growing process.
}
\examples{
# Use Bonferroni correction with alpha = 0.01
ctrl <- grmtree.control(p_adjust = "bonferroni", alpha = 0.01)

}
\seealso{
\code{\link{grmtree}} fits a Graded Response Model Tree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long-grmtree-threshpar.R
\name{threshpar_longitudinal_grmtree}
\alias{threshpar_longitudinal_grmtree}
\title{Extract Threshold Parameters from Longitudinal GRM Tree}
\usage{
threshpar_longitudinal_grmtree(object, node = NULL, clean_names = TRUE, ...)
}
\arguments{
\item{object}{A \code{longitudinal_grmtree} object.}

\item{node}{Optional vector of node IDs. If NULL (default), extracts from all
terminal nodes.}

\item{clean_names}{Logical. If TRUE (default), clean item names by removing
common prefixes (e.g., "resp_wideMOS_") and suffixes (e.g., "_BL", "_T1").}

\item{...}{Additional arguments (currently unused).}
}
\value{
A data.frame with columns:
\describe{
\item{Node}{Terminal node ID}
\item{Item}{Item name (cleaned if \code{clean_names = TRUE})}
\item{b1, b2, ..., bK}{Threshold parameters for each category boundary}
}
Contains \code{n_items} rows per node (not \code{2 * n_items}).
}
\description{
Extracts threshold (difficulty) parameters for each unique item from all
terminal nodes of a longitudinal GRM tree. Only T1 item parameters are
returned since the constrained model enforces equal thresholds across T1 and
T2 within each node.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Phase 1: fit the longitudinal GRM tree
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Print the threshold parameters
  thresholds <- threshpar_longitudinal_grmtree(ltree)
  print(thresholds)

  # Using the raw names
  threshpar_longitudinal_grmtree(ltree, clean_names = FALSE)  # raw names
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{longitudinal_grmtree}} for Phase 1 (tree fitting),
\code{\link{discrpar_longitudinal_grmtree}} for extracting discrimination
parameters for longitudinal GRMTree,
\code{\link{itempar_longitudinal_grmtree}} for extracting item parameters
for longitudinal GRMTree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree-data.R
\docType{data}
\name{grmtree_data}
\alias{grmtree_data}
\title{Medical Outcomes Study Social Support Survey (MOS-SS) Test Data}
\format{
A tibble with 3,500 rows and 17 variables:
\describe{
\item{MOS_Listen}{Someone you can count on to listen to you when you need to talk (1-5 Likert scale)}
\item{MOS_Info}{Someone to give you information to help you understand a situation (1-5 Likert scale)}
\item{MOS_Advice_Crisis}{Someone to give good advice about a crisis (1-5 Likert scale)}
\item{MOS_Confide}{Someone to confide in or talk to about yourself or your problems (1-5 Likert scale)}
\item{MOS_Advice_Want}{Someone whose advice you really want (1-5 Likert scale)}
\item{MOS_Fears}{Someone to share private worries or fears (1-5 Likert scale)}
\item{MOS_Personal}{Someone to turn to for suggestions about how to deal with a personal problem (1-5 Likert scale)}
\item{MOS_Understand}{Someone who understands your problems (1-5 Likert scale)}
\item{sex}{Gender (Male, Female)}
\item{age}{Age in years (numeric)}
\item{residency}{Residence location (rural, urban)}
\item{depressed}{Depression status (No, Yes)}
\item{bmi}{Body Mass Index (numeric)}
\item{Education}{Education level (Primary/High school, College/University)}
\item{job}{Employment status (Employed, Unemployed)}
\item{smoker}{Smoking status (No, Yes)}
\item{multimorbidity}{Number of chronic conditions (0, 1, 2+)}
}
}
\source{
Simulated data generated for package testing and demonstration
}
\usage{
grmtree_data
}
\description{
A dataset containing sample responses to the MOS-SS emotional domain items
and demographic variables. This data is provided for testing and
demonstration purposes within the grmtree package. The items are numbered
1-5, representing None of the time,	A little of the time,	Some of the
time,	Most of the time, All of the time, respectively.
}
\examples{
\donttest{
  library(dplyr)

  # Load and take a glimpse at the data
  data(grmtree_data, package = "grmtree")
  glimpse(grmtree_data)
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree-itempar.R
\name{itempar_grmtree}
\alias{itempar_grmtree}
\title{Extract Item Parameters from GRM Tree}
\usage{
itempar_grmtree(object, node = NULL, ...)
}
\arguments{
\item{object}{A \code{grmtree} object.}

\item{node}{Optional vector of node IDs to extract from. If NULL (default),
extracts from all terminal nodes.}

\item{...}{Additional arguments (currently unused).}
}
\value{
A data.frame with item parameters for each item in each node, with columns:
\item{Node}{Node ID}
\item{Item}{Item name}
\item{Discrimination}{Discrimination parameter (a1)}
\item{AvgThreshold}{Average of threshold parameters}
\item{Thresholds}{All threshold parameters as a list column}
}
\description{
Extracts both discrimination parameters and average threshold parameters for each item
from all terminal nodes of a graded response model tree. This provides a comprehensive
view of item characteristics across different nodes of the tree.
}
\examples{
\donttest{
  library(grmtree)
  library(hlt)
  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Fit GRM tree with gender and group as partitioning variables
  tree <- grmtree(resp ~ gender + group,
          data = asti,
          control = grmtree.control(minbucket = 30))

  # Get all item parameters
  items <- itempar_grmtree(tree)
  print(items)
}

}
\seealso{
\code{\link{grmtree}} fits a Graded Response Model Tree,
\code{\link{grmforest}} for GRM Forests,
\code{\link{fscores_grmtree}} for computing factor scores,
\code{\link{threshpar_grmtree}} for extracting threshold parameters,
\code{\link{discrpar_grmtree}} for extracting discrimination parameters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree-applytomodels.R
\name{apply_to_models}
\alias{apply_to_models}
\title{Internal Function: Apply Function to Models in Nodes}
\usage{
apply_to_models(object, node = NULL, FUN = NULL, drop = FALSE, ...)
}
\arguments{
\item{object}{A \code{grmtree} object.}

\item{node}{Vector of node IDs (default: all nodes).}

\item{FUN}{Function to apply to each model.}

\item{drop}{Logical indicating whether to drop list structure if length 1.}

\item{...}{Additional arguments passed to FUN.}
}
\value{
List of results (or single result if drop=TRUE and length(node)=1).
}
\description{
Applies a function to the GRM models in specified nodes of a tree.
This is an internal helper function and not meant to be called directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long-grmtree-plot.R
\name{node_regionplot_longitudinal}
\alias{node_regionplot_longitudinal}
\title{Region Plot Panel for Longitudinal GRM Tree Nodes}
\usage{
node_regionplot_longitudinal(
  mobobj,
  n_items,
  names = FALSE,
  abbreviate = TRUE,
  ylim = NULL,
  off = 0.1,
  col_fun = grDevices::gray.colors,
  bg = "white",
  ylines = 2,
  ...
)
}
\arguments{
\item{mobobj}{A \code{longitudinal_grmtree} object.}

\item{n_items}{Number of items per time point.}

\item{names}{Logical, character vector, or FALSE for labeling.}

\item{abbreviate}{Logical or numeric for name abbreviation.}

\item{ylim}{Optional y-axis limits.}

\item{off}{Offset between items (default: 0.1).}

\item{col_fun}{Color function for response categories (default:
\code{gray.colors}).}

\item{bg}{Background color (default: "white").}

\item{ylines}{Width of y-axis margin in lines (default: 2).}

\item{...}{Additional arguments (unused).}
}
\value{
A panel function suitable for use with
\code{\link[partykit]{plot.modelparty}}.
}
\description{
Internal function that generates the terminal node panel function for
region plots of longitudinal GRM trees. Extracts only T1 item thresholds
(items 1 through n_items) since T2 items have identical constrained
parameters.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rs_characterize.R
\name{print.rs_characterization}
\alias{print.rs_characterization}
\title{Print Method for Response Shift Characterization Results}
\usage{
\method{print}{rs_characterization}(x, ...)
}
\arguments{
\item{x}{An \code{rs_characterization} object returned by
\code{\link{rs_characterize}}.}

\item{...}{Additional arguments (currently unused).}
}
\value{
Invisibly returns \code{x}. Called for its side effect of printing
formatted output to the console.
}
\description{
Displays a formatted summary of response shift characterization results
from \code{\link{rs_characterize}}, including omnibus RS test results per
terminal node and item-level RS testing results with significance markers.
}
\details{
The output is organized in two sections:

\strong{Omnibus RS Test:} For each terminal node, displays the LRT
statistic, degrees of freedom, raw and adjusted p-values (if applicable),
RS detection status, effect sizes (pseudo R-squared and AIC difference),
latent mean shift, T1-T2 correlation, and convergence warnings.

\strong{Item-Level RS Testing:} For nodes where omnibus RS was detected,
displays per-item LRT statistics, raw and adjusted p-values, RS type
classification, and significance markers (\code{***} for items significant
after adjustment). Also reports the p-value adjustment method and RS
classification threshold used.
}
\seealso{
\code{\link{rs_characterize}} for computing the results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmforest.R
\name{print.grmforest}
\alias{print.grmforest}
\title{Print Method for GRM Forest}
\usage{
\method{print}{grmforest}(x, ...)
}
\arguments{
\item{x}{A \code{grmforest} object}

\item{...}{Additional arguments (currently unused)}
}
\value{
Invisibly returns the input object
}
\description{
Print Method for GRM Forest
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree.R
\name{grmtree}
\alias{grmtree}
\title{Fit a Graded Response Model Tree for Differential Item Functioning Detection}
\usage{
grmtree(
  formula,
  data,
  na.action = na.omit,
  control = grmtree.control(),
  mtry = NULL,
  ...
)
}
\arguments{
\item{formula}{A formula specifying the model structure with the response
matrix on the left and partitioning variables on the right (e.g.,
\code{response_matrix ~ age + gender}).}

\item{data}{A data frame containing the variables in the model.}

\item{na.action}{How to handle missing values (default: \code{na.omit}).}

\item{control}{A list of control parameters created by \code{grmtree.control()}.}

\item{mtry}{Number of variables randomly sampled as candidates at each split.
If NULL, all variables are considered.}

\item{...}{Additional arguments passed to the fitting function.}
}
\value{
An object of class \code{grmtree} inheriting from \code{modelparty} containing
the fitted tree structure.
}
\description{
This function implements a tree-based graded response model (GRM) using
model-based recursive partitioning to detect and account for differential
item functioning (DIF) in polytomous items. The GRMTree combines the
statistical framework of item response theory with recursive partitioning to
identify heterogeneous subgroups in the population where item parameters
(discrimination and thresholds) vary systematically across covariates.
}
\details{
The algorithm works by first estimating a global GRM for the entire sample,
then recursively testing for parameter instability with respect to available
covariates. When significant DIF is detected, the sample is partitioned into
homogeneous subgroups, each with their own set of item parameters. This
approach allows for the identification of complex interaction effects and
provides interpretable tree structures that visualize how item functioning
varies across different patient subgroups.

GRMTree is particularly useful in health outcomes research where
patient-reported outcome measures may function differently across diverse
demographic, clinical, or socioeconomic subgroups. The resulting tree
diagrams facilitate the development of personalized assessment strategies and
can inform targeted interventions by identifying specific patient
characteristics associated with differential item interpretation.
\subsection{Conventional Graded Response Model (GRM)}{

Let \eqn{Y_{im}} denote the response of the \eqn{i^{th}} (\eqn{i=1,\ldots,N})
individual to the \eqn{m^{th}} (\eqn{m = 1,2,\ldots,M}) item. The graded
response model is described as:

\deqn{P(Y_{im} \geq j | \tau_{mj}, \lambda_m, \theta_i) =
\frac{\exp(-(\tau_{mj} - \lambda_m \theta_i))}{1 + \exp(-(\tau_{mj} -
\lambda_m \theta_i))}}

where:
\itemize{
\item \eqn{P(Y_{im} \geq j | \tau_{mj}, \lambda_m, \theta_i)} is the probability that
individual \eqn{i}'s response is in category \eqn{j} or higher on item
\eqn{m},
\item \eqn{\tau_{mj}} is the threshold parameter between categories \eqn{j-1} and
\eqn{j} for item \eqn{m},
\item \eqn{\lambda_m} is the discrimination parameter for item \eqn{m},
\item \eqn{\theta_i \sim N(0,1)} is the latent trait score for individual \eqn{i}.
}

This parametrization is equivalent to the conventional IRT formulation where
item discrimination is \eqn{a_m = \lambda_m} and item difficulty is
\eqn{b_{mj} = \tau_{mj} / \lambda_m}.
}

\subsection{Graded Response Model Tree (GRMTree) Implementation}{

The GRMTree is a hybrid model that integrates the GRM with model-based
recursive partitioning to detect and account for differential item
functioning (DIF) across subgroups defined by covariates. The algorithm
proceeds through the following steps:

\strong{Step 1: Global Model Estimation}

Estimate the GRM item parameters \eqn{(\hat{\tau}_{mj}, \hat{\lambda}_m)}
jointly for all individuals in the study cohort at the root node via maximum
likelihood estimation:

\deqn{\hat{\beta}_{\text{global}} = \arg\max_{\beta} \sum_{i=1}^N \log
L(\beta; \mathbf{y}_i)}

where \eqn{\beta = (a_1, \ldots, a_J, b_{11}, \ldots, b_{J,m-1})} contains
all item parameters (discrimination and difficulty), providing a baseline
model assuming parameter invariance.

\strong{Step 2: Parameter Stability Testing}

For each available covariate \eqn{X_p} (\eqn{p = 1, \ldots, P}), assess the
stability of the item parameters by conducting score-based structural change
tests. This involves: 1. Calculating the score function contributions
\eqn{s(\hat{\beta}; y_i, x_i)} for each individual, 2. Ordering these scores
with respect to each covariate \eqn{X_p}, 3. Testing the null hypothesis
\eqn{H_0: \mathbb{E}[s(\hat{\beta}; y_i, x_i)] = 0} for all \eqn{i} against
the alternative that scores fluctuate systematically with \eqn{X_p},
indicating parameter instability (DIF).

\strong{Step 3: Recursive Partitioning}

If significant instability is detected (\eqn{p < \alpha_{\text{adj}}}):
\itemize{
\item \strong{Covariate Selection:} Identify the covariate \eqn{X_p^*} with the most
significant instability (smallest adjusted p-value),
\item \strong{Split Point Determination:} Find the optimal cut-point \eqn{c^*} that
maximizes the partitioned log-likelihood: \deqn{\ell_{\text{left}}(\beta) +
\ell_{\text{right}}(\beta) = \sum_{i: X_{pi} \leq c^*} \log L(\beta; y_i) +
\sum_{i: X_{pi} > c^*} \log L(\beta; y_i)} over all possible cut-points on
\eqn{X_p^*},
\item \strong{Sample Splitting:} Partition the sample into two child nodes based on the
rule \eqn{X_p^* \leq c^*}.
}

\strong{Step 4: Recursive Application & Stopping Criteria}

Repeat Steps 1-3 recursively within each resulting child node until one of
the following stopping criteria is met:
\enumerate{
\item \strong{No Significant Instability:} No covariate shows significant parameter
instability after multiple testing correction (\eqn{\alpha_{\text{adj}} =
\alpha/m}, where multiple adjustment methods can be applied, including
Bonferroni, Holm, Benjamini-Hochberg, etc.).
\item \strong{Minimum Node Size:} The subsample size falls below a prespecified
minimum (e.g., \eqn{n < 10 \times} the number of item parameters).
}
}

\subsection{Formal GRMTree Structure}{

The final GRMTree provides a piecewise GRM where each terminal node
represents a subgroup with homogeneous item parameters, explicitly modeling
the detected DIF structure within the data. The resulting GRMTree model can
be expressed as a mixture of subgroup-specific GRMs:

\deqn{P(Y_{ij} = k | \theta_i, \mathbf{x}_i) = \sum_{b=1}^B I(\mathbf{x}_i
\in \mathcal{X}_b) \cdot P_b(Y_{ij} = k | \theta_i)}

where:
\itemize{
\item \eqn{B} is the number of terminal nodes,
\item \eqn{\mathcal{X}_b} is the covariate subspace defining terminal node \eqn{b},
\item \eqn{P_b} is the node-specific GRM with parameters \eqn{\beta_b},
\item \eqn{I(\cdot)} is the indicator function.
}

Each terminal node \eqn{b} contains a complete GRM with:
\itemize{
\item \strong{Node-specific item parameters:} \eqn{\beta_b = (a_{1b}, \ldots, a_{Jb}, b_{11b}, \ldots, b_{J,m-1,b})}
\item \strong{Local ability distribution:} \eqn{\theta_i | \mathbf{x}_i \in \mathcal{X}_b \sim N(0, 1)}
}

This approach allows \strong{differential item functioning (DIF)} to be detected
and modeled explicitly through the tree structure, where item parameters can
vary across subgroups defined by covariates, while maintaining the
conditional distribution of the latent trait within each subgroup.
}

\subsection{Post-Hoc Multiple Comparison Adjustments}{

For holm, BH, BY, hochberg, and
hommel methods, the algorithm employs a two-stage approach: (1) build a tree
using \code{initial_alpha} as the splitting threshold, (2) apply global
p-value adjustment across all splits, and (3) prune splits that do not meet
the adjusted threshold \code{alpha}. This differs from Bonferroni correction,
which is applied locally during tree construction via \code{partykit::mob}.
The choice of \code{initial_alpha} represents a statistical trade-off. The
default (\code{min(3 * alpha, 0.20)}) provides a balance between power and
computational efficiency. Note that global post-hoc adjustments in
hierarchical tree structures may be conservative compared to per-node
adjustments, as they account for all tests performed during tree exploration.
This global adjustment approach controls the tree-wide error rate and may be
conservative (Type I error often <3\%). This conservativeness ensures strong
control of family-wise error rate or false discovery rate across all splits
in the tree. Users requiring less conservative control may prefer the
Bonferroni method, which applies per-node adjustment during tree
construction.
}
}
\examples{
\donttest{
  library(grmtree)
  library(hlt)

  # Prepare the asti data (from the hlt package)
  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Fit GRM tree with gender and group as partitioning variables
  tree <- grmtree(resp ~ gender + group,
          data = asti,
          control = grmtree.control(minbucket = 30))

  ## Print and plot the tree
  print(tree)
  plot(tree)

  # Extract item parameters for specific subgroups
  discr_params <- discrpar_grmtree(tree)
  threshold_params <- threshpar_grmtree(tree)
}

}
\references{
\subsection{Methodological Foundations}{

Samejima, F. (1969). Estimation of latent ability using a response pattern
of graded scores. Psychometrika Monograph Supplement, 34, 100-114.

Strobl, C., Kopf, J., & Zeileis, A. (2015). Rasch trees: A new method for
detecting differential item functioning in the Rasch model. Psychometrika,
80(2), 289-316.

Komboz, B., Strobl, C., & Zeileis, A. (2018). Tree-based global model tests
for polytomous Rasch models. Educational and psychological measurement,
78(1), 128-166. https://doi.org/10.1177/0013164416664394

Arimoro, O. I., Lix, L. M., Patten, S. B., Sawatzky, R., Sebille, V., Liu,
J., Wiebe, S., Josephson, C. B., & Sajobi, T. T. (2025). Tree-based latent
variable model for assessing differential item functioning in
patient-reported outcome measures: a simulation study. Quality of Life
Research. https://doi.org/10.1007/s11136-025-04018-6
}

\subsection{Applied Examples}{

Arimoro, O. I., Josephson, C. B., James, M. T., Patten, S. B., Wiebe, S.,
Lix, L. M., & Sajobi, T. T. (2024). Screening for depression in patients
with epilepsy: same questions but different meaning to different patients.
Quality of Life Research, 33(12), 3409-3419.
https://doi.org/10.1007/s11136-024-03782-1
}
}
\seealso{
\code{\link{print.grmtree}} prints the detailed summary results of
the \code{grmtree} object, \code{\link{grmtree.control}} creates a control
object for \code{grmtree}, \code{\link{plot.grmtree}} creates plot for the
\code{grmtree} object, \code{\link{grmforest}} for GRM Forests,
\code{\link{varimp}} calculates the variable importance for GRM Forest,
\code{\link{fscores_grmtree}} for computing factor scores,
\code{\link{threshpar_grmtree}} for extracting threshold parameters,
\code{\link{discrpar_grmtree}} for extracting discrimination parameters,
\code{\link{itempar_grmtree}} for extracting item parameters
}
\author{
Olayinka Imisioluwa Arimoro \email{olayinka.arimoro@ucalgary.ca},
Lisa M. Lix, Tolulope T. Sajobi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/longitudinal_grmtree.R
\name{longitudinal_grmtree}
\alias{longitudinal_grmtree}
\title{Fit a Longitudinal Graded Response Model Tree for Response Shift Detection}
\usage{
longitudinal_grmtree(
  formula,
  data,
  n_items = NULL,
  na.action = na.omit,
  control = grmtree.control(),
  mtry = NULL,
  ...
)
}
\arguments{
\item{formula}{A formula specifying the model structure with a wide-format
response matrix on the left-hand side and partitioning variables on the
right-hand side (e.g., \code{resp_wide ~ age + sex + comorbidity_count}).
The response matrix must have \code{2 * n_items} columns: the first
\code{n_items} columns are Time 1 (T1) responses and the next
\code{n_items} columns are Time 2 (T2) responses for the same items.}

\item{data}{A data frame containing the variables in the formula. Must
include the response matrix as a matrix-valued column (created via
\code{data$resp_wide <- as.matrix(...)}) and all partitioning variables
as separate columns.}

\item{n_items}{Integer specifying the number of items per time point. If
\code{NULL} (default), automatically detected as half the number of
columns in the response matrix. Must satisfy
\code{ncol(response) == 2 * n_items}.}

\item{na.action}{How to handle missing values. Default is \code{na.omit},
which removes rows with any missing values in the response matrix or
partitioning variables.}

\item{control}{A list of control parameters created by
\code{\link[grmtree]{grmtree.control}}. Key parameters include:
\describe{
\item{\code{minbucket}}{Minimum number of observations in a terminal
node. Should be at least 10 times the number of free parameters per
node to ensure stable estimation. For 8 items with 5 categories each,
the constrained longitudinal GRM has approximately 40 item parameters
plus 3 latent parameters, suggesting \code{minbucket >= 200}.}
\item{\code{alpha}}{Significance level for the parameter instability
tests. Default is 0.05.}
\item{\code{p_adjust}}{Method for adjusting p-values across covariates
at each split. Options include \code{"bonferroni"} (default, applied
locally during tree construction), \code{"BH"}
(Benjamini-Hochberg, applied post-hoc with pruning), and others.
See \code{\link[grmtree]{grmtree.control}} for details.}
}}

\item{mtry}{Number of variables randomly sampled as candidates at each
split. If \code{NULL} (default), all variables are considered. Can be
used for random forest extensions.}

\item{...}{Additional arguments passed to the internal fitting function
\code{\link{longitudinal_grmfit}} and ultimately to
\code{\link[mirt]{mirt}}.}
}
\value{
An object of class \code{c("longitudinal_grmtree", "grmtree",
  "modelparty", "party")} containing the fitted tree structure. The object
inherits from \code{\link[partykit]{modelparty}} and includes:
\describe{
\item{Tree structure}{Accessible via standard \code{partykit} methods
such as \code{\link[partykit]{nodeids}},
\code{\link[partykit]{data_party}}, and indexing with \code{[[}.}
\item{Node models}{Each terminal node contains a fitted constrained
longitudinal GRM with item parameters (discrimination and thresholds),
latent trait means (\eqn{\mu_{T2}}), and latent trait covariance
matrix (\eqn{\sigma^2_{T2}}, \eqn{r_{T1,T2}}).}
\item{\code{info$n_items}}{Number of items per time point.}
\item{\code{info$model_type}}{\code{"longitudinal_grm"}.}
\item{\code{info$p_adjust}}{The p-value adjustment method used.}
\item{\code{info$call}}{The original function call.}
}
}
\description{
This function implements a tree-based longitudinal graded response model
using model-based recursive partitioning (MOB) to detect measurement
heterogeneity in patient-reported outcome measures (PROMs) measured at two
time points. The Longitudinal GRMTree extends the cross-sectional
\code{\link[grmtree]{grmtree}} to the longitudinal setting by embedding
a constrained two-factor GRM within the MOB framework. The resulting tree
identifies patient subgroups where the longitudinal measurement model
differs, providing the foundation for subgroup-specific response shift
characterization via \code{\link{rs_characterize}}.
}
\details{
\subsection{Overview}{

The Longitudinal GRMTree is a unified framework for response shift
detection that operates in two phases:

\strong{Phase 1 (this function):} Identifies patient subgroups where the
constrained longitudinal measurement model differs. The constrained model
represents the null hypothesis of no response shift (item parameters equal
across time). MOB tests whether this null model's parameters are stable
across patient covariates and recursively partitions the sample where
instability is detected.

\strong{Phase 2 (\code{\link{rs_characterize}}):} Within each terminal
node, tests whether item parameters actually change from T1 to T2 by
comparing the constrained model to an unconstrained model using likelihood
ratio tests. This phase characterizes response shift at the item level,
classifying changes as recalibration, reprioritization, or both.
}

\subsection{Constrained Two-Factor Longitudinal GRM}{

Let \eqn{Y_{im}} denote the response of individual \eqn{i} to item \eqn{m}
at time \eqn{t}. The longitudinal GRM specifies two correlated latent
factors:

\deqn{\theta_{i,T1} \sim N(0, 1)}
\deqn{\theta_{i,T2} \sim N(\mu_{T2}, \sigma^2_{T2})}
\deqn{Cov(\theta_{i,T1}, \theta_{i,T2}) = \sigma_{12}}

For item \eqn{m} at time \eqn{t}, the graded response model is:

\deqn{P(Y_{im,t} \geq j | \theta_{i,t}) = \frac{\exp(a_{m,t}(\theta_{i,t}
- b_{mj,t}))}{1 + \exp(a_{m,t}(\theta_{i,t} - b_{mj,t}))}}

where \eqn{a_{m,t}} is the discrimination parameter and \eqn{b_{mj,t}} are
threshold parameters for item \eqn{m} at time \eqn{t}.

Under the no-response-shift constraint:
\deqn{a_{m,T1} = a_{m,T2} \quad \text{and} \quad b_{mj,T1} = b_{mj,T2}
\quad \forall m, j}

This means item parameters are identical across time, so any observed
changes in responses are attributed to true latent change (\eqn{\mu_{T2}})
rather than changes in measurement properties.
}

\subsection{The Longitudinal GRMTree Algorithm}{

\strong{Step 1: Global Model Estimation.} Fit the constrained longitudinal
GRM to all individuals at the root node, estimating item parameters
\eqn{\hat{\beta}} and latent parameters \eqn{(\mu_{T2}, \sigma^2_{T2},
\sigma_{12})} via maximum likelihood.

\strong{Step 2: Parameter Stability Testing.} For each covariate
\eqn{X_p}, compute individual score function contributions
\eqn{s(\hat{\beta}; \mathbf{y}_i)} and test whether these scores
fluctuate systematically with \eqn{X_p} using structural change tests.
The null hypothesis is that model parameters are stable across all
values of \eqn{X_p}.

\strong{Step 3: Recursive Partitioning.} If significant instability is
detected (after p-value adjustment):
\itemize{
\item Select the covariate \eqn{X_p^*} with strongest instability
\item Find the optimal split point \eqn{c^*} maximizing the partitioned
log-likelihood
\item Split the sample: \eqn{X_p^* \leq c^*} vs. \eqn{X_p^* > c^*}
}

\strong{Step 4: Recursion.} Repeat Steps 1--3 within each child node
until no significant instability remains or the minimum node size is
reached.

\strong{Step 5 (Post-hoc):} Apply \code{\link{rs_characterize}} to test
for response shift within each terminal node.
}

\subsection{Formal Model Structure}{

The fitted Longitudinal GRMTree provides a piecewise constrained
longitudinal GRM:

\deqn{P(Y_{im,t} = k | \theta_{i,t}, \mathbf{x}_i) = \sum_{b=1}^B
I(\mathbf{x}_i \in \mathcal{X}_b) \cdot P_b(Y_{im,t} = k | \theta_{i,t})}

where \eqn{B} is the number of terminal nodes, \eqn{\mathcal{X}_b} is
the covariate subspace defining terminal node \eqn{b}, and \eqn{P_b} is
the node-specific constrained longitudinal GRM. Each terminal node
contains:
\itemize{
\item Node-specific constrained item parameters (equal across T1 and T2)
\item Node-specific latent trait parameters: \eqn{\mu_{T2,b}},
\eqn{\sigma^2_{T2,b}}, \eqn{r_{T1,T2,b}}
}
}

\subsection{Interpretation}{

The tree structure identifies patient subgroups where the constrained
longitudinal measurement model differs. This can reflect:
\itemize{
\item Different item discrimination patterns across subgroups
\item Different threshold locations across subgroups
\item Different magnitudes of true latent change (\eqn{\mu_{T2}})
\item Different test-retest correlations
}

Crucially, the tree does \strong{not} directly detect response shift
(temporal change in item parameters within a subgroup). Response shift
is tested in Phase 2 using \code{\link{rs_characterize}}, which relaxes
the equality constraints within each terminal node. A tree with only one
terminal node (no split) indicates that no covariate moderates the
measurement model; it does \strong{not} imply the absence of response
shift, since \code{\link{rs_characterize}} can still detect uniform RS in
the unsplit root node.
}

\subsection{Relationship to Existing Methods}{

The Longitudinal GRMTree extends several existing approaches:
\itemize{
\item \strong{GRMTree} (Arimoro et al., 2026): Cross-sectional DIF
detection using tree-based GRM. The longitudinal extension embeds a
two-factor model instead of a single-factor model.
\item \strong{LIRTree} (Ames & Leventhal, 2021): Longitudinal
tree-based IRT using Rasch/2PL models. The GRM extension allows
item-specific discrimination parameters, which are important for
PROMs where items vary in discriminating ability.
\item \strong{Oort SEM} (Oort, 2005): Response shift detection via
structural equation modeling. The tree-based approach removes the
requirement for pre-specified subgroups.
}
}

\subsection{Practical Recommendations}{

\itemize{
\item \strong{Minimum node size:} Use at least 10--25 times the number
of free parameters per node. For \eqn{M} items with \eqn{K} response
categories, the constrained model has approximately \eqn{M(K-1) + M + 3}
parameters (\eqn{M} discriminations, \eqn{M(K-1)} thresholds, and 3
latent parameters). For 8 items with 5 categories:
\eqn{8 \times 4 + 8 + 3 = 43} parameters, suggesting
\code{minbucket = 200--400}.
\item \strong{Number of covariates:} Bonferroni correction becomes
increasingly conservative with more covariates. Consider using
\code{p_adjust = "BH"} for exploratory analyses with many covariates.
\item \strong{Sample size:} With typical PROM instruments (5--15 items),
a total sample of at least 500--1000 is recommended for adequate power
to detect meaningful splits.
\item \strong{Response matrix preparation:} Use
\code{\link{prepare_longitudinal_data}} to construct the wide-format
response matrix from separate T1 and T2 item columns.
}
}
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

  # Load the synthetic longitudinal data
  data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Phase 1: fit the longitudinal GRM tree
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Print tree structure
  print(ltree)

  # Plot threshold regions
  plot(ltree, type = "regions", tnex = 2L)

  # Phase 2: characterize response shift within each subgroup
  rs <- rs_characterize(ltree, p_adjust = "fdr",
    global_p_adjust = "bonferroni")
  print(rs)

\dontshow{\}) # examplesIf}
}
\references{
Arimoro, O. I., Lix, L. M., Patten, S. B., Sawatzky, R., Sebille, V.,
Liu, J., Wiebe, S., Josephson, C. B., & Sajobi, T. T. (2025). Tree-based
latent variable model for assessing differential item functioning in
patient-reported outcome measures: a simulation study. \emph{Quality of
Life Research}. \doi{10.1007/s11136-025-04018-6}

Ames, A. J., & Leventhal, B. C. (2021). Application of a longitudinal
IRTree model: response style changes over time. \emph{Educational and
Psychological Measurement}, 81(3), 561--582.

Oort, F. J. (2005). Using structural equation modeling to detect response
shifts and true change. \emph{Quality of Life Research}, 14(3), 587--598.

Samejima, F. (1969). Estimation of latent ability using a response pattern
of graded scores. \emph{Psychometrika Monograph Supplement}, 34, 100--114.

Sprangers, M. A., & Schwartz, C. E. (1999). Integrating response shift
into health-related quality of life research. \emph{Social Science &
Medicine}, 48(11), 1507--1515.

Zeileis, A., Hothorn, T., & Hornik, K. (2008). Model-based recursive
partitioning. \emph{Journal of Computational and Graphical Statistics},
17(2), 492--514.
}
\seealso{
\code{\link{rs_characterize}} for Phase 2 response shift characterization,
\code{\link{prepare_longitudinal_data}} for data preparation,
\code{\link[grmtree]{grmtree}} for cross-sectional DIF detection,
\code{\link[grmtree]{grmtree.control}} for control parameters,
\code{\link[partykit]{mob}} for the underlying MOB framework
}
\author{
Olayinka Imisioluwa Arimoro \email{olayinka.arimoro@ucalgary.ca},
Lisa M. Lix, Tolulope T. Sajobi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree-threshpar.R
\name{threshpar_grmtree}
\alias{threshpar_grmtree}
\title{Extract Threshold Parameters from GRM Tree}
\usage{
threshpar_grmtree(object, node = NULL, ...)
}
\arguments{
\item{object}{A \code{grmtree} object.}

\item{node}{Optional vector of node IDs to extract from. If NULL (default),
extracts from all terminal nodes.}

\item{...}{Additional arguments (currently unused).}
}
\value{
A data.frame with threshold parameters for each item in each node,
with columns: \item{Node}{Node ID} \item{Item}{Item name} \item{d1, d2,
...}{Threshold parameters for each category}
}
\description{
Extracts threshold parameters for each item from all terminal nodes of a
graded response model tree. The thresholds represent the points on the latent
trait continuum where the probability of scoring in adjacent response
categories is equal.
}
\examples{
\donttest{
  library(grmtree)
  library(hlt)

  data("asti", package = "hlt")
  asti$resp <- data.matrix(asti[, 1:4])

  # Fit GRM tree with gender and group as partitioning variables
  tree <- grmtree(resp ~ gender + group,
          data = asti,
          control = grmtree.control(minbucket = 30))

  # Get all thresholds
  thresholds <- threshpar_grmtree(tree)
  print(thresholds)

}

}
\seealso{
\code{\link{grmtree}} fits a Graded Response Model Tree,
\code{\link{grmforest}} for GRM Forests, \code{\link{fscores_grmtree}} for
computing factor scores, \code{\link{discrpar_grmtree}} for extracting
discrimination parameters, \code{\link{itempar_grmtree}} for extracting item
parameters
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree.R
\name{grmfit}
\alias{grmfit}
\title{Internal Function: Fit Graded Response Model}
\usage{
grmfit(
  y,
  x = NULL,
  start = NULL,
  weights = NULL,
  offset = NULL,
  ...,
  estfun = FALSE,
  object = FALSE
)
}
\arguments{
\item{y}{A matrix of item responses (rows = persons, columns = items)}

\item{x}{Optional predictor matrix (not currently used)}

\item{start}{Optional starting values (passed to mirt)}

\item{weights}{Optional case weights (not currently used)}

\item{offset}{Optional offset (not currently used)}

\item{...}{Additional arguments passed to \code{mirt::mirt}}

\item{estfun}{Logical indicating whether to compute empirical estimating
functions}

\item{object}{Logical indicating whether to return the full mirt model object}
}
\value{
A list containing: \item{coefficients}{Item parameter estimates}
\item{objfun}{Negative log-likelihood value} \item{estfun}{Empirical
estimating functions if requested} \item{object}{Full mirt model object if
requested}
}
\description{
Fits a graded response model (GRM) to item response data. This is an internal
function called by \code{grmtree} and is not intended to be used directly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rs_characterize_plots.R
\name{plot_rs_heatmap}
\alias{plot_rs_heatmap}
\title{Plot Item-Level Response Shift Heatmap}
\usage{
plot_rs_heatmap(
  rs,
  item_labels = NULL,
  node_labels = NULL,
  show_chi2 = FALSE,
  title = NULL,
  ...
)
}
\arguments{
\item{rs}{An \code{rs_characterization} object from
\code{\link{rs_characterize}}.}

\item{item_labels}{Optional character vector of item labels.
If NULL, uses "Item 1", "Item 2", etc.}

\item{node_labels}{Optional character vector of node labels.
If NULL, uses "Node X (n=Y)" from the global results.}

\item{show_chi2}{Logical. If TRUE, display chi-squared values inside
cells. Default is FALSE.}

\item{title}{Optional plot title. If NULL, uses a default title.}

\item{...}{Additional arguments (unused).}
}
\value{
Invisibly returns the rs object. Called for its side effect of
producing a plot.
}
\description{
Creates a standalone heatmap showing RS type for each item across all
terminal nodes. Significant items are filled with RS-type colors;
non-significant items are white/light gray.
}
\details{
The heatmap uses the following color coding:
\describe{
\item{Blue}{Recalibration (threshold change)}
\item{Red/Orange}{Reprioritization (discrimination change)}
\item{Purple}{Both recalibration and reprioritization}
\item{Light orange}{Significant but small effect}
\item{White}{Not significant or not tested}
}

Cells with significant RS (after p-value adjustment) are marked with
an asterisk (*). The omnibus RS result for each node is displayed at
the top of each column.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Phase 1: fit the longitudinal GRM tree
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Phase 2: characterize response shift within each subgroup
  rs <- rs_characterize(ltree, p_adjust = "fdr",
    global_p_adjust = "bonferroni")

  # Basic heatmap
  plot_rs_heatmap(rs)

  # With custom labels
  plot_rs_heatmap(rs,
    item_labels = c("Listen", "Info", "Crisis", "Confide",
      "Advice", "Fears", "Personal", "Understand"))

  # Show chi-squared values
  plot_rs_heatmap(rs, show_chi2 = TRUE)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/longitudinal_grmtree.R
\name{prepare_longitudinal_data}
\alias{prepare_longitudinal_data}
\title{Prepare Wide-Format Longitudinal PROM Data}
\usage{
prepare_longitudinal_data(
  data,
  items_t1,
  items_t2,
  covariates = NULL,
  id = NULL
)
}
\arguments{
\item{data}{A data frame containing item response columns for both time
points and any covariates.}

\item{items_t1}{Character vector of column names for Time 1 item responses,
in the order they should appear in the response matrix.}

\item{items_t2}{Character vector of column names for Time 2 item responses,
in the same order as \code{items_t1}. Must have the same length.}

\item{covariates}{Optional character vector of column names for covariates
to include in the output data frame (e.g., \code{c("age", "sex")}).}

\item{id}{Optional character string specifying the subject ID column name.
If provided, the ID column is included in the output.}
}
\value{
A data frame with:
\describe{
\item{Covariates}{All columns specified in \code{covariates}}
\item{\code{resp_wide}}{A matrix-valued column with
\code{2 * length(items_t1)} columns. The first half contains T1
responses (named \code{Item1_T1}, ..., \code{ItemM_T1}) and the
second half contains T2 responses (named \code{Item1_T2}, ...,
\code{ItemM_T2}).}
}
Rows with any missing values in the response matrix are removed, with
a message indicating how many rows were dropped.
}
\description{
Constructs a wide-format data frame suitable for
\code{\link{longitudinal_grmtree}} from separate Time 1 and Time 2 item
response columns. The function creates a matrix-valued column containing
the concatenated T1 and T2 responses, along with any specified covariates.
}
\examples{
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix from separate T1 and T2 columns
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Check structure
  str(ld$resp_wide)  # 1500 x 16 matrix

}
\seealso{
\code{\link{longitudinal_grmtree}} for fitting the tree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmforest-varimp-final.R
\name{varimp}
\alias{varimp}
\title{Calculate Variable Importance for GRM Forest}
\usage{
varimp(forest, method = "permutation", verbose = FALSE, seed = NULL)
}
\arguments{
\item{forest}{A \code{grmforest} object created by \code{grmforest()}.}

\item{method}{Importance calculation method (currently only "permutation").}

\item{verbose}{Logical indicating whether to show progress messages.}

\item{seed}{Random seed for reproducibility.}
}
\value{
A named numeric vector of importance scores with class \code{varimp}.
Higher values indicate more important variables.
}
\description{
Computes permutation importance scores for variables in a GRM forest using
out-of-bag samples. Importance is measured by the decrease in log-likelihood
when a variable's values are permuted.
}
\examples{
\donttest{
library(grmtree)
library(hlt)
data("asti", package = "hlt")
asti$resp <- data.matrix(asti[, 1:4])

## Fit the GRM Forest
forest <- grmforest(resp ~ gender + group, data = asti,
control = grmforest.control(n_tree = 5))
  importance <- varimp(forest)

## Print and plot the variable importance scores
print(importance)
plot(importance)
}

}
\seealso{
\code{\link{grmtree}} fits a Graded Response Model Tree,
\code{\link{grmforest}} for GRM Forests, \code{\link{grmforest.control}}
creates a control object for \code{grmforest}, \code{\link{plot.varimp}} creates
a bar plot of variable importance scores
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree_long_data.R
\docType{data}
\name{grmtree_long_data}
\alias{grmtree_long_data}
\title{Synthetic Longitudinal MOS-SS Social Support Survey Data}
\format{
A tibble with 1,500 rows and 23 variables:
\describe{
\item{MOS_Listen}{Baseline: someone you can count on to listen when you need to talk (1--5)}
\item{MOS_Info}{Baseline: someone to give you information to help you understand a situation (1--5)}
\item{MOS_Advice_Crisis}{Baseline: someone to give good advice in a crisis (1--5)}
\item{MOS_Confide}{Baseline: someone to confide in or talk to about yourself or your problems (1--5)}
\item{MOS_Advice_Want}{Baseline: someone whose advice you really want (1--5)}
\item{MOS_Fears}{Baseline: someone to share your private worries or fears (1--5)}
\item{MOS_Personal}{Baseline: someone to turn to for suggestions about a personal problem (1--5)}
\item{MOS_Understand}{Baseline: someone who understands your problems (1--5)}
\item{MOS_Listen_year1}{One-year follow-up: listen item (1--5)}
\item{MOS_Info_year1}{One-year follow-up: information item (1--5)}
\item{MOS_Advice_Crisis_year1}{One-year follow-up: crisis advice item (1--5)}
\item{MOS_Confide_year1}{One-year follow-up: confide item (1--5)}
\item{MOS_Advice_Want_year1}{One-year follow-up: advice wanted item (1--5)}
\item{MOS_Fears_year1}{One-year follow-up: fears item (1--5)}
\item{MOS_Personal_year1}{One-year follow-up: personal suggestions item (1--5)}
\item{MOS_Understand_year1}{One-year follow-up: understand item (1--5)}
\item{sex}{Sex (Male, Female)}
\item{age}{Age in years (numeric)}
\item{residency}{Residence location (Urban, Rural)}
\item{job}{Employment status (Employed, Unemployed)}
\item{education}{Education level (Primary/High school, College/University)}
\item{ever_smoker}{Ever-smoker status (Yes, No)}
\item{comorbidity_count}{Number of chronic conditions (0--5)}
\item{bmi}{Body mass index (numeric)}
}
}
\source{
Synthetic data generated from a two-factor longitudinal graded
response model for package testing and demonstration. Contains no real
patient data.
}
\usage{
grmtree_long_data
}
\description{
A synthetic dataset containing responses to the eight MOS-SS emotional
domain items measured at two time points (baseline and one-year follow-up),
together with baseline demographic and clinical covariates. The data are
provided for testing and demonstration of the longitudinal GRMTree
functions (\code{\link{longitudinal_grmtree}}, \code{\link{rs_characterize}})
within the grmtree package.
}
\details{
The data were simulated from a two-factor longitudinal graded response model.
Response shift was intentionally built into the older subgroup (age > 61):
the \code{MOS_Info} item exhibits reprioritization (a discrimination change
over time) and the \code{MOS_Fears} item exhibits recalibration (a threshold
change over time). Younger patients show no response shift. As a result,
\code{\link{longitudinal_grmtree}} produces an age split, and
\code{\link{rs_characterize}} detects response shift within the older
subgroup. Items are coded 1--5, representing None of the time, A little of
the time, Some of the time, Most of the time, and All of the time,
respectively.
}
\examples{
library(grmtree)
library(dplyr)

  # Load the synthetic longitudinal data and take a glimpse
  data("grmtree_long_data", package = "grmtree")
  glimpse(grmtree_long_data)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rs_characterize.R
\name{rs_characterize}
\alias{rs_characterize}
\title{Characterize Response Shift Within Terminal Nodes of a Longitudinal GRM Tree}
\usage{
rs_characterize(
  object,
  node = NULL,
  item_level = TRUE,
  alpha = 0.05,
  p_adjust = "bonferroni",
  global_p_adjust = "bonferroni",
  rs_threshold = 0.3,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{object}{A \code{longitudinal_grmtree} object fitted by
\code{\link{longitudinal_grmtree}}.}

\item{node}{Optional integer vector of terminal node IDs to analyze. If
\code{NULL} (default), all terminal nodes are analyzed.}

\item{item_level}{Logical. If \code{TRUE} (default), perform item-level RS
testing by sequentially freeing one item at a time from the constrained
model. Only performed for nodes where the omnibus test is significant.}

\item{alpha}{Numeric significance level for RS tests. Default is 0.05.
Used for both omnibus and item-level significance decisions (after any
p-value adjustments).}

\item{p_adjust}{Character string specifying the method for adjusting
item-level p-values \strong{within each node}. Controls the error rate
when testing multiple items for RS within a single subgroup. Options:
\code{"bonferroni"} (default), \code{"holm"}, \code{"BH"}
(Benjamini-Hochberg), \code{"BY"}, \code{"fdr"}, \code{"hochberg"},
\code{"hommel"}, or \code{"none"}. See \code{\link[stats]{p.adjust}}.}

\item{global_p_adjust}{Character string specifying the method for adjusting
omnibus RS p-values \strong{across terminal nodes}. Controls the
family-wise error rate when testing RS in multiple subgroups. Options
are the same as \code{p_adjust}. Default is \code{"bonferroni"}.
The \code{RS_detected} column in the global results is updated based
on adjusted p-values. Only applied when there are 2 or more nodes.}

\item{rs_threshold}{Numeric threshold for classifying the magnitude of
parameter change in RS type determination, on the IRT parameter scale.
Default is 0.3. A discrimination difference exceeding this threshold
indicates reprioritization; a maximum threshold difference exceeding
this indicates recalibration. Larger values produce more conservative
classifications (fewer items classified as showing meaningful RS).
Items that are statistically significant but below this threshold are
classified as \code{"Significant (small effect)"}.}

\item{verbose}{Logical. If \code{TRUE} (default), print progress messages
including log-likelihoods, LRT statistics, and RS classifications for
each node during processing.}

\item{...}{Additional arguments passed to \code{\link[mirt]{mirt}} during
model fitting.}
}
\value{
A list of class \code{"rs_characterization"} containing:
\describe{
\item{\code{global}}{A \code{data.frame} with one row per terminal node
and columns:
\describe{
\item{\code{Node}}{Terminal node ID}
\item{\code{n}}{Sample size in the node}
\item{\code{LL_constrained}}{Log-likelihood of the constrained
(no-RS) model}
\item{\code{LL_unconstrained}}{Log-likelihood of the unconstrained
(RS-allowed) model}
\item{\code{LRT_chi2}}{Likelihood ratio test statistic:
\eqn{-2(LL_{con} - LL_{uncon})}}
\item{\code{LRT_df}}{Degrees of freedom (number of freed
constraints)}
\item{\code{LRT_p}}{Raw p-value from chi-squared distribution}
\item{\code{LRT_p_adj}}{Adjusted p-value (present only if
\code{global_p_adjust != "none"} and there are 2+ nodes)}
\item{\code{RS_detected}}{Logical: whether RS is detected at
the specified \code{alpha} level (uses adjusted p if available)}
\item{\code{pseudo_R2}}{McFadden's pseudo R-squared:
\eqn{1 - LL_{uncon}/LL_{con}}}
\item{\code{AIC_diff}}{AIC difference:
\eqn{AIC_{con} - AIC_{uncon}}. Positive values indicate the
unconstrained model fits better after parsimony penalty.}
\item{\code{mu_T2}}{Latent mean shift at T2 from the constrained
model. Positive = improvement, negative = decline.}
\item{\code{sigma2_T2}}{Latent variance at T2}
\item{\code{cor_T1_T2}}{Correlation between latent traits at T1
and T2, reflecting stability of individual differences}
\item{\code{converged_constrained}}{Logical: whether the
constrained model converged}
\item{\code{converged_unconstrained}}{Logical: whether the
unconstrained model converged}
}
}
\item{\code{item_level}}{A \code{data.frame} with item-level RS test
results (one row per item per node where omnibus RS was detected),
with columns:
\describe{
\item{\code{Node}}{Terminal node ID}
\item{\code{Item}}{Item number (1 to \code{n_items})}
\item{\code{LRT_chi2}}{LRT statistic for freeing this item}
\item{\code{LRT_df}}{Degrees of freedom (1 discrimination +
thresholds for this item)}
\item{\code{LRT_p}}{Raw p-value}
\item{\code{LRT_p_adj}}{Adjusted p-value (if \code{p_adjust !=
          "none"})}
\item{\code{RS_type}}{Classification: \code{"Recalibration"},
\code{"Reprioritization"}, \code{"Both"},
\code{"Significant (small effect)"}, or \code{"None"}}
}
\code{NULL} if no nodes showed significant omnibus RS or
\code{item_level = FALSE}.
}
\item{\code{parameters}}{A named list (one element per node) of lists
containing \code{constrained} and \code{unconstrained} item parameter
matrices (IRT parameterization with discrimination \code{a1}/\code{a2}
and thresholds \code{b1}, ..., \code{bK}).}
\item{\code{n_items}}{Number of items per time point.}
\item{\code{alpha}}{Significance level used.}
\item{\code{p_adjust}}{Item-level p-value adjustment method.}
\item{\code{global_p_adjust}}{Omnibus p-value adjustment method.}
\item{\code{rs_threshold}}{RS classification threshold.}
}
}
\description{
After fitting a \code{\link{longitudinal_grmtree}} (Phase 1), this function
performs Phase 2: post-hoc response shift (RS) characterization within each
terminal node. For each subgroup identified by the tree, it compares the
constrained model (no RS: item parameters equal across time) to an
unconstrained model (item parameters free across time) using likelihood
ratio tests (LRTs), then identifies which specific items exhibit response
shift and classifies the type of shift.
}
\details{
\subsection{The Two-Phase Framework}{

The Longitudinal GRMTree operates as an integrated two-phase method:

\strong{Phase 1 (\code{\link{longitudinal_grmtree}}):} Embeds a constrained
longitudinal GRM (item parameters equal across T1 and T2) within
model-based recursive partitioning. The tree identifies patient subgroups
requiring different constrained measurement models. This ensures that
subsequent RS testing is conducted within measurement-homogeneous groups.

\strong{Phase 2 (this function):} Within each terminal node, relaxes the
equality constraint and tests whether item parameters actually change over
time. This separates between-group measurement heterogeneity (captured by
the tree) from within-group temporal change (response shift).
}

\subsection{Omnibus RS Test}{

For each terminal node, the function fits two models to the node's data:
\enumerate{
\item \strong{Constrained model:} Item parameters are equal across T1
and T2 (no response shift). This model has the same structure as the
model fitted during Phase 1.
\item \strong{Unconstrained model:} Item parameters are free to differ
between T1 and T2 (response shift allowed). All equality constraints
on discrimination and threshold parameters are removed.
}

The likelihood ratio test statistic is:
\deqn{\chi^2 = -2(LL_{constrained} - LL_{unconstrained})}

with degrees of freedom equal to the number of freed constraints
(typically \eqn{M \times K} for \eqn{M} items with \eqn{K-1} thresholds
each, plus \eqn{M} discrimination parameters).
}

\subsection{Item-Level RS Testing}{

When the omnibus test is significant, each item is tested individually by
freeing only that item's constraints while keeping all other items
constrained. The LRT compares the fully constrained model to the partially
constrained model with one item freed. This identifies which specific
items contribute to the overall response shift.
}

\subsection{RS Type Classification}{

For items showing statistically significant RS (after p-value adjustment),
the type is classified by examining the freed item's T1 vs T2 parameters
from the partially constrained model:

\describe{
\item{\strong{Recalibration}}{Maximum absolute threshold difference
exceeds \code{rs_threshold}. Interpretation: the patient's internal
standards for endorsing response categories changed over time
(Sprangers & Schwartz, 1999).}
\item{\strong{Reprioritization}}{Absolute discrimination difference
exceeds \code{rs_threshold}. Interpretation: the item's relationship
to the underlying latent construct changed over time --- the item
became more or less central to the construct.}
\item{\strong{Both}}{Both discrimination and threshold changes exceed
\code{rs_threshold}.}
\item{\strong{Significant (small effect)}}{The LRT is significant but
neither parameter change exceeds \code{rs_threshold}. The RS is
statistically detectable but small in magnitude.}
}
}

\subsection{Hierarchical P-Value Correction}{

The function implements a two-level correction strategy:
\enumerate{
\item \strong{Across nodes (omnibus):} \code{global_p_adjust} controls
the family-wise error rate for the question "in which subgroups does
RS occur?" Default is Bonferroni.
\item \strong{Within nodes (item-level):} \code{p_adjust} controls the
error rate for the question "which items exhibit RS within a given
subgroup?" Default is Bonferroni; FDR (Benjamini-Hochberg) is
recommended for exploratory analyses.
}
}

\subsection{Effect Size Measures}{

\describe{
\item{McFadden's pseudo R-squared}{\eqn{1 - LL_{uncon}/LL_{con}}.
Inherently small (~0.02) in IRT models because item parameters
account for only a fraction of the total likelihood. Values should
be compared across nodes rather than interpreted in absolute terms.}
\item{AIC difference}{\eqn{AIC_{con} - AIC_{uncon}}. Positive values
indicate the unconstrained model fits better even after penalizing
for additional parameters. Negative values suggest the constrained
model is preferred (response shift is not substantial enough to
justify the additional parameters).}
}
}
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # After fitting the tree (Phase 1)
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Phase 2: characterize response shift within each subgroup
  rs <- rs_characterize(ltree, p_adjust = "fdr",
    global_p_adjust = "bonferroni")
  print(rs)

  # Phase 2: Characterize RS
  # Default: Bonferroni at both levels, threshold = 0.3
  rs <- rs_characterize(ltree)

  # Recommended: FDR for item-level, Bonferroni for omnibus
  rs <- rs_characterize(ltree,
    p_adjust = "fdr",
    global_p_adjust = "bonferroni",
    rs_threshold = 0.3
  )

  # No omnibus correction (each node tested independently)
  rs <- rs_characterize(ltree, global_p_adjust = "none")

  # More conservative RS classification
  rs <- rs_characterize(ltree, rs_threshold = 0.5)

  # View results
  print(rs)

  # Access components
  rs$global         # Omnibus results per node
  rs$item_level     # Item-level results
  rs$parameters     # Raw parameter matrices
\dontshow{\}) # examplesIf}
}
\references{
Sprangers, M. A., & Schwartz, C. E. (1999). Integrating response shift
into health-related quality of life research. \emph{Social Science &
Medicine}, 48(11), 1507--1515.

Benjamini, Y., & Hochberg, Y. (1995). Controlling the false discovery
rate: a practical and powerful approach to multiple testing. \emph{Journal
of the Royal Statistical Society: Series B}, 57(1), 289--300.
}
\seealso{
\code{\link{longitudinal_grmtree}} for Phase 1 (tree fitting),
\code{\link{print.rs_characterization}} for the print method,
\code{\link{prepare_longitudinal_data}} for data preparation,
\code{\link[stats]{p.adjust}} for p-value adjustment methods
}
\author{
Olayinka Imisioluwa Arimoro \email{olayinka.arimoro@ucalgary.ca},
Lisa M. Lix, Tolulope T. Sajobi
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/longitudinal_grmtree.R
\name{longitudinal_grmfit}
\alias{longitudinal_grmfit}
\title{Internal Function: Fit Constrained Longitudinal Graded Response Model}
\usage{
longitudinal_grmfit(
  y,
  x = NULL,
  start = NULL,
  weights = NULL,
  offset = NULL,
  ...,
  estfun = FALSE,
  object = FALSE
)
}
\arguments{
\item{y}{A numeric matrix of item responses in wide format. The first
\code{n_items} columns contain responses at Time 1 (T1) and the next
\code{n_items} columns contain responses at Time 2 (T2), for a total of
\code{2 * n_items} columns. Each row represents one individual. The number
of columns must be even and at least 4.}

\item{x}{Optional predictor matrix. Handled internally by the MOB framework
and not used directly in model fitting.}

\item{start}{Optional starting values for model parameters. Passed to
\code{\link[mirt]{mirt}}.}

\item{weights}{Optional case weights. Not currently used.}

\item{offset}{Optional offset. Not currently used.}

\item{...}{Additional arguments passed to \code{\link[mirt]{mirt}}.}

\item{estfun}{Logical indicating whether to compute empirical estimating
(score) functions for the MOB parameter instability tests. When
\code{TRUE}, standard errors are computed during model fitting (SE = TRUE)
and \code{\link[mirt]{estfun.AllModelClass}} is called to extract the score
contributions for each individual. Default is \code{FALSE}.}

\item{object}{Logical indicating whether to return the full
\code{\link[mirt]{mirt}} model object. Default is \code{FALSE}. Set to
\code{TRUE} when the fitted model is needed for post-hoc analyses (e.g., in
\code{\link{rs_characterize}}).}
}
\value{
A list containing:
\item{coefficients}{Item parameter estimates from
\code{mirt::coef(fit, IRTpars = TRUE, simplify = TRUE)}, including
discrimination (\code{a1}, \code{a2}) and threshold (\code{b1}, ...,
\code{bK}) parameters for all \code{2 * n_items} items, plus latent
trait means and covariance matrix.}
\item{objfun}{Negative log-likelihood of the fitted model. Used by MOB
to evaluate splits via partitioned log-likelihood maximization.}
\item{estfun}{Empirical estimating (score) functions if \code{estfun =
    TRUE}; \code{NULL} otherwise. An \code{N x p} matrix where \code{N}
is the number of individuals and \code{p} is the number of model
parameters. Used by MOB for score-based structural change tests.}
\item{object}{The full \code{SingleGroupClass} model object
if \code{object = TRUE}; \code{NULL} otherwise.}
}
\description{
Fits a constrained two-factor longitudinal graded response model (GRM) to
wide-format item response data from two time points. This is an internal
function called by \code{\link{longitudinal_grmtree}} during model-based
recursive partitioning and is not intended to be used directly.
}
\details{
This production version builds the equality constraints directly from the
known parameter structure rather than fitting an unconstrained model first to
discover that structure. Because the function is called hundreds of times
during tree construction, avoiding the extra (unconstrained) model fit at
every node yields a substantial speed-up with no change in results.
\subsection{Constrained Two-Factor Longitudinal GRM}{

The function specifies a two-factor GRM where the latent factors
\eqn{\theta_{T1}} and \eqn{\theta_{T2}} represent the construct at Time 1 and
Time 2, respectively. The model structure is:

\deqn{\theta_{T1} \sim N(0, 1)} \deqn{\theta_{T2} \sim N(\mu_{T2},
\sigma^2_{T2})} \deqn{Cov(\theta_{T1}, \theta_{T2}) = \sigma_{12}}

Items 1 through \eqn{M} load on \eqn{\theta_{T1}} and items \eqn{M+1} through
\eqn{2M} load on \eqn{\theta_{T2}}, where \eqn{M} is the number of items per
time point.

Under the null hypothesis of no response shift, item parameters are
constrained equal across time points. For each item \eqn{m}:
\itemize{
\item Discrimination: \eqn{a_{1,m} = a_{2,m+M}} (the T1 item's loading
on \eqn{\theta_{T1}} equals the T2 item's loading on \eqn{\theta_{T2}})
\item Thresholds: \eqn{b_{k,m} = b_{k,m+M}} for all category thresholds
\eqn{k = 1, \ldots, K_m}
}

The number of threshold constraints per item pair is determined by the
minimum number of observed response categories across T1 and T2 for that item
(minus 1). This handles cases where sparse response categories at one time
point produce fewer estimated thresholds (e.g., category 5 may be empty at T1
but observed at T2).
}

\subsection{Constraint Syntax}{

Constraints are specified using \code{mirt}'s CONSTRAIN syntax:
\itemize{
\item Discrimination: \code{(i, a1, j, a2)} constrains item \code{i}'s
\code{a1} parameter to equal item \code{j}'s \code{a2} parameter
\item Thresholds: \code{(i, j, dk)} constrains the \code{dk} threshold
to be equal between items \code{i} and \code{j}
}

where \code{i} is the T1 item index and \code{j = i + n_items} is the
corresponding T2 item index.
}

\subsection{Technical Details}{

The EM algorithm is run with a maximum of 1000 cycles (\code{technical =
list(NCYCLES = 1000)}) to accommodate the complexity of the constrained
two-factor model. Verbose output from \code{mirt} is suppressed. If the model
fails to converge or encounters an error, the function stops with an
informative error message.
}
}
\seealso{
\code{\link{grmtree}} fits a Graded Response Model Tree,
\code{\link{longitudinal_grmtree}} for the main tree-fitting function that
calls this internally, \code{\link[mirt]{mirt}} for the underlying IRT
estimation engine, \code{\link{rs_characterize}} for post-hoc response
shift testing within terminal nodes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmforest-varimp-plot.R
\name{plot.varimp}
\alias{plot.varimp}
\title{Plot Variable Importance}
\usage{
\method{plot}{varimp}(x, top_n = NULL, use_ggplot = TRUE, ...)
}
\arguments{
\item{x}{A \code{varimp} object from \code{varimp()}.}

\item{top_n}{Number of top variables to display (NULL for all).}

\item{use_ggplot}{Logical indicating whether to use ggplot2 (if available).}

\item{...}{Additional arguments passed to plotting functions.}
}
\value{
Invisibly returns the input object.
}
\description{
Creates a bar plot of variable importance scores with options for both
ggplot2 and base R graphics.
}
\examples{
\donttest{
 library(grmtree)
 library(hlt)
 data("asti", package = "hlt")
 asti$resp <- data.matrix(asti[, 1:4])

 # Fit forest with default parameters
 forest <- grmforest(resp ~ gender + group, data = asti)
 imp <- varimp(forest)
 plot(imp)
 plot(imp, top_n = 1) ## select top 1 importance variable
 plot(imp, use_ggplot = FALSE) # Use base R graphics
}

}
\seealso{
\code{\link{varimp}} calculates the variable importance for GRM
Forest, \code{\link{grmforest}} for GRM Forests,
\code{\link{grmforest.control}} creates a control object for \code{grmforest},
\code{\link{plot.grmtree}} creates plot for the \code{grmtree} object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long-grmtree-itempar.R
\name{itempar_longitudinal_grmtree}
\alias{itempar_longitudinal_grmtree}
\title{Extract Item Parameters from Longitudinal GRM Tree}
\usage{
itempar_longitudinal_grmtree(object, node = NULL, clean_names = TRUE, ...)
}
\arguments{
\item{object}{A \code{longitudinal_grmtree} object.}

\item{node}{Optional vector of node IDs. If NULL, all terminal nodes.}

\item{clean_names}{Logical. If TRUE (default), clean item names.}

\item{...}{Additional arguments (currently unused).}
}
\value{
A data.frame with columns:
\describe{
\item{Node}{Terminal node ID}
\item{Item}{Item name}
\item{Discrimination}{Discrimination parameter}
\item{AvgThreshold}{Mean of all threshold parameters for the item}
\item{b1, b2, ..., bK}{Individual threshold parameters}
}
}
\description{
Extracts both discrimination and threshold parameters for each unique item
from all terminal nodes, combining them into a single data frame with an
average threshold column. Only T1 items are returned.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Phase 1: fit the longitudinal GRM tree
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Print the item parameters
  items <- itempar_longitudinal_grmtree(ltree)
  print(items)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{longitudinal_grmtree}} for Phase 1 (tree fitting),
\code{\link{discrpar_longitudinal_grmtree}} for extracting discrimination
parameters for longitudinal GRMTree,
\code{\link{threshpar_longitudinal_grmtree}} for extracting threshold
parameters for longitudinal GRMTree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rs_characterize_plots.R
\name{plot_rs_tree}
\alias{plot_rs_tree}
\title{Plot Response Shift Summary Tree}
\usage{
plot_rs_tree(
  tree,
  rs,
  item_labels = NULL,
  tnex = 2.5,
  drop_terminal = TRUE,
  ...
)
}
\arguments{
\item{tree}{A \code{longitudinal_grmtree} object.}

\item{rs}{An \code{rs_characterization} object from
\code{\link{rs_characterize}}.}

\item{item_labels}{Optional character vector of short item labels.
If NULL, uses "Item 1", "Item 2", etc.}

\item{tnex}{Numeric scaling factor for terminal node panels (default: 2.5).}

\item{drop_terminal}{Logical (default: TRUE).}

\item{...}{Additional arguments passed to \code{plot.modelparty}.}
}
\value{
Invisibly returns the tree object.
}
\description{
Displays the Longitudinal GRMTree structure with RS characterization
results annotated in each terminal node panel. Each panel shows the
omnibus test result, latent trait parameters, and an item-level RS
heatmap color-coded by RS type.
}
\details{
Each terminal node panel contains:
\itemize{
\item Omnibus RS result: chi-squared, adjusted p-value, and detection
status
\item Latent parameters: mu_T2 and cor(T1,T2)
\item Item-level RS bar: colored cells for each item indicating RS type
(blue = recalibration, red = reprioritization, purple = both,
orange = significant small effect, gray = none/not tested)
}
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Phase 1: fit the longitudinal GRM tree
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Phase 2: characterize response shift within each subgroup
  rs <- rs_characterize(ltree, p_adjust = "fdr",
    global_p_adjust = "bonferroni")

  plot_rs_tree(ltree, rs,
    item_labels = c("Listen", "Info", "Crisis", "Confide",
      "Advice", "Fears", "Personal", "Understand"))
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long-grmtree-latentpar.R
\name{latentpar_longitudinal_grmtree}
\alias{latentpar_longitudinal_grmtree}
\title{Extract Latent Trait Parameters from Longitudinal GRM Tree}
\usage{
latentpar_longitudinal_grmtree(object, node = NULL, ...)
}
\arguments{
\item{object}{A \code{longitudinal_grmtree} object.}

\item{node}{Optional vector of node IDs. If NULL, all terminal nodes.}

\item{...}{Additional arguments (currently unused).}
}
\value{
A data.frame with columns:
\describe{
\item{Node}{Terminal node ID}
\item{n}{Sample size in the node}
\item{mu_T2}{Latent mean at T2 (positive = improvement)}
\item{sigma2_T2}{Latent variance at T2}
\item{cor_T1_T2}{Correlation between T1 and T2 latent traits}
}
}
\description{
Extracts the latent trait distribution parameters from each terminal node of
a longitudinal GRM tree: the T2 latent mean shift (\eqn{\mu_{T2}}), the T2
latent variance (\eqn{\sigma^2_{T2}}), and the T1-T2 correlation.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Phase 1: fit the longitudinal GRM tree
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Print latent trait summary
  latent <- latentpar_longitudinal_grmtree(ltree)
  print(latent)
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{longitudinal_grmtree}} for Phase 1 (tree fitting),
\code{\link{fscores_longitudinal_grmtree}}, for computing factor scores for
longitudinal GRMTree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/long-grmtree-plot.R
\name{plot.longitudinal_grmtree}
\alias{plot.longitudinal_grmtree}
\title{Plot Method for Longitudinal GRM Tree Objects}
\usage{
\method{plot}{longitudinal_grmtree}(
  x,
  type = "regions",
  tnex = 2L,
  drop_terminal = TRUE,
  names = FALSE,
  abbreviate = TRUE,
  ...
)
}
\arguments{
\item{x}{A \code{longitudinal_grmtree} object.}

\item{type}{Type of terminal node plot. Currently only \code{"regions"}
is supported for the longitudinal model.}

\item{tnex}{Numeric scaling factor for terminal node extension (default: 2).}

\item{drop_terminal}{Logical indicating whether to drop terminal node IDs
(default: TRUE).}

\item{names}{Logical or character vector. If \code{TRUE}, use item names
from the response matrix. If a character vector, use as custom labels.
If \code{FALSE} (default), use numeric indices 1 through n_items.}

\item{abbreviate}{Logical or numeric. If \code{TRUE}, abbreviate item
names. If numeric, abbreviate to that many characters.}

\item{...}{Additional arguments passed to the terminal panel function.}
}
\value{
Invisibly returns the tree object. Called for its side effect of
producing a plot.
}
\description{
Visualizes a longitudinal GRM tree with threshold region plots in terminal
nodes. Unlike the cross-sectional \code{\link[grmtree]{plot.grmtree}},
this method displays only the unique item parameters (T1 items), since
the constrained longitudinal model enforces equal parameters across T1
and T2 within each node.
}
\details{
The region plot displays threshold parameters as colored horizontal bands
for each item within each terminal node. Darker shading represents lower
response categories and lighter shading represents higher categories.
The height of each band corresponds to the range of the latent trait
over which that response category is most likely.

Because the constrained longitudinal GRM enforces
\eqn{a_{m,T1} = a_{m,T2}} and \eqn{b_{k,m,T1} = b_{k,m,T2}}, the T1
and T2 item parameters are identical. The plot therefore shows only the
\code{n_items} unique items rather than all \code{2 * n_items} columns
in the response matrix.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
library(grmtree)

# Load the synthetic longitudinal data
data("grmtree_long_data", package = "grmtree")

  # Prepare the wide-format response matrix
  items_t1 <- c("MOS_Listen", "MOS_Info", "MOS_Advice_Crisis", "MOS_Confide",
                "MOS_Advice_Want", "MOS_Fears", "MOS_Personal", "MOS_Understand")
  ld <- prepare_longitudinal_data(
    data = grmtree_long_data,
    items_t1 = items_t1,
    items_t2 = paste0(items_t1, "_year1"),
    covariates = c("sex", "age", "residency", "job",
                   "education", "comorbidity_count", "ever_smoker")
  )

  # Phase 1: fit the longitudinal GRM tree
  ltree <- longitudinal_grmtree(
    resp_wide ~ sex + age + residency + job +
      education + comorbidity_count + ever_smoker,
    data = ld, n_items = 8,
    control = grmtree.control(minbucket = 200)
  )

  # Region plot with numeric labels
  plot(ltree)

  # Region plot with item names
  plot(ltree, names = TRUE)

  # Custom labels
  plot(ltree, names = c("Listen", "Info", "Crisis",
    "Confide", "Advice", "Fears", "Personal", "Understand"))
\dontshow{\}) # examplesIf}
}
\seealso{
\code{\link{longitudinal_grmtree}} for fitting the tree,
\code{\link[grmtree]{plot.grmtree}} for cross-sectional tree plots
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grmtree-plot.R
\name{plot.grmtree}
\alias{plot.grmtree}
\title{Plot Method for GRM Tree Objects}
\usage{
\method{plot}{grmtree}(
  x,
  type = c("regions", "profile", "histogram"),
  what = c("item", "threshold", "discrimination"),
  tnex = 2L,
  drop_terminal = TRUE,
  spacing = 0.1,
  ...
)
}
\arguments{
\item{x}{A GRM tree object of class 'grmtree'.}

\item{type}{Type of terminal node plot to display:
\describe{
\item{"regions"}{Threshold regions plot (default)}
\item{"profile"}{Item parameter profile plot}
\item{"histogram"}{Histogram of factor scores with normal curve}
}}

\item{what}{Type of parameters to plot when \code{type = "profile"}:
\describe{
\item{"item"}{Both discrimination and threshold parameters (default)}
\item{"threshold"}{Only threshold parameters}
\item{"discrimination"}{Only discrimination parameters}
}}

\item{tnex}{Numeric scaling factor for terminal node extension (default: 2).}

\item{drop_terminal}{Logical indicating whether to drop terminal node IDs
(default: TRUE).}

\item{spacing}{Numeric value controlling spacing between elements (default:
0.1).}

\item{...}{Additional arguments passed to the terminal panel functions.}
}
\value{
Invisibly returns the GRM tree object. Primarily called for its side
effect of producing a plot.
}
\description{
Visualizes a GRM (Graded Response Model) tree with different types of
terminal node plots. This function extends \code{plot.modelparty} from the
partykit package with specialized visualizations for GRM trees.
}
\details{
The function provides three visualization types:
\itemize{
\item \strong{Regions plot}: Shows threshold parameters as colored regions, useful
for visualizing the difficulty parameters across items and nodes.
\item \strong{Profile plot}: Displays either item parameters (discrimination and
average thresholds), just thresholds, or just discrimination parameters as
line plots across items.
\item \strong{Histogram}: Shows the distribution of factor scores in each node
with an overlaid normal curve.
}
}
\examples{
\donttest{

library(grmtree)
library(hlt)
data("asti", package = "hlt")
asti$resp <- data.matrix(asti[, 1:4])

# Fit GRM tree with gender and group as partitioning variables
tree <- grmtree(resp ~ gender + group,
          data = asti,
          control = grmtree.control(minbucket = 30))

# Default regions plot
plot(tree)

# Profile plot showing item parameters
plot(tree, type = "profile")

# Profile plot showing only thresholds
plot(tree, type = "profile", what = "threshold")

# Histograms of factor scores
plot(tree, type = "histogram")
}

}
\seealso{
\code{\link[partykit]{plot.modelparty}} for the underlying plotting
infrastructure, \code{\link{grmtree}} for creating GRM tree objects,
\code{\link{plot.varimp}} creates a bar plot of variable importance scores
}
